/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.List;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.element.Copy;
import org.ow2.orchestra.facade.data.def.ActivityDefinitionData;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * @author Charles Souillard
 */
public class Assign extends AutomaticActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  /** list of all copy elements of this assign. */
  protected List<Copy> copies;

  /** validate attribute of the assign element. */
  protected boolean validate = false;

  public Assign() {
    super();
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  @Override
  public void executeBusinessLogic(final BpelExecution execution) {
    for (Copy copy : copies) {
      copy.execute(execution);
    }
  }

  public List<Copy> getCopies() {
    return this.copies;
  }

  public void setCopies(final List<Copy> copies) {
    this.copies = copies;
  }

  public boolean isValidate() {
    return this.validate;
  }

  public void setValidate(final boolean validate) {
    this.validate = validate;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.ASSIGN;
  }

  @Override
  public ActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new ActivityDefinitionData(bpelProcess.getProcessDefinitionData(), uuid, null,
      getType(), name);
  }

}
