/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.definition.activity;

import javax.xml.namespace.QName;

import org.ow2.orchestra.definition.element.Variable;
import org.ow2.orchestra.exception.FaultWithMessageVariable;
import org.ow2.orchestra.facade.def.full.impl.CatchFullDefinitionImpl;
import org.ow2.orchestra.facade.exception.FaultWithElement;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.util.Misc;

/**
 * @author S. Ali Tokmen
 */
public class CatchActivity extends BaseCatchActivity {

  private static final long serialVersionUID = -245494906565371303L;

  protected QName faultQName;

  protected Variable catchVariableDefinition;

  // Keep Hibernate happy
  protected CatchActivity() {
    super(null, null, null);
  }

  public CatchActivity(final NodeImpl child, final ActivityDefinitionUUID uuid, final String name,
    final CatchFullDefinitionImpl catchDefinition) {
    super(child, uuid, name);
    this.faultQName = catchDefinition.getFaultName();

    if (catchDefinition.getFaultVariable() != null) {
      this.catchVariableDefinition = new Variable();
      this.catchVariableDefinition.setName(catchDefinition.getFaultVariable());

      if (catchDefinition.getFaultMessageType() != null) {
        this.catchVariableDefinition.setMessageType(catchDefinition.getFaultMessageType());
      } else if (catchDefinition.getFaultElement() != null) {
        this.catchVariableDefinition.setElement(catchDefinition.getFaultElement());
      } else {
        Misc.unreachableStatement("faultVariable is non null but both faultMessageType and faultElement are null");
      }
    }
  }

  @Override
  public ActivityType getType() {
    return ActivityType.CATCH_HANDLER;
  }


  /**
   * @return the faultQName
   */
  public QName getFaultName() {
    return this.faultQName;
  }

  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {
    super.beforeRunning(bpelExecution);

    if (this.catchVariableDefinition != null) {
      final ScopeRuntime scopeRuntime = bpelExecution.getCurrentScopeRuntime();
      if (scopeRuntime.containsVariable(this.catchVariableDefinition.getName())) {
        Misc.unreachableStatement("The scope runtime " + scopeRuntime + " already contains a variable called "
          + this.catchVariableDefinition.getName());
      }

      final VariableRuntime vr = scopeRuntime.createVariable(bpelExecution, this.catchVariableDefinition);

      if (this.catchVariableDefinition.getMessageType() != null) {
        final FaultWithMessageVariable fault = (FaultWithMessageVariable) scopeRuntime.getLastException();
        vr.setValue(fault.getFaultMessage().duplicate());
      } else if (this.catchVariableDefinition.getElement() != null) {
        final FaultWithElement fault = (FaultWithElement) scopeRuntime.getLastException();
        vr.setValue(fault.getFaultElement(), true);
      } else {
        Misc.unreachableStatement("catchVariableDefinition's messageType and element attributes are both null");
      }
      // record variable updated
      vr.updated(bpelExecution.getActivityInstanceUUID());
    }
  }

  @Override
  public void signal(final BpelExecution execution) {
    super.signal(execution);

    if (this.catchVariableDefinition != null) {
      final ScopeRuntime scopeRuntime = execution.getCurrentScopeRuntime();
      scopeRuntime.removeVariable(execution, this.catchVariableDefinition);
    }
  }

  /**
   * @return
   */
  public QName getFaultMessageType() {
    if (this.catchVariableDefinition == null) {
      return null;
    }
    return this.catchVariableDefinition.getMessageType();
  }

  /**
   * @return
   */
  public QName getFaultElement() {
    if (this.catchVariableDefinition == null) {
      return null;
    }
    return this.catchVariableDefinition.getElement();
  }

  /**
   * @return
   */
  public String getFaultVariable() {
    if (this.catchVariableDefinition == null) {
      return null;
    }
    return this.catchVariableDefinition.getName();
  }

}
