/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.definition.activity;

import java.util.logging.Logger;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;

/**
 * @author S. Ali Tokmen
 */
public class CompensationHandlerActivity extends ActivityWithSingleChild {

  private static final long serialVersionUID = -1532327461542960780L;

  private static Logger log = Logger.getLogger(CompensationHandlerActivity.class.getName());

  protected CompensationHandlerActivity() {
    // Keep Hibernate happy
    super(null, null, null);
  }

  public CompensationHandlerActivity(final NodeImpl child, final ActivityDefinitionUUID uuid, final String name) {
    super(child, uuid, name);
  }

  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {
    super.beforeRunning(bpelExecution);

    final ScopeRuntime scopeRuntime = bpelExecution.getCurrentScopeRuntime();
    scopeRuntime.setState(ScopeState.COMPENSATING);
  }

  @Override
  public void signal(final BpelExecution execution) {
    // this is child execution returning
    // end execution
    execution.end(Execution.STATE_ENDED);

    final BpelExecution chExecution = execution.getParent();
    // remove finished execution from parent
    chExecution.removeExecution(execution);


    // end ch execution
    chExecution.end(Execution.STATE_ENDED);

    final BpelExecution parentExecution = chExecution.getParent();
    // remove finished execution from parent
    parentExecution.removeExecution(chExecution);

    // Signal calling compensate activity
    final ScopeRuntime scopeRuntime = chExecution.popScopeRuntime(false);
    scopeRuntime.setState(ScopeState.COMPENSATED);

    final BpelExecution compensateExecution = scopeRuntime.getCompensateExecution();
    scopeRuntime.setCompensateExecution(null);
    compensateExecution.signal();
  }

  @Override
  public ActivityType getType() {
    return ActivityType.COMPENSATION_HANDLER;
  }

}
