/**
 * Copyright (C) 2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Date;

import org.ow2.orchestra.definition.element.OnEvent;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.IdFactory;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.session.MessageSession;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.MessageCarrier;
import org.ow2.orchestra.services.job.ExecuteScopeMessage;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.ReceivingElementUtil;
import org.ow2.orchestra.var.MessageVariable;


/**
 * @author Guillaume Porcher
 *
 */
public class EventHandlerScope extends Scope {

  public static final String ON_EVENT = "onEvent";
  public static final String MESSAGE = "message";
  public static final String MESSAGE_CARRIER = "messageCarrier";

  @Override
  public void beforeRunning(final BpelExecution bpelExecution) {
    final ActivityInstanceUUID activityInstanceUUID = IdFactory.getNewActivityUUID();
    bpelExecution.setStartedDate(new Date());
    bpelExecution.setActivityInstanceUUID(activityInstanceUUID);

    EnvTool.getRecorder().recordEventStarted(this.createRuntimeData(bpelExecution));

  }

  @Override
  protected void executeEventHandlers(final BpelExecution execution) {
    // Execute onEvent before launching the associated scope event handlers
    final OnEvent onEvent = (OnEvent) execution.getVariable(EventHandlerScope.ON_EVENT);
    if (onEvent != null) {
      final MessageCarrier messageCarrier = (MessageCarrier) execution.getVariable(EventHandlerScope.MESSAGE_CARRIER);
      final MessageVariable message = (MessageVariable) execution.getVariable(EventHandlerScope.MESSAGE);
      ReceivingElementUtil.receive(onEvent, message, messageCarrier, execution);
    }
    super.executeEventHandlers(execution);
  }

  @Override
  public void executeMainActivity(final BpelExecution execution) {
    // Add additional variables (used in ForEach)
    final OnEvent onEvent = (OnEvent) execution.getVariable(EventHandlerScope.ON_EVENT);
    if (onEvent != null) {
      execution.removeVariable(EventHandlerScope.MESSAGE_CARRIER);
      execution.removeVariable(EventHandlerScope.MESSAGE);
      execution.removeVariable(EventHandlerScope.ON_EVENT);
      Environment.getFromCurrent(MessageSession.class).send(new ExecuteScopeMessage(execution));
      execution.waitForSignal();
      execution.setState(Execution.STATE_ASYNC);
    } else {
      super.executeMainActivity(execution);
    }
  }
}
