/**
 * Copyright (C) 2007-2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Set;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.data.def.ExitActivityDefinitionData;
import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.handlers.FinishedInstanceHandler;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.Misc;

/**
 * the runtime behaviour of the exit activity.
 *
 * @author Tom Baeyens, Charles Souillard, S. Ali Tokmen
 */
public class Exit extends AbstractBpelActivity {

  /**
   * Auto-generated serial version UID.
   */
  private static final long serialVersionUID = -3418645121959511051L;

  public Exit() {
    super();
  }

  /**
   * Terminates the root process.
   *
   * @param execution Current execution.
   */
  @Override
  public void executeActivity(final BpelExecution execution) {
    // Get process
    final BpelExecution processInstance = execution.getProcessInstance();
    if (processInstance.getParent() != null) {
      Misc.unreachableStatement("Process instance has a parent!");
    }

    // Stop activity
    execution.waitForSignal();

    // Exit process instance
    processInstance.exit();

    final ProcessInstanceData pid = processInstance.getProcessInstanceData();

    final Set<BpelExecution> waitingExecutions = execution.getProcessInstance().getWaitingExecutions();
    if (waitingExecutions != null && waitingExecutions.size() > 0) {
      Misc.unreachableStatement("There are " + waitingExecutions.size() + " executions waiting after Exit!");
    }

    // Mark process as finished
    //this.afterRunned(execution);
    EnvTool.getRecorder().recordInstanceEnded(pid);
    final FinishedInstanceHandler handler = EnvTool.get(FinishedInstanceHandler.class, FinishedInstanceHandler.DEFAULT_KEY);
    handler.handleFinishedInstance(pid);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.EXIT;
  }

  @Override
  public ExitActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new ExitActivityDefinitionData(bpelProcess.getProcessDefinitionData(), this.uuid, null, this.name);
  }

}
