/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Map;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.model.Node;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * the runtime behaviour of the repeatUntil activity.
 *
 * @author Tom Baeyens
 */
public class RepeatUntil extends AbstractBpelActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

//  private static Logger log = Logger.getLogger(RepeatUntil.class.getName());

  protected Expression condition;

  public RepeatUntil() {
    super();
  }


  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  @Override
  public void executeActivity(final BpelExecution execution) {
    final Node node = execution.getNode();
    final Node activity = node.getNodes().get(0);
    //Execute nested activity in a child execution
    execution.createExecution().execute(activity);
  }

  /**
   * Signal method of the NodeBehaviour.
   *
   * @param execution
   *            execution
   * @param signal
   *            signal
   */
  @Override
  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      // this is the child execution returning
      final Node node = execution.getNode();
      final Node activity = node.getNodes().get(0);
      final boolean conditionResult = this.condition.getEvaluator().evaluateBoolean(execution);
      if (conditionResult) {
        // end child execution
        execution.end(Execution.STATE_ENDED);

        final BpelExecution repeatUntilExecution = execution.getParent();
        // remove finished execution from parent
        repeatUntilExecution.removeExecution(execution);

        // signal must be non-null to make the distinction with child
        // executions
        // in the first line of this method
        repeatUntilExecution.signal("finished");
      } else {
        execution.execute(activity);
      }
    } else if (signal.equals("finished")) {
      afterRunned(execution);
    }
  }

  public Expression getCondition() {
    return this.condition;
  }

  public void setCondition(final Expression condition) {
    this.condition = condition;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.REPEAT_UNTIL;
  }

}
