/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.data.def.RethrowActivityDefinitionData;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.pvm.internal.model.ExecutionImpl.Propagation;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;
import org.ow2.orchestra.util.Misc;

/**
 * The runtime behaviour of the rethrow activity.
 *
 * @author S. Ali Tokmen
 */
public class Rethrow extends AbstractBpelActivity {

  private static final long serialVersionUID = 1859357075672007231L;

  private static Logger log = Logger.getLogger(Rethrow.class.getName());

  public Rethrow() {
    super();
  }

  /**
   * throws a Bpel exception
   *
   * @param execution execution
   */
  @Override
  public void executeActivity(final BpelExecution execution) {
    final ScopeRuntime scopeRuntime = execution.getCurrentScopeRuntime();
    if (scopeRuntime == null) {
      Misc.unreachableStatement("Current ScopeRuntime is null!");
    }
    if (!ScopeState.HANDLING_FAULT.equals(scopeRuntime.getState())) {
      Misc.unreachableStatement("State is " + scopeRuntime.getState() + ", it should be " + ScopeState.HANDLING_FAULT);
    }
    final BpelFaultException lastException = scopeRuntime.getLastException();
    if (lastException == null) {
      Misc.unreachableStatement("LastException is null!");
    }

    execution.setPropagation(Propagation.EXPLICIT);
    throw lastException;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.RETHROW;
  }

  @Override
  public RethrowActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new RethrowActivityDefinitionData(bpelProcess.getProcessDefinitionData(), this.uuid, null, this.name);
  }

}
