/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.List;
import java.util.Map;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.model.Node;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * implementation of the bpel sequence.
 *
 * @author Charles Souillard, Tom Baeyens
 * @version $Id: Sequence.java 2901 2009-09-01 10:31:20Z porcherg $
 */
public class Sequence extends AbstractBpelActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  public Sequence() {
    super();
  }


  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  @Override
  public void executeActivity(final BpelExecution execution) {
    // get the current sequence node
    final Node node = execution.getNode();
    // check that the node's previousNeeded is activated
    if (!node.isPreviousNeeded()) {
      throw new OrchestraRuntimeException("behaviour " + Sequence.class.getName()
          + " requires that node.isPreviousNeeded() is set to true");
    }
    // get the first nested nodes in the sequence
    final Node firstNode = node.getNodes().get(0);
    // record that we're going to execute the first node in the sequence
    // record("next-index[" + execution.getNode().getName() + ",0]");
    // execute the first node

    // execute first nested node in a child execution
    execution.createExecution().execute(firstNode);
  }

  /**
   * Signal method of the NodeBehaviour.
   *
   * @param execution
   *            execution
   * @param signal
   *            signal
   */
  @Override
  public void signal(final BpelExecution execution, final String signal, final Map<String, Object> parameters) {
    if (signal == null) {
      final NodeImpl node = execution.getNode();
      final List<Node> nestedNodes = node.getNodes();

      final Node previousNode = execution.getPreviousNode();

      final int nextNodeIndex = nestedNodes.indexOf(previousNode) + 1;

      // this is the child execution returning
      // end child execution
      execution.end(Execution.STATE_ENDED);

      final BpelExecution sequenceExecution = execution.getParent();
      // remove finished execution from parent
      sequenceExecution.removeExecution(execution);

      // if the index is equal to the size of the list of nested nodes
      if (nestedNodes.size() == nextNodeIndex) {
        // sequence is ended, signal parent execution


        // signal must be non-null to make the distinction with child
        // executions
        // in the first line of this method
        sequenceExecution.signal("finished");
      } else {
        // else just increment the index and execute the next node
        final Node nextNode = nestedNodes.get(nextNodeIndex);
        // record the next index
        // record("next-index[" + execution.getNode().getName() + "," +
        // nextNodeIndex + "]");
        // execute the next node in a child execution
        sequenceExecution.createExecution().execute(nextNode);
      }
    } else if (signal.equals("finished")) {
      afterRunned(execution);
    }
  }

  @Override
  public ActivityType getType() {
    return ActivityType.SEQUENCE;
  }

}
