/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.definition.activity;

import java.util.logging.Logger;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.pvm.Execution;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.ScopeRuntime;

/**
 * @author S. Ali Tokmen
 */
public class TerminationHandlerActivity extends ActivityWithSingleChild {

  private static final long serialVersionUID = -5965747816807695599L;

  private static Logger log = Logger.getLogger(TerminationHandlerActivity.class.getName());

  private NodeImpl scope;

  protected TerminationHandlerActivity() {
    // Keep Hibernate happy
    super(null, null, null);
  }

  public TerminationHandlerActivity(final NodeImpl child, final ActivityDefinitionUUID uuid, final String name,
    final NodeImpl scope) {
    super(child, uuid, name);
    this.scope = scope;
  }

  @Override
  public void signal(final BpelExecution execution) {
    // this is the child execution returning
    // end child execution
    execution.end(Execution.STATE_ENDED);

    final BpelExecution scopeExecution = execution.getParent();

    // remove finished execution from parent
    scopeExecution.removeExecution(execution);

    final ScopeRuntime scopeRuntime = scopeExecution.getCurrentScopeRuntime();
    scopeRuntime.setState(ScopeState.TERMINATED);

    scopeExecution.terminate();
  }

  @Override
  public ActivityType getType() {
    return ActivityType.TERMINATION_HANDLER;
  }

}
