/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.activity;

import java.util.Date;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.element.OnAlarm;
import org.ow2.orchestra.facade.data.def.ActivityDefinitionData;
import org.ow2.orchestra.facade.data.runtime.TimerData;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.EnvTool;

/**
 * the runtime behaviour of the wait activity.
 *
 * @author Tom Baeyens, Charles Souillard, Pascal Verdage
 */
public class Wait extends AbstractBpelActivity {

  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  private static Logger log = Logger.getLogger(Wait.class.getName());

  private Expression durationExpression = null;

  private Expression deadlineExpression = null;

  public Wait() {
    super();
  }

  /**
   * business logic.
   *
   * @param execution
   *            execution
   */
  /*
   * sample duration: 'P1Y2M15DT2H59M59S':
   *    1 year, 2 months, 15 days, 2 hours, 59 minutes, 59 seconds
   * sample deadline: '2008-03-15T18:00'
   */
  @Override
  public void executeActivity(final BpelExecution execution) {
    final Date date = OnAlarm.computeTimer(execution,
        this.deadlineExpression, this.durationExpression, null);
    execution.createTimer(null, "timeout", null, date, null, null, null);
    EnvTool.getRecorder().recordTimerStarted(new TimerData(execution.getActivityInstanceUUID(), null, date));

    execution.waitForSignal();
  }

  public void setDurationExpression(final Expression durationExpression) {
    this.durationExpression = durationExpression;
  }

  public void setDeadlineExpression(final Expression deadlineExpression) {
    this.deadlineExpression = deadlineExpression;
  }

  @Override
  public void signal(final BpelExecution execution, final String signal,
      final Map<String, Object> parameters) {
    if (Wait.log.isLoggable(Level.FINE)) {
      Wait.log.fine("timer reached execution time at " + System.currentTimeMillis());
    }
    this.afterRunned(execution);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.WAIT;
  }

  @Override
  public ActivityDefinitionData createDefinitionData(final BpelProcess bpelProcess) {
    return new ActivityDefinitionData(bpelProcess.getProcessDefinitionData(), this.uuid, null,
      this.getType(), this.name);
  }

}
