/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * @author Charles Souillard
 */
public class Copy {

  private static Logger log = Logger.getLogger(Copy.class.getName());


  protected long dbid;

  /** keepSrcElementName attribute of this copy. */
  protected boolean keepSrcElementName;

  /** ignoreMissingFromData attribute of this copy. */
  protected boolean ignoreMissingFromData;

  /** from of this copy. */
  protected From from;

  /** to of this copy. */
  protected To to;

  /**
   * Executes this copy : copy from into to.
   *
   * @param execution
   *            execution
   */
  public void execute(final BpelExecution execution) {
    final Object fromValue = this.from.getValue(execution);
    if (Copy.log.isLoggable(Level.FINE)) {
      Copy.log.fine("Copy from value = " + fromValue);
    }
    if (fromValue == null) {
      throw new OrchestraRuntimeException("fromValue is null, copy = " + this.toString());
    }

    this.to.setValue(execution, fromValue, this.keepSrcElementName);
  }

  @Override
  public String toString() {
    String result = "<copy  keepSrcElementName=\"" + this.keepSrcElementName
        + "\">\n";
    result += this.from.toString();
    result += this.to.toString();
    result += "</copy>\n";
    return result;
  }

  public boolean isKeepSrcElementName() {
    return this.keepSrcElementName;
  }

  public From getFrom() {
    return this.from;
  }

  public To getTo() {
    return this.to;
  }

  public boolean isIgnoreMissingFromData() {
    return this.ignoreMissingFromData;
  }

  public void setIgnoreMissingFromData(final boolean ignoreMissingFromData) {
    this.ignoreMissingFromData = ignoreMissingFromData;
  }

  public void setFrom(final From from) {
    this.from = from;
  }

  public void setKeepSrcElementName(final boolean keepSrcElementName) {
    this.keepSrcElementName = keepSrcElementName;
  }

  public void setTo(final To to) {
    this.to = to;
  }
}
