/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.io.Serializable;

import javax.xml.namespace.QName;

import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.CorrelationSetRuntime;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.var.MessageVariable;

/**
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Guillaume Porcher
 */
public class Correlation implements Serializable {
  /** serialVersionUID. */
  private static final long serialVersionUID = 1L;

  /** correlationSet referenced by this correlation based on the set attribute. */
  protected CorrelationSet correlationSet;
  /** initiate attribute of this correlation. */
  protected Initiate initiate = Initiate.no;

  /**
   * Different values of initiate field.
   */
  public enum Initiate {
    /**
     * The related activity must attempt to initiate the correlation set.
     */
    yes,
    /**
     * The related activity must attempt to initiate the correlation set, if
     * the correlation set is not yet initiated.
     */
    join,
    /**
     * The related activity must not attempt to initiate the correlation set.
     */
    no
  }

  /**
   * Setter based on a string.
   * @param initiate correct values : yes, no, join.
   */
  public void setInitiate(final String initiate) {
    if (initiate.equals("yes")) {
      this.initiate = Initiate.yes;
    } else if (initiate.equals("join")) {
      this.initiate = Initiate.join;
    } else {
      this.initiate = Initiate.no;
    }
  }

  protected QName getMessageType(final MessageVariable message,
      final ReceivingElement receivingElement, final BpelExecution bpelExecution) {
    return bpelExecution.getWsdlInfos().getPortType(
        receivingElement.getPortTypeQName()).getOperation(receivingElement.getOperation(),null, null)
        .getInput().getMessage().getQName();
  }

  public boolean correlationOK(final MessageVariable message,
    final ReceivingElement receivingElement, final BpelExecution bpelExecution) {
    final CorrelationSetRuntime correlationSetRuntime = bpelExecution.getCorrelationSetByName(this.correlationSet.getName());
    if (!correlationSetRuntime.isInitialized()) {
      return true;
    }
    final QName messageType = this.getMessageType(message, receivingElement, bpelExecution);
    return correlationSetRuntime.validateCSValues(bpelExecution, message, messageType);
  }

  public CorrelationSet getCorrelationSet() {
    return this.correlationSet;
  }
  public void setCorrelationSet(final CorrelationSet correlationSet) {
    this.correlationSet = correlationSet;
  }
  public Initiate getInitiate() {
    return this.initiate;
  }
  public void setInitiate(final Initiate initiate) {
    this.initiate = initiate;
  }

}


