/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.facade.def.Variant;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.var.MessageVariable;
import org.w3c.dom.Element;

/**
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Stanislas Giraudet
 *         De Boudemange
 */
public class From extends FromTo {

  private static Logger log = Logger.getLogger(From.class.getName());

  /** literal child of this from. */
  protected Literal literal;

  public From() {
    super();
  }

  /**
   * get the selected node.
   *
   * @param execution The execution environment for the Activity.
   * @return the node selectionned.
   */
  public Object getValue(final BpelExecution execution) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("Getting from value of " + toString());
    }
    if (variant == Variant.LITERAL) {
      return literal.getValue();
    } else if (variant == Variant.EXPR) {
      return expression.getEvaluator().evaluate(execution);
    } else if (variant == Variant.VAR) {
      VariableRuntime var = execution.getVariableByName(variable);
      if (part == null) {
        if (variableQuery == null) {
          return var.getValue();
        } else {
          Element value = (Element) var.getValue();
          return variableQuery.getEvaluator().evaluate(value, execution);
        }
      } else {
        if (variableQuery == null) {
          return ((MessageVariable) var.getValue()).getPartValue(part);
        } else {
          Element value = ((MessageVariable) var.getValue()).getPartValue(part);
          return variableQuery.getEvaluator().evaluate(value, execution);
        }
      }
    } else if (variant == Variant.PROP) {
      Element contextNode = getContextNode(execution);
      return propertyQuery.getEvaluator().evaluate(contextNode, execution);
    } else if (variant == Variant.PLNK) {
      if (endpointReference != null) {
        if (endpointReference.equals("myRole")) {
          return execution.getPartnerLinkRuntime(partnerLink).getMyRoleEndPointReference();
        } else {
          return execution.getPartnerLinkRuntime(partnerLink).getPartnerRoleEndPointReference();
        }
      } else {
        return execution.getPartnerLinkRuntime(partnerLink).getPartnerRoleEndPointReference();
      }
    } else {
      throw new OrchestraRuntimeException("Unknow VARIANT TYPE : " + variant);
    }
  }

  @Override
  public String toString() {
    String result = "<from";
    if (variant == Variant.EXPR) {
      result += "  expressionLanguage=\"" + this.expressionLanguage + "\"";
      result += ">";
      result += expression.getText();
    } else if (variant == Variant.VAR) {
      result += " variable=\"" + variable + "\"";
      if (part != null) {
        result += " part=\"" + part + "\"";
      }
      result += ">\n";
      if (variableQuery != null) {
        result += variableQuery.toString();
      }
    } else if (variant == Variant.PROP) {
      result += " variable=\"" + variable + "\"" + " property=\"" + this.property + "\"";
      result += ">\n";
    } else if (variant == Variant.PLNK) {
      result += " partnerLink=\"" + partnerLink + "\"" + " endpointReference=\"" + this.endpointReference + "\"";
      result += ">\n";
    } else if (variant == Variant.LITERAL) {
      result += ">\n";
      result += literal.toString();
    }

    result += "</from>\n";
    return result;
  }

  public Literal getLiteral() {
    return literal;
  }

  public void setLiteral(final Literal literal) {
    this.literal = literal;
  }

}
