/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.List;
import java.util.logging.Logger;

import javax.xml.namespace.QName;

import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.util.XmlConstants;

/**
 * @author Charles Souillard, Guillaume Renault
 */
public class OnEvent implements ReceivingElement {

  protected long dbid;
  protected List<Correlation> correlations;
  protected String messageExchange = XmlConstants.BPEL_DEFAULT_MESSAGE_EXCHANGE;
  protected OperationKey operationKey;
  protected String partnerLinkName;
  /** fromParts management. */
  protected List<Copy> inCopies;
  protected String variable = null;
  protected Variable anonymousInboundMessageVariable;

  protected NodeImpl activity;

  private static Logger log = Logger.getLogger(OnEvent.class.getName());

  // ////////////////////////////////////////////////////////////////////
  // Getters
  // ////////////////////////////////////////////////////////////////////

  public List<Correlation> getCorrelations() {
    return this.correlations;
  }

  public String getMessageExchange() {
    return this.messageExchange;
  }

  public String getOperation() {
    return this.getOperationKey().getOperationName();
  }

  public String getVariable() {
    return this.variable;
  }

  public String getPartnerLinkName() {
    return this.partnerLinkName;
  }

  public OperationKey getOperationKey() {
    return this.operationKey;
  }

  public QName getPortTypeQName() {
    return this.getOperationKey().getPortTypeQName();
  }

  public long getDbid() {
    return this.dbid;
  }

  public Variable getAnonymousInboundMessageVariable() {
    return this.anonymousInboundMessageVariable;
  }
  public List<Copy> getInCopies() {
    return this.inCopies;
  }
  // ////////////////////////////////////////////////////////////////////
  // Setters
  // ////////////////////////////////////////////////////////////////////

  public void setCorrelations(final List<Correlation> correlations) {
    this.correlations = correlations;
  }

  public void setMessageExchange(final String messageExchange) {
    this.messageExchange = messageExchange;
  }


  public void setPartnerLinkName(final String partnerLinkName) {
    this.partnerLinkName = partnerLinkName;
  }

  public void setInCopies(final List<Copy> inCopies) {
    this.inCopies = inCopies;
  }

  public void setVariable(final String variable) {
    this.variable = variable;
  }

  /**
   * @param operationKey the operationKey to set
   */
  public void setOperationKey(final OperationKey operationKey) {
    this.operationKey = operationKey;
  }

  public void setAnonymousInboundMessageVariable(final Variable anonymousInboundMessageVariable) {
    this.anonymousInboundMessageVariable = anonymousInboundMessageVariable;
  }

  public void setActivity(final NodeImpl activity) {
    this.activity = activity;
  }
  /**
   * @return the activity
   */
  public NodeImpl getActivity() {
    return this.activity;
  }
}
