/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.definition.element;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.facade.def.Variant;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.runtime.PartnerLinkRuntime;
import org.ow2.orchestra.runtime.VariableRuntime;
import org.ow2.orchestra.var.MessageVariable;
import org.w3c.dom.Element;

/**
 * @author Goulven Le Jeune, Charles Souillard, Candy Chlad, Stanislas Giraudet
 *         De Boudemange
 */
public class To extends FromTo {
  private static Logger log = Logger.getLogger(To.class.getName());

  public void setValue(final BpelExecution execution, final Object value, final boolean keepSrcElementName) {
    if (To.log.isLoggable(Level.FINE)) {
      To.log.fine("Setting to value of " + this.toString());
    }
    if (value instanceof MessageVariable) {
      String toMessageVariableName = null;
      if (this.variant == Variant.VAR) {
        toMessageVariableName = this.variable;

      } else if (this.variant == Variant.EXPR) {
        throw new OrchestraRuntimeException("Unsupported from message to Expression");
      }
      final VariableRuntime var = execution.getVariableByName(toMessageVariableName);
      final MessageVariable fromMessage = (MessageVariable) value;
      var.setValue(fromMessage.duplicate());
      // record variable updated
      var.updated(execution.getActivityInstanceUUID());
      return;
    }

    if (this.variant == Variant.EXPR) {
      final String variableName = this.expression.getEvaluator().assign(execution, value, keepSrcElementName);
      // variable was updated
      if (variableName != null) {
        final VariableRuntime var = execution.getVariableByName(variableName);
        var.updated(execution.getActivityInstanceUUID());
      }
    } else if (this.variant == Variant.VAR) {
      final VariableRuntime var = execution.getVariableByName(this.variable);
      if (this.part == null) {
        if (this.variableQuery == null) {
          var.setValue(value, keepSrcElementName);
        } else {
          final Element contextNode = (Element) var.getValue();
          this.variableQuery.getEvaluator().assign(contextNode, execution, value, keepSrcElementName);
        }
      } else {
        if (this.variableQuery == null) {
          final MessageVariable mess = (MessageVariable) var.getValue();
          mess.setPart(this.part, value, keepSrcElementName);
        } else {
          final Element contextNode = ((MessageVariable) var.getValue()).getPartValue(this.part);
          this.variableQuery.getEvaluator().assign(contextNode, execution, value, keepSrcElementName);
        }
      }
      // record variable updated
      var.updated(execution.getActivityInstanceUUID());
    } else if (this.variant == Variant.PROP) {
      final Element contextNode = this.getContextNode(execution);
      this.propertyQuery.getEvaluator().assign(contextNode, value, execution, keepSrcElementName);
      // record variable updated
      execution.getVariableByName(this.variable).updated(execution.getActivityInstanceUUID());
    } else if (this.variant == Variant.PLNK) {
      final PartnerLinkRuntime partnerLinkRuntime = execution
          .getPartnerLinkRuntime(this.partnerLink);
      if (this.endpointReference != null) {
        if (this.endpointReference.equals("myRole")) {
          partnerLinkRuntime.setMyRoleEndPointReference((Element) value);
        } else {
          partnerLinkRuntime.setPartnerRoleEndPointReference((Element) value, execution.getActivityInstanceUUID());
        }
      } else {
        partnerLinkRuntime.setPartnerRoleEndPointReference((Element) value, execution.getActivityInstanceUUID());
      }
    } else {
      throw new OrchestraRuntimeException("Unknow VARIANT TYPE : " + this.variant);
    }
  }

  /**
   * toString method.
   *
   * @return String string
   */
  @Override
  public String toString() {
    String result = "<to";
    if (this.expressionLanguage != null) {
      result += "  expressionLanguage=\"" + this.expressionLanguage + "\"";
    }
    if (this.variable != null) {
      result += "  variable=\"" + this.variable + "\"";
    }
    if (this.part != null) {
      result += "  part=\"" + this.part + "\"";
    }
    if (this.property != null) {
      result += "  property=\"" + this.property + "\"";
    }
    if (this.partnerLink != null) {
      result += "  partnerLink=\"" + this.partnerLink + "\"";
    }
    result += ">\n";
    if (this.variableQuery != null) {
      result += this.variableQuery.toString();
    }
    if (this.expression != null) {
      result += this.expression.toString();
    }
    result += "</to>\n";
    return result;
  }
}
