/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.env;

import org.hibernate.HibernateException;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.PvmException;
import org.ow2.orchestra.pvm.internal.svc.RetryInterceptor;
import org.ow2.orchestra.util.ArchiveTool.AtomicArchiveException;


/**
 * @author Guillaume Porcher
 *
 */
public class OrchestraRetryInterceptor extends RetryInterceptor {

  @Override
  protected boolean shouldRetry(final RuntimeException e) {
    if (e instanceof OrchestraRuntimeException) {
      return true;
    }
    if (e instanceof PvmException) {
      return true;
    }
    if (e instanceof AtomicArchiveException) {
      return true;
    }
    if (e instanceof IllegalStateException) {
      // thrown by misc.unreachableStatement()
      // can be due to a concurrent modification of the journal.
      return true;
    }
    if (e instanceof HibernateException) {
      return true;
    }
    return super.shouldRetry(e);
  }

}
