/**
 * Copyright (C) 2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.exception;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.exception.FaultWithMessage;
import org.ow2.orchestra.var.MessageVariable;

/**
 * Represents a fault resulting from an Invoke.
 */
public class FaultWithMessageVariable extends BpelFaultException {

  private static final long serialVersionUID = 2381218713694655654L;

  protected MessageVariable faultMessage;

  // for persistence
  protected FaultWithMessageVariable() {
  }

  public FaultWithMessageVariable(final QName faultName, final MessageVariable faultMessage) {
    super(faultName);
    if (faultMessage != null && !faultMessage.getParts().isEmpty()) {
      this.faultMessage = faultMessage.duplicate();
    }
  }

  public MessageVariable getFaultMessage() {
    return this.faultMessage;
  }

  @Override
  public String toString() {
    return "Fault " + this.faultName + (this.faultMessage != null ? " (data: " + this.faultMessage + ")." : ".");
  }

  @Override
  public FaultWithMessageVariable fullCopy() {
    return new FaultWithMessageVariable(this.getFaultName(), this.getFaultMessage());
  }

  @Override
  public FaultWithMessage copy() {
    return new FaultWithMessage(this.faultName, this.faultMessage.copy());
  }
}
