/**
 * Copyright (C) 2008-2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.SourceFull;
import org.ow2.orchestra.facade.def.full.TargetFull;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public abstract class BpelActivityFullDefinitionImpl extends ActivityFullDefinitionImpl implements BpelActivityFullDefinition {

  private static final long serialVersionUID = 2162174359075211450L;

  protected boolean suppressJoinFailure;

  protected List<SourceFull> sourceList;

  protected List<TargetFull> targetList;

  protected String targetsJoinConditionExpressionLanguage;

  protected String targetsJoinConditionBooleanExpression;

  protected BpelActivityFullDefinitionImpl() {
  }

  public BpelActivityFullDefinitionImpl(final ActivityDefinitionUUID uuid, final ProcessDefinitionUUID processUUID,
    final String name) {
    super(uuid, processUUID, name);
    sourceList = new ArrayList<SourceFull>();
    targetList = new ArrayList<TargetFull>();
  }

  public BpelActivityFullDefinitionImpl(final BpelActivityFullDefinition activityFullDefinition) {
    super(activityFullDefinition);
    this.targetsJoinConditionBooleanExpression = activityFullDefinition.getTargetsJoinConditionBooleanExpression();
    this.targetsJoinConditionExpressionLanguage = activityFullDefinition.getTargetsJoinConditionExpressionLanguage();
    this.suppressJoinFailure = activityFullDefinition.isSuppressJoinFailure();
    this.sourceList = FullCopyUtil.fullCopyList(activityFullDefinition.getSourceList());
    this.targetList = FullCopyUtil.fullCopyList(activityFullDefinition.getTargetList());
  }

  /**
   * @return the suppressJoinFailure
   */
  public boolean isSuppressJoinFailure() {
    return suppressJoinFailure;
  }

  /**
   * @param suppressJoinFailure the suppressJoinFailure to set
   */
  public void setSuppressJoinFailure(final boolean suppressJoinFailure) {
    this.suppressJoinFailure = suppressJoinFailure;
  }

  public List<SourceFull> getSourceList() {
    return sourceList;
  }

  public List<TargetFull> getTargetList() {
    return targetList;
  }

  public String getTargetsJoinConditionBooleanExpression() {
    return targetsJoinConditionBooleanExpression;
  }

  public String getTargetsJoinConditionExpressionLanguage() {
    return targetsJoinConditionExpressionLanguage;
  }

  public void addTarget(final TargetFull target) {
    targetList.add(target);
  }

  public void addSource(final SourceFull sourceFull) {
    sourceList.add(sourceFull);
  }

  /**
   * @param targetsJoinConditionExpressionLanguage the targetsJoinConditionExpressionLanguage to set
   */
  public void setTargetsJoinConditionExpressionLanguage(final String targetsJoinConditionExpressionLanguage) {
    this.targetsJoinConditionExpressionLanguage = targetsJoinConditionExpressionLanguage;
  }

  /**
   * @param targetsJoinConditionBooleanExpression the targetsJoinConditionBooleanExpression to set
   */
  public void setTargetsJoinConditionBooleanExpression(final String targetsJoinConditionBooleanExpression) {
    this.targetsJoinConditionBooleanExpression = targetsJoinConditionBooleanExpression;
  }

}
