/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityWithChildrenFullDefinition;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public abstract class BpelActivityWithChildrenFullDefinitionImpl extends BpelActivityFullDefinitionImpl implements
  BpelActivityWithChildrenFullDefinition {

  private static final long serialVersionUID = 8901738568448948365L;

  /**
   * Warning this field is not persisted so you have to use getter instead of
   * direct reference.
   */
  protected List<ActivityDefinitionUUID> enclosedActivitiesUUID;

  protected List<BpelActivityFullDefinition> enclosedActivities;

  protected BpelActivityWithChildrenFullDefinitionImpl() {
  }

  public BpelActivityWithChildrenFullDefinitionImpl(final ActivityDefinitionUUID uuid, final ProcessDefinitionUUID processUUID,
    final String name) {
    super(uuid, processUUID, name);
    this.enclosedActivities = new ArrayList<BpelActivityFullDefinition>();
  }

  public BpelActivityWithChildrenFullDefinitionImpl(final BpelActivityWithChildrenFullDefinition activityFullDefinition) {
    super(activityFullDefinition);
    this.enclosedActivities = (List) FullCopyUtil.fullCopyList(activityFullDefinition.getEnclosedActivities());
  }

  public List<BpelActivityFullDefinition> getEnclosedActivities() {
    return this.enclosedActivities;
  }

  public void addEnclosedActivity(final BpelActivityFullDefinition enclosedActivity) {
    this.enclosedActivities.add(enclosedActivity);
    this.getEnclosedActivitiesUUID().add(enclosedActivity.getUUID());
  }

  public List<ActivityDefinitionUUID> getEnclosedActivitiesUUID() {
    if (this.enclosedActivitiesUUID == null) {
      final List<ActivityDefinitionUUID> enclosedActivitiesUUID = new ArrayList<ActivityDefinitionUUID>();
      for (final BpelActivityFullDefinition enclosedActivity : this.enclosedActivities) {
        enclosedActivitiesUUID.add(enclosedActivity.getUUID());
      }
      this.enclosedActivitiesUUID = enclosedActivitiesUUID;
    }
    return this.enclosedActivitiesUUID;
  }

}
