/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.CopyDefinition;
import org.ow2.orchestra.facade.def.FromDefinition;
import org.ow2.orchestra.facade.def.ToDefinition;
import org.ow2.orchestra.facade.def.full.CopyFullDefinition;
import org.ow2.orchestra.facade.def.full.FromFullDefinition;
import org.ow2.orchestra.facade.def.full.ToFullDefinition;
import org.ow2.orchestra.facade.def.impl.CopyDefinitionImpl;

public class CopyFullDefinitionImpl implements CopyFullDefinition {

  protected long dbid;

  private static final long serialVersionUID = 6935772225303468627L;

  private List<FromDefinition> fromDefinitionList;

  private boolean keepSrcElementName;

  private List<ToDefinition> toDefinitionList;

  public CopyFullDefinitionImpl() {
    this.fromDefinitionList = new ArrayList<FromDefinition>();
    this.toDefinitionList = new ArrayList<ToDefinition>();
  }

  public CopyFullDefinitionImpl(final CopyFullDefinition copyFullDefinition) {
    this.keepSrcElementName = copyFullDefinition.isKeepSrcElementName();
    this.fromDefinitionList = FullCopyUtil.fullCopyList((List) copyFullDefinition.getFromDefinitionList());
    this.toDefinitionList = FullCopyUtil.fullCopyList((List) copyFullDefinition.getToDefinitionList());
  }

  public List<FromDefinition> getFromDefinitionList() {
    return fromDefinitionList;
  }

  public boolean isKeepSrcElementName() {
    return keepSrcElementName;
  }

  public CopyDefinition copy() {
    return new CopyDefinitionImpl(this);
  }

  public CopyFullDefinition fullCopy() {
    return new CopyFullDefinitionImpl(this);
  }

  public void addFromDefinition(final FromFullDefinition fromFullDefinition) {
    this.fromDefinitionList.add(fromFullDefinition);
  }

  public void addToDefinition(final ToFullDefinition toFullDefinition) {
    this.toDefinitionList.add(toFullDefinition);
  }

  public List<ToDefinition> getToDefinitionList() {
    return toDefinitionList;
  }

}
