/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.CompletionConditionDefinition;
import org.ow2.orchestra.facade.def.ForeachActivityDefinition;
import org.ow2.orchestra.facade.def.full.ForeachActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.ForeachActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class ForeachActivityFullDefinitionImpl extends BpelActivityWithSingleChildFullDefinitionImpl implements
  ForeachActivityFullDefinition {

  private String finalCounterExpression;

  private String finalCounterExpressionLanguage;

  private String startCounterExpression;

  private String startCounterExpressionLanguage;

  private String counterName;

  private boolean parallelValue;

  private CompletionConditionDefinition completionConditionDefinition;

  protected ForeachActivityFullDefinitionImpl(){
  }

  public ForeachActivityFullDefinitionImpl(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processDefinitionUUID, name);
  }

  public ForeachActivityFullDefinitionImpl(final ForeachActivityFullDefinition foreachActivityFullDefinition) {
    super(foreachActivityFullDefinition);
    this.counterName = foreachActivityFullDefinition.getCounterName();
    this.parallelValue = foreachActivityFullDefinition.isParallel();
    this.finalCounterExpression = foreachActivityFullDefinition.getFinalCounterExpression();
    this.finalCounterExpressionLanguage = foreachActivityFullDefinition.getFinalCounterExpressionLanguage();
    this.startCounterExpression = foreachActivityFullDefinition.getStartCounterExpression();
    this.startCounterExpressionLanguage = foreachActivityFullDefinition.getStartCounterExpressionLanguage();
    this.completionConditionDefinition = CopyUtil.copy(foreachActivityFullDefinition.getCompletionConditionDefinition());
  }

  @Override
  public ActivityType getType() {
    return ActivityType.FOR_EACH;
  }

  public String getFinalCounterExpression() {
    return this.finalCounterExpression;
  }

  public String getFinalCounterExpressionLanguage() {
    return this.finalCounterExpressionLanguage;
  }

  public String getStartCounterExpression() {
    return this.startCounterExpression;
  }

  public String getStartCounterExpressionLanguage() {
    return this.startCounterExpressionLanguage;
  }

  public void setFinalCounterExpression(final String finalCounterExpression) {
    this.finalCounterExpression = finalCounterExpression;
  }

  public void setFinalCounterExpressionLanguage(final String finalCounterExpressionLanguage) {
    this.finalCounterExpressionLanguage = finalCounterExpressionLanguage;
  }

  public void setStartCounterExpression(final String startCounterExpression) {
    this.startCounterExpression = startCounterExpression;
  }

  public void setStartCounterExpressionLanguage(final String startCounterExpressionLanguage) {
    this.startCounterExpressionLanguage = startCounterExpressionLanguage;
  }

  public ForeachActivityDefinition copy() {
    return new ForeachActivityDefinitionImpl(this);
  }

  public ForeachActivityFullDefinition fullCopy() {
    return new ForeachActivityFullDefinitionImpl(this);
  }

  public String getCounterName() {
    return this.counterName;
  }

  public boolean isParallel() {
    return this.parallelValue;
  }

  public void setCounterName(final String counterName) {
    this.counterName = counterName;
  }

  public void setParallelValue(final boolean parallelValue) {
    this.parallelValue = parallelValue;
  }

  public void setCompletionConditionDefintion(final CompletionConditionDefinition completionConditionDefinition) {
    this.completionConditionDefinition = CopyUtil.copy(completionConditionDefinition);
  }

  public CompletionConditionDefinition getCompletionConditionDefinition() {
    return this.completionConditionDefinition;
  }
}
