/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.IfActivityDefinition;
import org.ow2.orchestra.facade.def.full.ElseFullDefinition;
import org.ow2.orchestra.facade.def.full.ElseIfFullDefinition;
import org.ow2.orchestra.facade.def.full.IfActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.IfActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class IfActivityFullDefinitionImpl extends BpelActivityWithSingleChildFullDefinitionImpl implements
  IfActivityFullDefinition {

  private String expressionLanguage;

  private String expression;

  private List<ElseIfFullDefinition> elseIfDefinitionList;

  private ElseFullDefinition elseDefinition;

  protected IfActivityFullDefinitionImpl() {
  }

  public IfActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
    this.elseIfDefinitionList = new ArrayList<ElseIfFullDefinition>();
  }

  public IfActivityFullDefinitionImpl(final IfActivityFullDefinition ifActivityFullDefinition) {
    super(ifActivityFullDefinition);
    this.expression = ifActivityFullDefinition.getExpression();
    this.expressionLanguage = ifActivityFullDefinition.getExpressionLanguage();
    this.elseDefinition = FullCopyUtil.fullCopy(ifActivityFullDefinition.getElseDefinition());
    this.elseIfDefinitionList = FullCopyUtil.fullCopyList(ifActivityFullDefinition.getElseIfDefinitionList());
  }

  @Override
  public ActivityType getType() {
    return ActivityType.IF;
  }

  public IfActivityDefinition copy() {
    return new IfActivityDefinitionImpl(this);
  }

  public IfActivityFullDefinition fullCopy() {
    return new IfActivityFullDefinitionImpl(this);
  }

  public void setExpression(final String expression) {
    this.expression = expression;
  }

  public void setExpressionLanguage(final String expressionLanguage) {
    this.expressionLanguage = expressionLanguage;
  }

  public String getExpression() {
    return expression;
  }

  public String getExpressionLanguage() {
    return expressionLanguage;
  }

  public void addElseIfFullDefinition(final ElseIfFullDefinition elseIfFullDefinition) {
    this.elseIfDefinitionList.add(elseIfFullDefinition);
  }

  public List<ElseIfFullDefinition> getElseIfDefinitionList() {
    return FullCopyUtil.fullCopyList(elseIfDefinitionList);
  }

  public void setElseFullDefinition(final ElseFullDefinition elseFullDefinition) {
    this.elseDefinition = elseFullDefinition;
  }

  public ElseFullDefinition getElseDefinition() {
    return elseDefinition;
  }
}
