/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.CorrelationDefinition;
import org.ow2.orchestra.facade.def.FromPartDefinition;
import org.ow2.orchestra.facade.def.InvokeActivityDefinition;
import org.ow2.orchestra.facade.def.ToPartDefinition;
import org.ow2.orchestra.facade.def.full.InvokeActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.InvokeDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public class InvokeActivityFullDefinitionImpl extends BpelActivityFullDefinitionImpl implements InvokeActivityFullDefinition {

  private static final long serialVersionUID = -6313088303704824547L;

  private List<CorrelationDefinition> correlationDefinitions;

  private List<FromPartDefinition> fromPartDefinitions;

  private List<ToPartDefinition> toPartDefinitions;

  private String inputVariable;

  private String outputVariable;

  private String operation;

  private QName portType;

  private String partnerLink;

  protected InvokeActivityFullDefinitionImpl() {
  }

  public InvokeActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
    this.correlationDefinitions = new ArrayList<CorrelationDefinition>();
    this.toPartDefinitions = new ArrayList<ToPartDefinition>();
    this.fromPartDefinitions = new ArrayList<FromPartDefinition>();
  }

  public InvokeActivityFullDefinitionImpl(final InvokeActivityFullDefinition invokeDefinition) {
    super(invokeDefinition);
    this.correlationDefinitions = CopyUtil.copyList(invokeDefinition.getCorrelationDefinitions());
    this.fromPartDefinitions = CopyUtil.copyList(invokeDefinition.getFromPartDefinitions());
    this.toPartDefinitions = CopyUtil.copyList(invokeDefinition.getToPartDefinitions());
    this.partnerLink = invokeDefinition.getPartnerLink();
    this.inputVariable = invokeDefinition.getInputVariable();
    this.outputVariable = invokeDefinition.getOutputVariable();
    this.operation = invokeDefinition.getOperation();
    this.portType = invokeDefinition.getPortType();
  }

  /**
   * @return the correlationDefinitions
   */
  public List<CorrelationDefinition> getCorrelationDefinitions() {
    return this.correlationDefinitions;
  }

  /**
   * @return the fromPartDefinitions
   */
  public List<FromPartDefinition> getFromPartDefinitions() {
    return this.fromPartDefinitions;
  }

  /**
   * @return the inputVariable
   */
  public String getInputVariable() {
    return this.inputVariable;
  }

  /**
   * @return the outputVariable
   */
  public String getOutputVariable() {
    return this.outputVariable;
  }

  /**
   * @return the operation
   */
  public String getOperation() {
    return this.operation;
  }

  /**
   * @return the toPartDefinitions
   */
  public List<ToPartDefinition> getToPartDefinitions() {
    return this.toPartDefinitions;
  }

  /**
   * @return the portType
   */
  public QName getPortType() {
    return this.portType;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.INVOKE;
  }

  public void addFromPartDefinition(final FromPartDefinition fromPartDefinition) {
    this.fromPartDefinitions.add(fromPartDefinition);
  }

  public void addToPartDefinition(final ToPartDefinition toPartDefinition) {
    this.toPartDefinitions.add(toPartDefinition);
  }

  public InvokeActivityDefinition copy() {
    return new InvokeDefinitionImpl(this);
  }

  public InvokeActivityFullDefinition fullCopy() {
    return new InvokeActivityFullDefinitionImpl(this);
  }


  /**
   * @param inputVariable the inputVariable to set
   */
  public void setInputVariable(final String inputVariable) {
    this.inputVariable = inputVariable;
  }


  /**
   * @param outputVariable the outputVariable to set
   */
  public void setOutputVariable(final String outputVariable) {
    this.outputVariable = outputVariable;
  }


  /**
   * @param operation the operation to set
   */
  public void setOperation(final String operation) {
    this.operation = operation;
  }


  /**
   * @param portType the portType to set
   */
  public void setPortType(final QName portType) {
    this.portType = portType;
  }


  /**
   * @return the partnerLink
   */
  public String getPartnerLink() {
    return this.partnerLink;
  }


  /**
   * @param partnerLink the partnerLink to set
   */
  public void setPartnerLink(final String partnerLink) {
    this.partnerLink = partnerLink;
  }


}
