/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.ForDefinition;
import org.ow2.orchestra.facade.def.OnAlarmDefinition;
import org.ow2.orchestra.facade.def.RepeatEveryDefinition;
import org.ow2.orchestra.facade.def.UntilDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.OnAlarmFullDefinition;
import org.ow2.orchestra.facade.def.impl.OnAlarmDefinitionImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;


public class OnAlarmFullDefinitionImpl implements OnAlarmFullDefinition {

  private UntilDefinition untilDefinition;
  private ForDefinition forDefinition;
  private RepeatEveryDefinition repeatEveryDefinition;
  private ActivityDefinitionUUID activityDefinitionUUID;
  private BpelActivityFullDefinition activityFullDefinition;

  public OnAlarmFullDefinitionImpl(){
  }

  public OnAlarmFullDefinitionImpl(final OnAlarmFullDefinition onAlarmFullDefinition) {
    this.activityDefinitionUUID = onAlarmFullDefinition.getActivityDefinitionUUID();
    this.activityFullDefinition = (BpelActivityFullDefinition) FullCopyUtil.fullCopy(onAlarmFullDefinition.getActivityFullDefinition());
    this.forDefinition = CopyUtil.copy(onAlarmFullDefinition.getForDefinition());
    this.untilDefinition = CopyUtil.copy(onAlarmFullDefinition.getUntilDefinition());
    this.repeatEveryDefinition = CopyUtil.copy(onAlarmFullDefinition.getRepeatEveryDefinition());
  }

  public BpelActivityFullDefinition getActivityFullDefinition() {
    return this.activityFullDefinition;
  }

  public ActivityDefinitionUUID getActivityDefinitionUUID() {
    return this.activityDefinitionUUID;
  }

  public ForDefinition getForDefinition() {
    return this.forDefinition;
  }

  public UntilDefinition getUntilDefinition() {
    return this.untilDefinition;
  }

  public RepeatEveryDefinition getRepeatEveryDefinition() {
    return this.repeatEveryDefinition;
  }

  public OnAlarmDefinition copy() {
    return new OnAlarmDefinitionImpl(this);
  }

  public OnAlarmFullDefinition fullCopy() {
    return new OnAlarmFullDefinitionImpl(this);
  }

  public void setActivityDefinitionUUID(final ActivityDefinitionUUID activityDefinitionUUID) {
    this.activityDefinitionUUID = activityDefinitionUUID;
  }

  public void setEnclosedActivityFullDefinition(final BpelActivityFullDefinition activityFullDefinition) {
    this.activityFullDefinition = activityFullDefinition;
  }

  public void setForDefinition(final ForDefinition forDefinition) {
    this.forDefinition = forDefinition;
  }

  public void setUntilDefinition(final UntilDefinition untilDefinition) {
    this.untilDefinition = untilDefinition;
  }

  public void setRepeatEveryDefinition(final RepeatEveryDefinition repeatEveryDefinition) {
    this.repeatEveryDefinition = repeatEveryDefinition;
  }
}
