/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.CorrelationDefinition;
import org.ow2.orchestra.facade.def.FromPartDefinition;
import org.ow2.orchestra.facade.def.OnEventDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.OnEventFullDefinition;
import org.ow2.orchestra.facade.def.impl.OnEventDefinitionImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;

public class OnEventFullDefinitionImpl implements OnEventFullDefinition {

  protected long dbid;

  private String variable;
  private QName portType;
  private String partnerLink;
  private String operation;
  private String messageExchange;
  private List<FromPartDefinition> fromPartDefinitionList;
  private List<CorrelationDefinition> correlationList;
  private ActivityDefinitionUUID activityDefinitionUUID;
  private BpelActivityFullDefinition activityFullDefinition;

  public OnEventFullDefinitionImpl() {
  }

  public OnEventFullDefinitionImpl(final OnEventFullDefinition onEventActivityFullDefinition) {
    this.variable = onEventActivityFullDefinition.getVariable();
    this.portType = onEventActivityFullDefinition.getPortType();
    this.partnerLink = onEventActivityFullDefinition.getPartnerLink();
    this.operation = onEventActivityFullDefinition.getOperation();
    this.messageExchange = onEventActivityFullDefinition.getMessageExchange();
    this.activityDefinitionUUID = onEventActivityFullDefinition.getActivityDefinitionUUID();
    this.activityFullDefinition = (BpelActivityFullDefinition) FullCopyUtil.fullCopy(onEventActivityFullDefinition.getActivityFullDefinition());
    this.fromPartDefinitionList = CopyUtil.copyList(onEventActivityFullDefinition.getFromPartDefinitionList());
    this.correlationList = CopyUtil.copyList(onEventActivityFullDefinition.getCorrelationDefinitionList());
  }

  public List<CorrelationDefinition> getCorrelationDefinitionList() {
    return this.correlationList;
  }

  public List<FromPartDefinition> getFromPartDefinitionList() {
    return this.fromPartDefinitionList;
  }

  public String getMessageExchange() {
    return this.messageExchange;
  }

  public String getOperation() {
    return this.operation;
  }

  public String getPartnerLink() {
    return this.partnerLink;
  }

  public QName getPortType() {
    return this.portType;
  }

  public String getVariable() {
    return this.variable;
  }

  public OnEventDefinition copy() {
    return new OnEventDefinitionImpl(this);
  }

  public OnEventFullDefinition fullCopy() {
    return new OnEventFullDefinitionImpl(this);
  }

  public void setVariable(final String variable) {
    this.variable = variable;
  }

  public void setPortType(final QName portType) {
    this.portType = portType;
  }

  public void setPartnerLink(final String partnerLink) {
    this.partnerLink = partnerLink;
  }

  public void setOperation(final String operation) {
    this.operation = operation;
  }

  public void setMessageExchange(final String messageExchange) {
    this.messageExchange = messageExchange;
  }

  public void setFromPartDefinition(final List<FromPartDefinition> fromPartDefinitionList) {
    this.fromPartDefinitionList = CopyUtil.copyList(fromPartDefinitionList);
  }

  public void setCorrelationDefinition(final List<CorrelationDefinition> correlationList) {
    this.correlationList = CopyUtil.copyList(correlationList);
  }

  public BpelActivityFullDefinition getActivityFullDefinition() {
    return this.activityFullDefinition;
  }

  public void setEnclosedActivityFullDefinition(final BpelActivityFullDefinition activityFullDefinition) {
    this.activityFullDefinition = activityFullDefinition;
  }

  public ActivityDefinitionUUID getActivityDefinitionUUID() {
    return this.activityDefinitionUUID;
  }

  public void setActivityDefinitionUUID(final ActivityDefinitionUUID activityDefinitionUUID) {
    this.activityDefinitionUUID = activityDefinitionUUID;
  }
}
