/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.CorrelationDefinition;
import org.ow2.orchestra.facade.def.FromPartDefinition;
import org.ow2.orchestra.facade.def.OnMessageDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.OnMessageFullDefinition;
import org.ow2.orchestra.facade.def.impl.OnMessageDefinitionImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;

public class OnMessageFullDefinitionImpl implements OnMessageFullDefinition {

  protected long dbid;

  private String variable;
  private QName portType;
  private String partnerLink;
  private String operation;
  private String messageExchange;
  private List<FromPartDefinition> fromPartDefinitionList;
  private List<CorrelationDefinition> correlationList;
  private ActivityDefinitionUUID activityDefinitionUUID;
  private BpelActivityFullDefinition activityFullDefinition;

  public OnMessageFullDefinitionImpl() {
  }

  public OnMessageFullDefinitionImpl(final OnMessageFullDefinition onMessageActivityFullDefinition) {
    this.variable = onMessageActivityFullDefinition.getVariable();
    this.portType = onMessageActivityFullDefinition.getPortType();
    this.partnerLink = onMessageActivityFullDefinition.getPartnerLink();
    this.operation = onMessageActivityFullDefinition.getOperation();
    this.messageExchange = onMessageActivityFullDefinition.getMessageExchange();
    this.activityDefinitionUUID = onMessageActivityFullDefinition.getActivityDefinitionUUID();
    this.activityFullDefinition = (BpelActivityFullDefinition) FullCopyUtil.fullCopy(onMessageActivityFullDefinition.getActivityFullDefinition());
    this.fromPartDefinitionList = CopyUtil.copyList(onMessageActivityFullDefinition.getFromPartDefinitionList());
    this.correlationList = CopyUtil.copyList(onMessageActivityFullDefinition.getCorrelationDefinitionList());
  }

  public List<CorrelationDefinition> getCorrelationDefinitionList() {
    return this.correlationList;
  }

  public List<FromPartDefinition> getFromPartDefinitionList() {
    return this.fromPartDefinitionList;
  }

  public String getMessageExchange() {
    return this.messageExchange;
  }

  public String getOperation() {
    return this.operation;
  }

  public String getPartnerLink() {
    return this.partnerLink;
  }

  public QName getPortType() {
    return this.portType;
  }

  public String getVariable() {
    return this.variable;
  }

  public OnMessageDefinition copy() {
    return new OnMessageDefinitionImpl(this);
  }

  public OnMessageFullDefinition fullCopy() {
    return new OnMessageFullDefinitionImpl(this);
  }

  public void setVariable(final String variable) {
    this.variable = variable;
  }

  public void setPortType(final QName portType) {
    this.portType = portType;
  }

  public void setPartnerLink(final String partnerLink) {
    this.partnerLink = partnerLink;
  }

  public void setOperation(final String operation) {
    this.operation = operation;
  }

  public void setMessageExchange(final String messageExchange) {
    this.messageExchange = messageExchange;
  }

  public void setFromPartDefinition(final List<FromPartDefinition> fromPartDefinitionList) {
    this.fromPartDefinitionList = CopyUtil.copyList(fromPartDefinitionList);
  }

  public void setCorrelationDefinition(final List<CorrelationDefinition> correlationList) {
    this.correlationList = CopyUtil.copyList(correlationList);
  }

  public BpelActivityFullDefinition getActivityFullDefinition() {
    return this.activityFullDefinition;
  }

  public void setEnclosedActivityFullDefinition(final BpelActivityFullDefinition activityFullDefinition) {
    this.activityFullDefinition = activityFullDefinition;
  }

  public ActivityDefinitionUUID getActivityDefinitionUUID() {
    return this.activityDefinitionUUID;
  }

  public void setActivityDefinitionUUID(final ActivityDefinitionUUID activityDefinitionUUID) {
    this.activityDefinitionUUID = activityDefinitionUUID;
  }
}
