/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.PickActivityDefinition;
import org.ow2.orchestra.facade.def.full.OnAlarmFullDefinition;
import org.ow2.orchestra.facade.def.full.OnMessageFullDefinition;
import org.ow2.orchestra.facade.def.full.PickActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.PickActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class PickActivityFullDefinitionImpl extends BpelActivityFullDefinitionImpl implements PickActivityFullDefinition {

  private boolean createInstance;

  private List<OnMessageFullDefinition> onMessageFullDefinitionList;

  private List<OnAlarmFullDefinition> onAlarmFullDefinitionList;

  protected PickActivityFullDefinitionImpl() {
  }

  public PickActivityFullDefinitionImpl(final PickActivityFullDefinition pickActivityFullDefinition) {
    super(pickActivityFullDefinition);
    this.createInstance = pickActivityFullDefinition.isCreateInstance();
    this.onMessageFullDefinitionList = FullCopyUtil.fullCopyList(pickActivityFullDefinition.getOnMessageDefinitionList());
    this.onAlarmFullDefinitionList = FullCopyUtil.fullCopyList(pickActivityFullDefinition.getOnAlarmDefinitionList());
  }

  public PickActivityFullDefinitionImpl(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processDefinitionUUID, name);
    this.onMessageFullDefinitionList = new ArrayList<OnMessageFullDefinition>();
    this.onAlarmFullDefinitionList = new ArrayList<OnAlarmFullDefinition>();
  }

  @Override
  public ActivityType getType() {
    return ActivityType.PICK;
  }

  public void setOnMessageDefinitionList(final List<OnMessageFullDefinition> onMessageFullDefinitionList) {
    this.onMessageFullDefinitionList = FullCopyUtil.fullCopyList(onMessageFullDefinitionList);
  }

  public List<OnMessageFullDefinition> getOnMessageDefinitionList() {
    return onMessageFullDefinitionList;
  }

  public PickActivityDefinition copy() {
    return new PickActivityDefinitionImpl(this);
  }

  public PickActivityFullDefinition fullCopy() {
    return new PickActivityFullDefinitionImpl(this);
  }

  public boolean isCreateInstance() {
    return createInstance;
  }

  public void setCreateInstance(final boolean createInstance) {
    this.createInstance = createInstance;
  }

  public List<OnAlarmFullDefinition> getOnAlarmDefinitionList() {
    return onAlarmFullDefinitionList;
  }

  public void setOnAlarmDefinitionList(final List<OnAlarmFullDefinition> onAlarmFullDefinitionList) {
    this.onAlarmFullDefinitionList = FullCopyUtil.fullCopyList(onAlarmFullDefinitionList);
  }
}
