/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.Date;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.def.ProcessState;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.def.impl.ProcessDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public class ProcessFullDefinitionImpl extends ScopeActivityFullDefinitionImpl implements ProcessFullDefinition {

  private static final long serialVersionUID = 2606705701938051916L;

  protected String nameSpace;

  private String expressionLanguage;
  private String queryLanguage;

  private Date deployedDate;
  private Date undeployedDate;

  private ProcessState state;

  protected ProcessFullDefinitionImpl() {
    super();
  }

  /**
   * @param uuid uuid of process definition
   * @param nameSpace nameSpace of the process
   * @param name name of process
   */
  public ProcessFullDefinitionImpl(final ProcessDefinitionUUID uuid, final String nameSpace, final String name,
    final String queryLanguage, final String expressionLanguage, final Date deployedDate) {
    super(uuid, uuid, name);
    this.nameSpace = nameSpace;
    this.queryLanguage = queryLanguage;
    this.expressionLanguage = expressionLanguage;
    this.deployedDate = deployedDate;
    this.state = ProcessState.DEPLOYED;
  }

  public ProcessFullDefinitionImpl(final ProcessFullDefinition src) {
    super(src);
    this.nameSpace = src.getNameSpace();
    this.expressionLanguage = src.getExpressionLanguage();
    this.queryLanguage = src.getQueryLanguage();
    this.deployedDate = src.getDeployedDate();
    this.undeployedDate = src.getUndeployedDate();
    this.state = src.getState();
  }

  @Override
  public ProcessDefinitionUUID getUUID() {
    return processDefinitionUUID;
  }

  public String getNameSpace() {
    return nameSpace;
  }

  public String getExpressionLanguage() {
    return expressionLanguage;
  }

  public String getQueryLanguage() {
    return queryLanguage;
  }

  public Date getDeployedDate() {
    return deployedDate;
  }

  public Date getUndeployedDate() {
    return undeployedDate;
  }

  public void setUndeployedDate(final Date undeployedDate) {
    this.undeployedDate = undeployedDate;
    this.state = ProcessState.UNDEPLOYED;
  }

  @Override
  public ProcessDefinitionImpl copy() {
    return new ProcessDefinitionImpl(this);
  }

  @Override
  public ProcessFullDefinitionImpl fullCopy() {
    return new ProcessFullDefinitionImpl(this);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.PROCESS;
  }

  public QName getQName() {
    return new QName(getNameSpace(), getName());
  }

  public ProcessState getState() {
    return state;
  }
}
