/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.CorrelationDefinition;
import org.ow2.orchestra.facade.def.FromPartDefinition;
import org.ow2.orchestra.facade.def.ReceiveActivityDefinition;
import org.ow2.orchestra.facade.def.full.ReceiveActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.ReceiveActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class ReceiveActivityFullDefinitionImpl extends BpelActivityFullDefinitionImpl implements
  ReceiveActivityFullDefinition {

  private static final long serialVersionUID = 8371773564543385510L;

  private boolean createInstance;

  private String variable;

  private QName portType;

  private String partnerLink;

  private String operation;

  private String messageExchange;

  private List<CorrelationDefinition> correlationDefinitionList = null;

  private List<FromPartDefinition> fromPartDefinitionList = null;

  protected ReceiveActivityFullDefinitionImpl() {
  }

  public ReceiveActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
    this.correlationDefinitionList = new ArrayList<CorrelationDefinition>();
    this.fromPartDefinitionList = new ArrayList<FromPartDefinition>();
  }

  public ReceiveActivityFullDefinitionImpl(final ReceiveActivityFullDefinition receiveActivityFullDefinition) {
    super(receiveActivityFullDefinition);
    this.createInstance = receiveActivityFullDefinition.isCreateInstance();
    this.variable = receiveActivityFullDefinition.getVariable();
    this.portType = receiveActivityFullDefinition.getPortType();
    this.partnerLink = receiveActivityFullDefinition.getPartnerLink();
    this.operation = receiveActivityFullDefinition.getOperation();
    this.messageExchange = receiveActivityFullDefinition.getMessageExchange();
    this.correlationDefinitionList = new ArrayList<CorrelationDefinition>();
    this.correlationDefinitionList = CopyUtil.copyList(receiveActivityFullDefinition.getCorrelationDefinitions());
    this.fromPartDefinitionList = new ArrayList<FromPartDefinition>();
    this.fromPartDefinitionList = CopyUtil.copyList(receiveActivityFullDefinition.getFromPartDefinitions());
  }

  public boolean isCreateInstance() {
    return this.createInstance;
  }

  public void setCreateInstance(final boolean createInstance) {
    this.createInstance = createInstance;
  }

  public String getVariable() {
    return this.variable;
  }

  public void setVariable(final String variable) {
    this.variable = variable;
  }

  public QName getPortType() {
    return this.portType;
  }

  public void setPortType(final QName portType) {
    this.portType = portType;
  }

  public String getPartnerLink() {
    return this.partnerLink;
  }

  public void setPartnerLink(final String partnerLink) {
    this.partnerLink = partnerLink;
  }

  public String getOperation() {
    return this.operation;
  }

  public void setOperation(final String operation) {
    this.operation = operation;
  }

  public String getMessageExchange() {
    return this.messageExchange;
  }

  public void setMessageExchange(final String messageExchange) {
    this.messageExchange = messageExchange;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.RECEIVE;
  }

  public ReceiveActivityDefinition copy() {
    return new ReceiveActivityDefinitionImpl(this);
  }

  public ReceiveActivityFullDefinition fullCopy() {
    return new ReceiveActivityFullDefinitionImpl(this);
  }

  public List<CorrelationDefinition> getCorrelationDefinitions() {
    return this.correlationDefinitionList;
  }

  public void setCorrelationDefinition(final List<CorrelationDefinition> correlationDefinitionList) {
    this.correlationDefinitionList = CopyUtil.copyList(correlationDefinitionList);
  }

  public List<FromPartDefinition> getFromPartDefinitions() {
    return this.fromPartDefinitionList;
  }

  public void setFromPartDefinition(final List<FromPartDefinition> fromPartDefinitionList) {
    this.fromPartDefinitionList = CopyUtil.copyList(fromPartDefinitionList);
  }


}
