/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import org.ow2.orchestra.facade.def.RepeatUntilActivityDefinition;
import org.ow2.orchestra.facade.def.full.RepeatUntilActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.RepeatUntilActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 *
 * @author Gael Lalire
 */
public class RepeatUntilActivityFullDefinitionImpl extends BpelActivityWithSingleChildFullDefinitionImpl implements
  RepeatUntilActivityFullDefinition {

  private static final long serialVersionUID = 592699355982800225L;

  private String conditionExpressionLanguage;

  private String conditionBooleanExpression;

  protected RepeatUntilActivityFullDefinitionImpl() {
  }

  public RepeatUntilActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
  }

  public RepeatUntilActivityFullDefinitionImpl(final RepeatUntilActivityFullDefinition repeatUntilActivityDefinition) {
    super(repeatUntilActivityDefinition);
    conditionBooleanExpression = repeatUntilActivityDefinition.getConditionBooleanExpression();
    conditionExpressionLanguage = repeatUntilActivityDefinition.getConditionExpressionLanguage();
  }

  public RepeatUntilActivityDefinition copy() {
    return new RepeatUntilActivityDefinitionImpl(this);
  }

  public RepeatUntilActivityFullDefinition fullCopy() {
    return new RepeatUntilActivityFullDefinitionImpl(this);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.REPEAT_UNTIL;
  }

  /**
   * @return the conditionExpressionLanguage
   */
  public String getConditionExpressionLanguage() {
    return conditionExpressionLanguage;
  }


  /**
   * @param conditionExpressionLanguage the conditionExpressionLanguage to set
   */
  public void setConditionExpressionLanguage(final String conditionExpressionLanguage) {
    this.conditionExpressionLanguage = conditionExpressionLanguage;
  }


  /**
   * @return the conditionBooleanExpression
   */
  public String getConditionBooleanExpression() {
    return conditionBooleanExpression;
  }


  /**
   * @param conditionBooleanExpression the conditionBooleanExpression to set
   */
  public void setConditionBooleanExpression(final String conditionBooleanExpression) {
    this.conditionBooleanExpression = conditionBooleanExpression;
  }

}
