/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.CorrelationDefinition;
import org.ow2.orchestra.facade.def.ReplyActivityDefinition;
import org.ow2.orchestra.facade.def.ToPartDefinition;
import org.ow2.orchestra.facade.def.full.ReplyActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.ReplyActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class ReplyActivityFullDefinitionImpl extends BpelActivityFullDefinitionImpl implements ReplyActivityFullDefinition {

  private QName faultName;
  private String variable;
  private QName portType;
  private String partnerLink;
  private String operation;
  private String messageExchange;

  private List<ToPartDefinition> toPartDefinitionList;
  private List<CorrelationDefinition> correlationDefinitionList;

  protected ReplyActivityFullDefinitionImpl() {
  }

  public ReplyActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
    this.correlationDefinitionList = new ArrayList<CorrelationDefinition>();
    this.toPartDefinitionList = new ArrayList<ToPartDefinition>();
  }

  public ReplyActivityFullDefinitionImpl(final ReplyActivityFullDefinitionImpl replyActivityFullDefinitionImpl) {
    super(replyActivityFullDefinitionImpl);
    this.variable = replyActivityFullDefinitionImpl.getVariable();
    this.portType = replyActivityFullDefinitionImpl.getPortType();
    this.partnerLink = replyActivityFullDefinitionImpl.getPartnerLink();
    this.operation = replyActivityFullDefinitionImpl.getOperation();
    this.messageExchange = replyActivityFullDefinitionImpl.getMessageExchange();
    this.faultName = replyActivityFullDefinitionImpl.getFaultName();
    this.correlationDefinitionList = new ArrayList<CorrelationDefinition>();
    this.correlationDefinitionList = CopyUtil.copyList(replyActivityFullDefinitionImpl.getCorrelationDefinitions());
    this.toPartDefinitionList = new ArrayList<ToPartDefinition>();
    this.toPartDefinitionList = CopyUtil.copyList(replyActivityFullDefinitionImpl.getToPartDefinitions());
  }
  @Override
  public ActivityType getType() {
    return ActivityType.REPLY;
  }

  public String getMessageExchange() {
    return this.messageExchange;
  }

  public String getOperation() {
    return this.operation;
  }

  public String getPartnerLink() {
    return this.partnerLink;
  }

  public QName getPortType() {
    return this.portType;
  }

  public String getVariable() {
    return this.variable;
  }

  public QName getFaultName() {
    return this.faultName;
  }

  public void setVariable(final String variable) {
    this.variable = variable;
  }

  public void setPortType(final QName portType) {
    this.portType = portType;
  }

  public void setPartnerLink(final String partnerLink) {
    this.partnerLink = partnerLink;
  }

  public void setOperation(final String operation) {
    this.operation = operation;
  }

  public void setMessageExchange(final String messageExchange) {
    this.messageExchange = messageExchange;
  }

  public List<CorrelationDefinition> getCorrelationDefinitions() {
    return this.correlationDefinitionList;
  }

  public void setCorrelationDefinition(final List<CorrelationDefinition> correlationDefinitionList) {
    this.correlationDefinitionList = CopyUtil.copyList(correlationDefinitionList);
  }

  public List<ToPartDefinition> getToPartDefinitions() {
    return this.toPartDefinitionList;
  }

  public void setToPartDefinition(final List<ToPartDefinition> toPartDefinitionList) {
    this.toPartDefinitionList = CopyUtil.copyList(toPartDefinitionList);
  }

  public ReplyActivityDefinition copy() {
    return new ReplyActivityDefinitionImpl(this);
  }

  public ReplyActivityFullDefinition fullCopy() {
    return new ReplyActivityFullDefinitionImpl(this);
  }

  public void setFaultName(final QName faultName) {
    this.faultName = faultName;
  }

}
