/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.def.BaseCatchDefinition;
import org.ow2.orchestra.facade.def.CorrelationSetDefinition;
import org.ow2.orchestra.facade.def.PartnerLinkDefinition;
import org.ow2.orchestra.facade.def.VariableDefinition;
import org.ow2.orchestra.facade.def.full.BaseCatchFullDefinition;
import org.ow2.orchestra.facade.def.full.CompensationHandlerFullDefinition;
import org.ow2.orchestra.facade.def.full.OnAlarmFullDefinition;
import org.ow2.orchestra.facade.def.full.OnEventFullDefinition;
import org.ow2.orchestra.facade.def.full.ScopeActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.TerminationHandlerFullDefinition;
import org.ow2.orchestra.facade.def.impl.ScopeActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

/**
 * @author Alfonso Fuca, S. Ali Tokmen
 */
public class ScopeActivityFullDefinitionImpl extends BpelActivityWithSingleChildFullDefinitionImpl implements
  ScopeActivityFullDefinition {

  private static final long serialVersionUID = -8581646083876368009L;

  private boolean exitOnStandardFault;

  protected List<CorrelationSetDefinition> correlationSets = null;

  protected List<VariableDefinition> variables = null;

  protected List<PartnerLinkDefinition> partnerLinks = null;

  protected List<OnEventFullDefinition> onEvents = null;

  protected List<OnAlarmFullDefinition> onAlarms = null;

  private List<String> messageExchanges = null;

  protected List<BaseCatchFullDefinition< ? >> faultHandlers = null;

  protected TerminationHandlerFullDefinition terminationHandler = null;

  protected CompensationHandlerFullDefinition compensationHandler = null;

  protected ScopeActivityFullDefinitionImpl() {
  }

  public ScopeActivityFullDefinitionImpl(final ProcessDefinitionUUID processUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processUUID, name);
    this.variables = new ArrayList<VariableDefinition>();
    this.correlationSets = new ArrayList<CorrelationSetDefinition>();
    this.partnerLinks = new ArrayList<PartnerLinkDefinition>();
    this.messageExchanges = new ArrayList<String>();
    this.faultHandlers = new ArrayList<BaseCatchFullDefinition< ? >>();
  }

  public ScopeActivityFullDefinitionImpl(final ScopeActivityFullDefinition scopeActivityFullDefinition) {
    super(scopeActivityFullDefinition);
    this.variables = CopyUtil.copyList(scopeActivityFullDefinition.getVariables());
    this.correlationSets = CopyUtil.copyList(scopeActivityFullDefinition.getCorrelationSets());
    this.partnerLinks = CopyUtil.copyList(scopeActivityFullDefinition.getPartnerLinks());
    this.onEvents = FullCopyUtil.fullCopyList(scopeActivityFullDefinition.getOnEventEventHandlers());
    this.onAlarms = FullCopyUtil.fullCopyList(scopeActivityFullDefinition.getOnAlarmEventHandlers());

    this.messageExchanges = new ArrayList<String>();
    for (final String messageExchange : scopeActivityFullDefinition.getMessageExchanges()) {
      this.messageExchanges.add(messageExchange);
    }
    this.faultHandlers = new ArrayList<BaseCatchFullDefinition< ? >>();
    for (final BaseCatchFullDefinition< ? > faultHandler : scopeActivityFullDefinition.getFaultHandlers()) {
      this.faultHandlers.add((BaseCatchFullDefinition< ? >) faultHandler.fullCopy());
    }
    this.terminationHandler = (TerminationHandlerFullDefinition) scopeActivityFullDefinition.getTerminationHandler().fullCopy();
    this.compensationHandler = (CompensationHandlerFullDefinition) scopeActivityFullDefinition.getCompensationHandler().fullCopy();
  }

  public ScopeActivityDefinitionImpl copy() {
    return new ScopeActivityDefinitionImpl(this);
  }

  public ScopeActivityFullDefinitionImpl fullCopy() {
    return new ScopeActivityFullDefinitionImpl(this);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.SCOPE;
  }

  public boolean getExitOnStandardFault() {
    return this.exitOnStandardFault;
  }

  public void setExitOnStandardFault(final boolean exitOnStandardFault) {
    this.exitOnStandardFault = exitOnStandardFault;
  }

  public List<VariableDefinition> getVariables() {
    return this.variables;
  }

  public void setVariableDefinition(final List<VariableDefinition> variableDefinitions) {
    this.variables = CopyUtil.copyList(variableDefinitions);
  }

  public List<CorrelationSetDefinition> getCorrelationSets() {
    return this.correlationSets;
  }

  public void setCorrelationSetDefinition(final List<CorrelationSetDefinition> correlationSetDefinitions) {
    this.correlationSets = CopyUtil.copyList(correlationSetDefinitions);
  }

  public List<PartnerLinkDefinition> getPartnerLinks() {
    return this.partnerLinks;
  }

  public void setPartnerLinkDefinition(final List<PartnerLinkDefinition> partnerLinkDefinitions) {
    this.partnerLinks = CopyUtil.copyList(partnerLinkDefinitions);
  }

  public List<String> getMessageExchanges() {
    return this.messageExchanges;
  }

  public void setMessageExchange(final Set<String> messageExchangeList) {
    this.messageExchanges = new ArrayList<String>();
    for (final String messageExchange : messageExchangeList) {
      this.messageExchanges.add(messageExchange);
    }
  }

  public List<BaseCatchFullDefinition< ? >> getFaultHandlers() {
    return this.faultHandlers;
  }

  public void setFaultHandlers(final List<BaseCatchFullDefinition< ? >> faultHandlers) {
    this.faultHandlers = new ArrayList<BaseCatchFullDefinition< ? >>();
    for (final BaseCatchFullDefinition< ? > faultHandler : faultHandlers) {
      this.faultHandlers.add(faultHandler);
    }
  }

  public List<ActivityDefinitionUUID> getFaultHandlerUUIDs() {
    final List<ActivityDefinitionUUID> faultHandlerUUIDs = new ArrayList<ActivityDefinitionUUID>();
    for (final BaseCatchDefinition< ? > faultHandler : this.faultHandlers) {
      faultHandlerUUIDs.add(faultHandler.getUUID());
    }
    return faultHandlerUUIDs;
  }

  public TerminationHandlerFullDefinition getTerminationHandler() {
    return this.terminationHandler;
  }

  public void setTerminationHandler(final TerminationHandlerFullDefinition terminationHandler) {
    this.terminationHandler = terminationHandler;
  }

  public ActivityDefinitionUUID getTerminationHandlerDefinitionUUID() {
    if (this.terminationHandler != null) {
      return this.terminationHandler.getUUID();
    } else {
      return null;
    }
  }

  public CompensationHandlerFullDefinition getCompensationHandler() {
    return this.compensationHandler;
  }

  public void setCompensationHandler(final CompensationHandlerFullDefinition compensationHandler) {
    this.compensationHandler = compensationHandler;
  }

  public ActivityDefinitionUUID getCompensationHandlerDefinitionUUID() {
    if (this.compensationHandler != null) {
      return this.compensationHandler.getUUID();
    } else {
      return null;
    }
  }

  public List<OnEventFullDefinition> getOnEventEventHandlers() {
    return this.onEvents;
  }
  public void setOnEventEventHandlers(final List<OnEventFullDefinition> onEventEventHandlers) {
    this.onEvents = onEventEventHandlers;
  }

  public List<OnAlarmFullDefinition> getOnAlarmEventHandlers() {
    return this.onAlarms;
  }
  public void setOnAlarmEventHandlers(final List<OnAlarmFullDefinition> onAlarmEventHandlers) {
    this.onAlarms = onAlarmEventHandlers;
  }
}
