/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.def.full.impl;

import org.ow2.orchestra.facade.CopyUtil;
import org.ow2.orchestra.facade.def.BpelActivityDefinition;
import org.ow2.orchestra.facade.def.ForDefinition;
import org.ow2.orchestra.facade.def.UntilDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.WaitActivityFullDefinition;
import org.ow2.orchestra.facade.def.impl.WaitActivityDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;

public class WaitActivityFullDefinitionImpl extends BpelActivityFullDefinitionImpl implements WaitActivityFullDefinition {

  private static final long serialVersionUID = 6376991406774071714L;

  private ForDefinition forDefinition;

  private UntilDefinition untilDefinition;

  protected WaitActivityFullDefinitionImpl() {
  }

  public WaitActivityFullDefinitionImpl(final WaitActivityFullDefinition waitActivityFullDefinition) {
    super(waitActivityFullDefinition);
    this.forDefinition = CopyUtil.copy(waitActivityFullDefinition.getForDefinition());
    this.untilDefinition = CopyUtil.copy(waitActivityFullDefinition.getUntilDefinition());

  }

  public WaitActivityFullDefinitionImpl(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name) {
    super(activityDefinitionUUID, processDefinitionUUID, name);
  }

  @Override
  public ActivityType getType() {
    return ActivityType.WAIT;
  }

  public BpelActivityDefinition copy() {
    return new WaitActivityDefinitionImpl(this);
  }

  public BpelActivityFullDefinition fullCopy() {
    return new WaitActivityFullDefinitionImpl(this);
  }

  public ForDefinition getForDefinition() {
    return this.forDefinition;
  }

  public void addForDefinition(final ForDefinition forDefinition) {
    this.forDefinition = CopyUtil.copy(forDefinition);
  }

  public void addUntilDefinition(final UntilDefinition untilDefinition) {
    this.untilDefinition = CopyUtil.copy(untilDefinition);
  }

  public UntilDefinition getUntilDefinition() {
    return this.untilDefinition;
  }

}
