/**
 * Copyright (C) 2007-2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.runtime.full.impl;

import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.util.Misc;

/**
 * @author Gael Lalire, S. Ali Tokmen
 */
public abstract class ActivityFullInstanceImpl extends RuntimeRecordFullImpl implements ActivityFullInstance {

  private static final long serialVersionUID = 8901738568448948365L;

  // for persistence
  protected long dbid;
  protected long dbversion;

  protected ActivityInstanceUUID uuid;
  protected ActivityDefinitionUUID definitionUUID;

  protected BpelFaultException lastException;

  protected ActivityFullInstanceImpl() {
  }

  public ActivityFullInstanceImpl(final ActivityInstanceUUID uuid,
    final ActivityDefinitionUUID definitionUUID, final ProcessDefinitionUUID processUUID,
      final ProcessInstanceUUID instanceUUID) {
    super(processUUID, instanceUUID);
    Misc.checkArgsNotNull(uuid, definitionUUID);
    this.uuid = uuid;
    this.definitionUUID = definitionUUID;
  }

  public ActivityFullInstanceImpl(final ActivityFullInstance activityFullInstance) {
    super(activityFullInstance);
    this.uuid = activityFullInstance.getUUID();
    this.definitionUUID = activityFullInstance.getDefinitionUUID();
    this.lastException = activityFullInstance.getLastException() != null ? activityFullInstance.getLastException().fullCopy() : null;
  }

  @Override
  public final int hashCode() {
    return this.getUUID().hashCode() + super.hashCode();
  }

  /**
   * uuid and class identify an activity so we do not need to override
   * this method.
   */
  @Override
  public final boolean equals(final Object obj) {
    if (obj == null) {
      return false;
    }
    if (!(obj.getClass().equals(this.getClass()))) {
      return false;
    }
    if (!super.equals(obj)) {
      return false;
    }
    if (!this.uuid.equals(((ActivityFullInstanceImpl) obj).getUUID())) {
      return false;
    }
    return true;
  }

  public ActivityInstanceUUID getUUID() {
    return this.uuid;
  }
  public ActivityDefinitionUUID getDefinitionUUID() {
    return this.definitionUUID;
  }

  public BpelFaultException getLastException() {
    return this.lastException;
  }

  public void setLastException(final BpelFaultException lastException) {
    this.lastException = lastException;
  }

  public abstract ActivityType getType();

  public boolean isSingleChildActivity() {
    return false;
  }
  public boolean isChildrenActivity() {
    return false;
  }
}
