/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityWithChildrenFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public abstract class ActivityWithChildrenFullInstanceImpl extends ActivityFullInstanceImpl implements
  ActivityWithChildrenFullInstance {

  private static final long serialVersionUID = 8901738568448948365L;

  /**
   * Warning this field is not persisted so you have to use getter instead of
   * direct reference.
   */
  protected List<ActivityInstanceUUID> enclosedActivitiesUUID;

  protected List<ActivityFullInstance> enclosedActivities;

  protected ActivityWithChildrenFullInstanceImpl() {
  }

  public ActivityWithChildrenFullInstanceImpl(final ActivityInstanceUUID uuid,
    final ActivityDefinitionUUID definitionUUID, final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    this.enclosedActivities = new ArrayList<ActivityFullInstance>();
  }

  public ActivityWithChildrenFullInstanceImpl(final ActivityWithChildrenFullInstance activityFullInstance) {
    super(activityFullInstance);
    this.enclosedActivities = FullCopyUtil.fullCopyList(activityFullInstance.getEnclosedActivities());
  }

  public List<ActivityFullInstance> getEnclosedActivities() {
    return this.enclosedActivities;
  }

  public void addEnclosedActivity(final ActivityFullInstance enclosedActivity) {
    this.enclosedActivities.add(enclosedActivity);
    this.getEnclosedActivitiesUUID().add(enclosedActivity.getUUID());
  }

  public List<ActivityInstanceUUID> getEnclosedActivitiesUUID() {
    if (this.enclosedActivitiesUUID == null) {
      final List<ActivityInstanceUUID> enclosedActivitiesUUID = new ArrayList<ActivityInstanceUUID>();
      for (final ActivityFullInstance enclosedActivity : this.enclosedActivities) {
        enclosedActivitiesUUID.add(enclosedActivity.getUUID());
      }
      this.enclosedActivitiesUUID = enclosedActivitiesUUID;
    }
    return this.enclosedActivitiesUUID;
  }

  @Override
  public boolean isChildrenActivity() {
    return true;
  }
}
