/**
 * Copyright (C) 2007-2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.runtime.full.impl;

import org.ow2.orchestra.facade.FullCopyUtil;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityWithSingleChildFullInstance;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.util.Misc;

/**
 * @author Gael Lalire, S. Ali Tokmen
 */
public abstract class ActivityWithSingleChildFullInstanceImpl extends ActivityFullInstanceImpl implements
  ActivityWithSingleChildFullInstance {

  private static final long serialVersionUID = 8901738568448948365L;

  protected ActivityFullInstance enclosedActivity;

  protected ActivityWithSingleChildFullInstanceImpl() {
  }

  public ActivityWithSingleChildFullInstanceImpl(final ActivityInstanceUUID uuid,
    final ActivityDefinitionUUID definitionUUID, final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
  }

  public ActivityWithSingleChildFullInstanceImpl(final ActivityWithSingleChildFullInstance activityFullInstance) {
    super(activityFullInstance);
    this.enclosedActivity = FullCopyUtil.fullCopy(activityFullInstance.getEnclosedActivity());
  }

  public ActivityFullInstance getEnclosedActivity() {
    return this.enclosedActivity;
  }

  public void setEnclosedActivity(final ActivityFullInstance enclosedActivity) {
    if (this.enclosedActivity != null) {
      Misc.unreachableStatement("This activity " + this.getUUID() + " has already a child activity. Cannot add " + enclosedActivity.getUUID());
    }
    this.enclosedActivity = enclosedActivity;
  }

  public ActivityInstanceUUID getEnclosedActivityUUID() {
    if (this.enclosedActivity != null) {
      return this.enclosedActivity.getUUID();
    }
    return null;
  }

  @Override
  public boolean isSingleChildActivity() {
    return true;
  }
}
