/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.FlowActivityInstance;
import org.ow2.orchestra.facade.runtime.TransitionConditionUpdate;
import org.ow2.orchestra.facade.runtime.full.FlowActivityFullInstance;
import org.ow2.orchestra.facade.runtime.impl.FlowActivityInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class FlowActivityFullInstanceImpl extends ActivityWithChildrenFullInstanceImpl implements
  FlowActivityFullInstance {

  private static final long serialVersionUID = -6173424422154758626L;

  private List<TransitionConditionUpdate> transitionConditionUpdateList;

  protected FlowActivityFullInstanceImpl() {
  }

  public FlowActivityFullInstanceImpl(final FlowActivityFullInstance flowActivityFullInstance) {
    super(flowActivityFullInstance);
    transitionConditionUpdateList = new ArrayList<TransitionConditionUpdate>();
    for (TransitionConditionUpdate transitionConditionUpdate : flowActivityFullInstance.getTransitionConditionUpdateList()) {
      transitionConditionUpdateList.add(new TransitionConditionUpdateFullImpl(transitionConditionUpdate));
    }
  }

  public FlowActivityFullInstanceImpl(final ActivityInstanceUUID uuid,
    final ActivityDefinitionUUID definitionUUID, final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    this.transitionConditionUpdateList = new ArrayList<TransitionConditionUpdate>();
  }

  @Override
  public ActivityType getType() {
    return ActivityType.FLOW;
  }

  public FlowActivityFullInstanceImpl fullCopy() {
    return new FlowActivityFullInstanceImpl(this);
  }

  public FlowActivityInstance copy() {
    return new FlowActivityInstanceImpl(this);
  }

  public List<TransitionConditionUpdate> getTransitionConditionUpdateList() {
    return transitionConditionUpdateList;
  }

}
