/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.Date;

import org.ow2.orchestra.facade.runtime.PartnerLinkUpdate;
import org.ow2.orchestra.facade.runtime.PartnerLinkUpdateType;
import org.ow2.orchestra.facade.runtime.impl.PartnerLinkUpdateImpl;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.w3c.dom.Element;

/**
 *
 * @author Gael Lalire
 */
public class PartnerLinkUpdateFullImpl implements PartnerLinkUpdate {

  private static final long serialVersionUID = -2069580133901045271L;

  // for persistence
  protected long dbid;

  private ActivityInstanceUUID activityUUID;

  private Date date;

  private String name;

  private PartnerLinkUpdateType partnerLinkUpdateType;

  private Element partnerLinkReference;

  protected PartnerLinkUpdateFullImpl() {
  }

  public PartnerLinkUpdateFullImpl(final ActivityInstanceUUID activityUUID, final Date date, final String name,
    final PartnerLinkUpdateType partnerLinkUpdateType, final Element partnerLinkReference) {
    this.activityUUID = activityUUID;
    this.date = new Date(date.getTime());
    this.name = name;
    this.partnerLinkUpdateType = partnerLinkUpdateType;
    if (partnerLinkReference != null) {
      this.partnerLinkReference = (Element) partnerLinkReference.cloneNode(true);
    }
  }

  public PartnerLinkUpdateFullImpl(final PartnerLinkUpdate record) {
    activityUUID = record.getActivityUUID();
    date = new Date(record.getDate().getTime());
    name = record.getName();
    partnerLinkReference = record.getPartnerLinkReference();
    if (partnerLinkReference != null) {
      partnerLinkReference = (Element) partnerLinkReference.cloneNode(true);
    }
    partnerLinkUpdateType = record.getPartnerLinkUpdateType();
  }

  public ActivityInstanceUUID getActivityUUID() {
    return activityUUID;
  }

  public Date getDate() {
    return date;
  }

  public String getName() {
    return name;
  }

  public Element getPartnerLinkReference() {
    return partnerLinkReference;
  }

  public PartnerLinkUpdateType getPartnerLinkUpdateType() {
    return partnerLinkUpdateType;
  }

  public PartnerLinkUpdate copy() {
    return new PartnerLinkUpdateImpl(this);
  }

}
