/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.WaitingActivity;
import org.ow2.orchestra.facade.runtime.full.PickActivityFullInstance;
import org.ow2.orchestra.facade.runtime.impl.PickActivityInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class PickActivityFullInstanceImpl extends ActivityWithSingleChildFullInstanceImpl implements PickActivityFullInstance {

  private static final long serialVersionUID = -6173424422154758626L;

  private List<WaitingActivity> waitingActivityList;

  protected PickActivityFullInstanceImpl() {
  }

  public PickActivityFullInstanceImpl(final PickActivityFullInstance pickActivityFullInstance) {
    super(pickActivityFullInstance);
    waitingActivityList = new ArrayList<WaitingActivity>();
    for (WaitingActivity waitingActivity : pickActivityFullInstance.getWaitingActivityList()) {
      waitingActivityList.add(new WaitingActivityFullImpl(waitingActivity));
    }
  }

  public PickActivityFullInstanceImpl(final ActivityInstanceUUID uuid,
    final ActivityDefinitionUUID definitionUUID, final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    this.waitingActivityList = new ArrayList<WaitingActivity>();
  }

  @Override
  public ActivityType getType() {
    return ActivityType.PICK;
  }

  public PickActivityFullInstanceImpl fullCopy() {
    return new PickActivityFullInstanceImpl(this);
  }

  public PickActivityInstanceImpl copy() {
    return new PickActivityInstanceImpl(this);
  }

  public List<WaitingActivity> getWaitingActivityList() {
    return waitingActivityList;
  }

  public void addWaitingActivity(final WaitingActivity waitingActivity) {
    waitingActivityList.add(waitingActivity);
  }

}
