/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.ProcessInstanceStateUpdate;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ProcessInstanceImpl;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 * Describes a full process instance.
 *
 * @author Alfonso Fuca, Guillaume Renault
 */
public class ProcessFullInstanceImpl extends ScopeActivityFullInstanceImpl implements ProcessFullInstance {

  private static final long serialVersionUID = -7228036760069352986L;

  /**
   * The list of state updates for this instance.
   */
  private List<ProcessInstanceStateUpdate> processInstanceState;


  public ProcessFullInstanceImpl() {
  }

  public ProcessFullInstanceImpl(final ProcessInstanceUUID uuid, final ProcessDefinitionUUID processUUID) {
    super(uuid, processUUID, processUUID, uuid);
    this.processInstanceState = new ArrayList<ProcessInstanceStateUpdate>();
  }

  public ProcessFullInstanceImpl(final ProcessFullInstance record) {
    super(record);
    this.processInstanceState = new ArrayList<ProcessInstanceStateUpdate>();
    for (ProcessInstanceStateUpdate pisu : record.getProcessInstanceStateUpdate()) {
      this.processInstanceState.add(new ProcessInstanceStateUpdateFullImpl(pisu));
    }
  }

  @Override
  public ActivityType getType() {
    return ActivityType.PROCESS;
  }

  @Override
  public ProcessInstanceImpl copy() {
    return new ProcessInstanceImpl(this);
  }

  @Override
  public ProcessFullInstanceImpl fullCopy() {
    return new ProcessFullInstanceImpl(this);
  }

  @Override
  public ProcessInstanceUUID getUUID() {
    return this.instanceUUID;
  }

  /**
   * @return the list of state updates for this instance.
   */
  public List<ProcessInstanceStateUpdate> getProcessInstanceStateUpdate() {
    return this.processInstanceState;
  }

  /**
   * Add a {@link ProcessInstanceStateUpdate} object in the list.
   */
  public void addProcessInstanceState(final ProcessInstanceStateUpdate state) {
    this.processInstanceState.add(state);
    this.activityState = state.getInstanceState();
  }

}
