/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.Date;

import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ProcessInstanceStateUpdate;
import org.ow2.orchestra.facade.runtime.impl.ProcessInstanceStateUpdateImpl;

/**
 * This object describes a full state update of a process instance.
 * 
 * @author Guillaume Renault
 */
public class ProcessInstanceStateUpdateFullImpl implements ProcessInstanceStateUpdate {

  /**
   * Serial Version UUID
   */
  private static final long serialVersionUID = -4799107776721517585L;

  // for persistence
  protected long dbid;

  /**
   * The date at which the update occured.
   */
  private Date date;

  /**
   * The state of the update.
   */
  private ActivityState state;

  /**
   * Empty constructor for hibernate.
   */
  public ProcessInstanceStateUpdateFullImpl() {
  }

  /**
   * Constructor used for the copy.
   * 
   * @param processInstanceStateUpdate the processInstanceStateUpdate to copy.
   */
  public ProcessInstanceStateUpdateFullImpl(final ProcessInstanceStateUpdate processInstanceStateUpdate) {
    this.date = new Date(processInstanceStateUpdate.getDate().getTime());
    this.state = processInstanceStateUpdate.getInstanceState();
  }

  /**
   * Main constructor.
   * 
   * @param date
   *          the date of the update.
   * @param state
   *          the state to associate to this update.
   */
  public ProcessInstanceStateUpdateFullImpl(final Date date, final ActivityState state) {
    this.date = new Date(date.getTime());
    this.state = state;
  }

  /**
   * @return the state of this update
   */
  public ActivityState getInstanceState() {
    return this.state;
  }

  /**
   * @return the date of this update.
   */
  public Date getDate() {
    return this.date;
  }

  /**
   * a copy of the update.
   */
  public ProcessInstanceStateUpdate copy() {
    return new ProcessInstanceStateUpdateImpl(this);
  }

}
