/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.full.ReplyActivityFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ReplyActivityInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Alfonso Fuca
 */
public class ReplyActivityFullInstanceImpl extends ActivityFullInstanceImpl implements ReplyActivityFullInstance {

  private static final long serialVersionUID = 6359042944279300930L;

  protected String operation;

  protected String variable;

  protected String messageExchange;

  protected ReplyActivityFullInstanceImpl() {
  }

  public ReplyActivityFullInstanceImpl(final ActivityInstanceUUID uuid, final ActivityDefinitionUUID definitionUUID,
    final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    this.operation = null;
    this.variable = null;
    this.messageExchange = null;
  }

  public ReplyActivityFullInstanceImpl(final ReplyActivityFullInstance record) {
    super(record);
    this.operation = record.getOperation();
    this.variable = record.getVariable();
    this.messageExchange = record.getMessageExchange();
  }

  public String getMessageExchange() {
    return messageExchange;
  }

  public String getOperation() {
    return operation;
  }

  public String getVariable() {
    return variable;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.REPLY;
  }

  public ReplyActivityFullInstanceImpl fullCopy() {
    return new ReplyActivityFullInstanceImpl(this);
  }

  public ReplyActivityInstanceImpl copy() {
    return new ReplyActivityInstanceImpl(this);
  }

}
