/**
 * Copyright (C) 2008-2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.ArrayList;
import java.util.List;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.CorrelationSetInitialization;
import org.ow2.orchestra.facade.runtime.PartnerLinkUpdate;
import org.ow2.orchestra.facade.runtime.ScopeStateUpdate;
import org.ow2.orchestra.facade.runtime.VariableUpdate;
import org.ow2.orchestra.facade.runtime.full.ActivityWithSingleChildFullInstance;
import org.ow2.orchestra.facade.runtime.full.CompensationHandlerActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ScopeActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.TerminationHandlerActivityFullInstance;
import org.ow2.orchestra.facade.runtime.impl.ScopeActivityInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.util.Misc;

/**
 * @author Alfonso Fuca, S. Ali Tokmen
 */
public class ScopeActivityFullInstanceImpl extends ActivityWithSingleChildFullInstanceImpl implements
  ScopeActivityFullInstance {

  private static final long serialVersionUID = 62610068746337790L;

  /** all runtime variables of this scope. */
  protected List<VariableUpdate> variables = null;

  protected List<CorrelationSetInitialization> correlationSet = null;

  protected List<PartnerLinkUpdate> partnerLink = null;

  protected List<ScopeStateUpdate> scopeState = null;

  protected List<ScopeActivityFullInstance> eventInstances = null;

  protected ActivityWithSingleChildFullInstance faultHandlerActivity = null;

  protected TerminationHandlerActivityFullInstance terminationHandlerActivity = null;

  protected CompensationHandlerActivityFullInstance compensationHandlerActivity = null;

  protected ScopeActivityFullInstanceImpl() {
  }

  public ScopeActivityFullInstanceImpl(final ActivityInstanceUUID uuid, final ActivityDefinitionUUID definitionUUID,
    final ProcessDefinitionUUID processUUID, final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    this.variables = new ArrayList<VariableUpdate>();
    this.correlationSet = new ArrayList<CorrelationSetInitialization>();
    this.partnerLink = new ArrayList<PartnerLinkUpdate>();
    this.scopeState = new ArrayList<ScopeStateUpdate>();
    this.eventInstances = new ArrayList<ScopeActivityFullInstance>();
  }

  public ScopeActivityFullInstanceImpl(final ScopeActivityFullInstance scopeActivityFullInstance) {
    super(scopeActivityFullInstance);
    this.variables = new ArrayList<VariableUpdate>();
    for (final VariableUpdate variable : scopeActivityFullInstance.getVariables()) {
      this.variables.add(new VariableUpdateFullImpl(variable));
    }
    this.correlationSet = new ArrayList<CorrelationSetInitialization>();
    for (final CorrelationSetInitialization correlationSetInitialization : scopeActivityFullInstance.getCorrelationSet()) {
      this.correlationSet.add(new CorrelationSetInitializationFullImpl(correlationSetInitialization));
    }
    this.partnerLink = new ArrayList<PartnerLinkUpdate>();
    for (final PartnerLinkUpdate partnerLinkUpdate : scopeActivityFullInstance.getPartnerLink()) {
      this.partnerLink.add(new PartnerLinkUpdateFullImpl(partnerLinkUpdate));
    }
    this.scopeState = new ArrayList<ScopeStateUpdate>();
    for (final ScopeStateUpdate scopeStateUpdate : scopeActivityFullInstance.getScopeState()) {
      this.scopeState.add(new ScopeStateUpdateFullImpl(scopeStateUpdate));
    }
    if (scopeActivityFullInstance.getFaultHandlerActivityInstance() != null) {
      final ActivityWithSingleChildFullInstance fhActivityInstance = scopeActivityFullInstance.getFaultHandlerActivityInstance();
      this.faultHandlerActivity = (ActivityWithSingleChildFullInstance) fhActivityInstance.fullCopy();
    }
    if (scopeActivityFullInstance.getTerminationHandlerActivityInstance() != null) {
      final TerminationHandlerActivityFullInstance thActivityInstance = scopeActivityFullInstance
        .getTerminationHandlerActivityInstance();
      this.terminationHandlerActivity = (TerminationHandlerActivityFullInstance) thActivityInstance.fullCopy();
    }
    if (scopeActivityFullInstance.getCompensationHandlerActivityInstance() != null) {
      final CompensationHandlerActivityFullInstance chActivityInstance = scopeActivityFullInstance
        .getCompensationHandlerActivityInstance();
      this.compensationHandlerActivity = (CompensationHandlerActivityFullInstance) chActivityInstance.fullCopy();
    }
    this.eventInstances = new ArrayList<ScopeActivityFullInstance>();
    for (final ScopeActivityFullInstance eventInstance : scopeActivityFullInstance.getEventInstances()) {
      this.eventInstances.add((ScopeActivityFullInstance) eventInstance.fullCopy());
    }
  }

  public List<VariableUpdate> getVariables() {
    return this.variables;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.SCOPE;
  }

  public ScopeActivityFullInstanceImpl fullCopy() {
    return new ScopeActivityFullInstanceImpl(this);
  }

  public ScopeActivityInstanceImpl copy() {
    return new ScopeActivityInstanceImpl(this);
  }

  public List<CorrelationSetInitialization> getCorrelationSet() {
    return this.correlationSet;
  }

  public List<PartnerLinkUpdate> getPartnerLink() {
    return this.partnerLink;
  }

  public List<ScopeStateUpdate> getScopeState() {
    return this.scopeState;
  }

  public ActivityInstanceUUID getFaultHandlerActivityUUID() {
    return this.faultHandlerActivity != null ? this.faultHandlerActivity.getUUID() : null;
  }

  public void setFaultHandlerActivityInstance(final ActivityWithSingleChildFullInstance catchActivity) {
    if (this.faultHandlerActivity != null) {
      Misc.unreachableStatement("Fault handler can only be executed once");
    }
    this.faultHandlerActivity = catchActivity;
  }

  public ActivityInstanceUUID getTerminationHandlerActivityUUID() {
    return this.terminationHandlerActivity != null ? this.terminationHandlerActivity.getUUID() : null;
  }

  public void setTerminationHandlerActivityInstance(final TerminationHandlerActivityFullInstance terminationActivity) {
    if (this.terminationHandlerActivity != null) {
      Misc.unreachableStatement("Termination handler can only be executed once");
    }
    this.terminationHandlerActivity = terminationActivity;
  }

  public ActivityInstanceUUID getCompensationHandlerActivityUUID() {
    return this.compensationHandlerActivity != null ? this.compensationHandlerActivity.getUUID() : null;
  }

  public void setCompensationHandlerActivityInstance(final CompensationHandlerActivityFullInstance compensationActivity) {
    if (this.compensationHandlerActivity != null) {
      Misc.unreachableStatement("Compensation handler can only be executed once");
    }
    this.compensationHandlerActivity = compensationActivity;
  }

  public ActivityWithSingleChildFullInstance getFaultHandlerActivityInstance() {
    return this.faultHandlerActivity;
  }

  public CompensationHandlerActivityFullInstance getCompensationHandlerActivityInstance() {
    return this.compensationHandlerActivity;
  }

  public TerminationHandlerActivityFullInstance getTerminationHandlerActivityInstance() {
    return this.terminationHandlerActivity;
  }

  public List<ScopeActivityFullInstance> getEventInstances() {
    return this.eventInstances;
  }

  public List<ActivityInstanceUUID> getEventInstanceUUIDs() {
    if (this.eventInstances == null) {
      return null;
    }
    final List<ActivityInstanceUUID> uuids = new ArrayList<ActivityInstanceUUID>();
    for (final ScopeActivityFullInstance i : this.eventInstances) {
      uuids.add(i.getUUID());
    }
    return uuids;
  }
  public void addEventInstance(final ScopeActivityFullInstance eventInstance) {
    this.eventInstances.add(eventInstance);
  }
}
