/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import java.util.Date;

import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.facade.runtime.ScopeStateUpdate;
import org.ow2.orchestra.facade.runtime.impl.ScopeStateUpdateImpl;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;

/**
 * @author S. Ali Tokmen
 */
public class ScopeStateUpdateFullImpl implements ScopeStateUpdate {

  private static final long serialVersionUID = -9150489690593766584L;

  // for persistence
  protected long dbid;

  private ActivityInstanceUUID activityUUID;

  private Date date;

  private ScopeState state;

  protected ScopeStateUpdateFullImpl() {
  }

  public ScopeStateUpdateFullImpl(final ScopeStateUpdate record) {
    activityUUID = record.getActivityUUID();
    date = new Date(record.getDate().getTime());
    state = record.getState();
  }

  public ScopeStateUpdateFullImpl(final ActivityInstanceUUID activityUUID, final Date date, final ScopeState state) {
    this.activityUUID = activityUUID;
    this.date = new Date(date.getTime());
    this.state = state;
  }

  public ActivityInstanceUUID getActivityUUID() {
    return activityUUID;
  }

  public Date getDate() {
    return date;
  }

  public ScopeState getState() {
    return state;
  }

  @Override
  public String toString() {
    return "Update scope state at " + date + " by " + activityUUID + " to [" + getState() + "]";
  }

  public ScopeStateUpdate copy() {
    return new ScopeStateUpdateImpl(this);
  }

}
