/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.facade.runtime.full.impl;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.WaitingActivity;
import org.ow2.orchestra.facade.runtime.full.WaitActivityFullInstance;
import org.ow2.orchestra.facade.runtime.impl.WaitActivityInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;

/**
 *
 * @author Gael Lalire
 */
public class WaitActivityFullInstanceImpl extends ActivityFullInstanceImpl implements WaitActivityFullInstance {

  private static final long serialVersionUID = -6173424422154758626L;

  private WaitingActivity waitingActivity;

  protected WaitActivityFullInstanceImpl() {
  }

  public WaitActivityFullInstanceImpl(final WaitActivityFullInstance waitActivityFullInstance) {
    super(waitActivityFullInstance);
    waitingActivity = waitActivityFullInstance.getWaitingActivity();
    if (waitingActivity != null) {
      waitingActivity = new WaitingActivityFullImpl(waitingActivity);
    }
  }

  public WaitActivityFullInstanceImpl(final ActivityInstanceUUID uuid, final ActivityDefinitionUUID definitionUUID,
    final ProcessDefinitionUUID processUUID,
    final ProcessInstanceUUID instanceUUID) {
    super(uuid, definitionUUID, processUUID, instanceUUID);
    waitingActivity = null;
  }

  @Override
  public ActivityType getType() {
    return ActivityType.WAIT;
  }

  public WaitActivityFullInstanceImpl fullCopy() {
    return new WaitActivityFullInstanceImpl(this);
  }

  public WaitActivityInstanceImpl copy() {
    return new WaitActivityInstanceImpl(this);
  }

  public void setWaitingActivity(final WaitingActivity timerData) {
    waitingActivity = timerData;
  }

  public WaitingActivity getWaitingActivity() {
    return waitingActivity;
  }

}
