/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.facade.uuid;

import java.util.UUID;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.util.Misc;


public abstract class IdFactory {

  private IdFactory() {

  }

  public static ProcessInstanceUUID getNewInstanceUUID() {
    final ProcessInstanceUUID processInstanceUUID = new ProcessInstanceUUID(Misc.getUniqueId("instance"));
    return processInstanceUUID;
  }

  public static ProcessDefinitionUUID getNewProcessUUID(final String targetNamespace, final String name) {
    final QName qName = new QName(targetNamespace, name);
    final ProcessDefinitionUUID processDefinitionUUID = new ProcessDefinitionUUID(qName, UUID.randomUUID().toString());
    return processDefinitionUUID;
  }

  public static ActivityInstanceUUID getNewActivityUUID() {
    final ActivityInstanceUUID activityDefinitionUUID = new ActivityInstanceUUID(Misc.getUniqueId("activity"));
    return activityDefinitionUUID;
  }

  public static ActivityDefinitionUUID getNewActivityDefUUID(final ProcessDefinitionUUID processUUID, final int activityCount) {
    final ActivityDefinitionUUID activityDefinitionUUID = new ActivityDefinitionUUID(processUUID, activityCount, ActivityType.UNKNOWN_ACTIVITY, null);
    return activityDefinitionUUID;
  }
}
