/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.jmx;

import java.lang.management.ManagementFactory;
import java.util.Properties;
import java.util.logging.Level;

import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.remote.JMXConnectorServer;
import javax.management.remote.JMXConnectorServerFactory;
import javax.management.remote.JMXServiceURL;

import org.ow2.orchestra.env.EnvFactoryRepository;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.facade.jmx.RemoteDeployer;
import org.ow2.orchestra.facade.jmx.RemoteDeployerMBean;
import org.ow2.orchestra.pvm.env.EnvironmentFactory;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobExecutor;
import org.ow2.orchestra.util.Misc;
import org.ow2.orchestra.util.OrchestraConstants;

public class JMXAgent {

  private MBeanServer mbs;

  private ObjectName objectName;

  private JMXConnectorServer cs;

  public void startOrchestra(final EnvironmentFactory envFactory) {

    EnvFactoryRepository.set(envFactory);

    final JobExecutor jobExecutor = envFactory.get(JobExecutor.class);
    if (jobExecutor != null) {
      jobExecutor.start();
    }
    // create api object
    final RemoteDeployerMBean remoteDeployer = new RemoteDeployer();

    // reinitialize orchestra
    remoteDeployer.initialize();

    // Orchestra properties
    final Properties orchestraProperties = (Properties) envFactory.get(OrchestraConstants.ORCHESTRA_PROPERTIES);

    // expose api as MBean
    try {
      // start rmiregistry
      final int jmxPort = Integer.parseInt(orchestraProperties.getProperty(OrchestraConstants.JMX_PORT));
      java.rmi.registry.LocateRegistry.createRegistry(jmxPort);

      this.mbs = ManagementFactory.getPlatformMBeanServer();

      // Uniquely identify the MBeans and register them with the MBeanServer
      this.objectName = ObjectName.getInstance(orchestraProperties.getProperty(OrchestraConstants.JMX_OBJECT_NAME));
      this.mbs.registerMBean(remoteDeployer, this.objectName);
      final JMXServiceURL url = new JMXServiceURL(orchestraProperties.getProperty(OrchestraConstants.JMX_SERVICE_URL));
      this.cs = JMXConnectorServerFactory.newJMXConnectorServer(url, null, this.mbs);
      this.cs.start();

      Misc.log(Level.INFO, "Orchestra is started using JMX connector [" + url.toString() + "]");
    } catch (final Exception e) {
      throw new OrchestraRuntimeException("Problem registering mbean in mbean server",
          e);
    }


  }

  public void stopOrchestra() {
    if (this.mbs != null) {
      try {
        this.cs.stop();
        this.mbs.unregisterMBean(this.objectName);
      } catch (final Exception e) {
        throw new OrchestraRuntimeException("Problem unregistering mbean", e);
      }
    }
    if (EnvFactoryRepository.get() != null) {
      final JobExecutor jobExecutor = EnvFactoryRepository.get().get(JobExecutor.class);
      if (jobExecutor != null) {
        jobExecutor.stop();
      }
      EnvFactoryRepository.get().close();
      EnvFactoryRepository.set(null);
    }
  };

}
