/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package org.ow2.orchestra.lang.jaxen;

import java.util.Iterator;

import org.jaxen.Context;
import org.jaxen.ContextSupport;
import org.jaxen.FunctionContext;
import org.jaxen.JaxenException;
import org.jaxen.SimpleVariableContext;
import org.ow2.orchestra.definition.activity.AbstractBpelActivity;
import org.ow2.orchestra.definition.element.Target;
import org.ow2.orchestra.lang.Snippet;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * Evaluator for expressions in join conditions. Only link status can be used
 * within join conditions and only join conditions can make use of link status.
 *
 * @author Juan Cantu
 * @version $Revision: 1.6 $ $Date: 2007/07/22 05:57:25 $
 */
class XPathJoinConditionEvaluator extends XPathExpressionEvaluator {

  /**
   * The jBPM configuration object name for the resource describing the function
   * library available to join conditions.
   */

  private static final long serialVersionUID = 1L;

  private static final FunctionContext FUNTION_LIBRARY = readFunctionLibrary();

  XPathJoinConditionEvaluator(String text) throws JaxenException {
    super(text);
  }

  protected FunctionContext createFunctionContext() {
    return FUNTION_LIBRARY;
  }

  /**
   * Gets a context for expression evaluation. The context for a join condition
   * is:
   * <ul>
   * <li>the variable bindings are the links targeting the current node of the
   * given token</li>
   * <li>the function library is read from resource comes from
   * {@link #createFunctionContext()}</li>
   * <li>the namespace declarations come from {@link Snippet#getNamespaces()}</li>
   * </ul>
   */
  protected Context getContext(Object o) {
    if (!(o instanceof BpelExecution)) {
      return super.getContext(o);
    }
    BpelExecution bpelExecution = (BpelExecution) o;
    AbstractBpelActivity activity = (AbstractBpelActivity) bpelExecution.getNode()
        .getBehaviour();
    SimpleVariableContext variableContext = new SimpleVariableContext();

    // set status of target links in the variable context
    for (Iterator<Target> i = activity.getTargets().iterator(); i.hasNext();) {
      Target target = i.next();
      Boolean status = bpelExecution.getFlowRuntimeWithLink(
          target.getLinkName()).getLinkStatus(target.getLinkName());
      variableContext.setVariableValue(target.getLinkName(), status);
    }

    ContextSupport support = new ContextSupport(getNamespaceContext(),
        getFunctionContext(), variableContext, getNavigator());
    return new Context(support);
  }
}
