/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the JBPM BPEL PUBLIC LICENSE AGREEMENT as
 * published by JBoss Inc.; either version 1.0 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
package org.ow2.orchestra.lang.jaxen;

import java.util.Collections;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.jaxen.Context;
import org.jaxen.ContextSupport;
import org.jaxen.FunctionContext;
import org.jaxen.JaxenException;
import org.jaxen.expr.Expr;
import org.jaxen.expr.LocationPath;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.exception.BpelFaultException.BpelFault;
import org.ow2.orchestra.lang.evaluator.VariableQueryEvaluator;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.util.BpelXmlUtil;
import org.w3c.dom.Node;


/**
 * Default evaluator for variable queries.
 * @author Alejandro Guizar
 * @version $Revision: 1.1 $ $Date: 2007/09/12 23:20:16 $
 */
class XPathVariableQueryEvaluator extends XPathEvaluator implements VariableQueryEvaluator {

  /**
   * The jBPM configuration object name for the resource describing the function library available
   * to variable queries.
   */

  private static final long serialVersionUID = 1L;

  private static final Logger LOG = Logger.getLogger(XPathVariableQueryEvaluator.class.getName());

  private static final FunctionContext FUNCTION_LIBRARY = XPathEvaluator.readFunctionLibrary();

  XPathVariableQueryEvaluator(final String text) throws JaxenException {
    super(text);
  }

  public Object evaluate(final Node contextNode, final BpelExecution bpelExecution) {
    final Context context = this.getContext(contextNode, bpelExecution);
    try {
      final List nodeSet = this.selectNodes(context);
      return this.narrowToSingleNode(nodeSet);
    } catch (final JaxenException e) {
      XPathVariableQueryEvaluator.LOG.log(Level.SEVERE, "query evaluation failed", e);
      throw BpelFaultException.generateBpelFaultException(BpelFault.subLanguageExecutionFault);
    }
  }

  public void assign(final Node contextNode, final BpelExecution bpelExecution, final Object value, final boolean keepSrcEltName) {
    final Expr expr = this.getRootExpr();
    final Context context = this.getContext(contextNode, bpelExecution);
    try {
      final List nodeSet = expr instanceof LocationPath ? this.selectOrCreateNodes((LocationPath) expr,
          context) : this.selectNodes(context);
      BpelXmlUtil.setObjectValue(this.narrowToSingleNode(nodeSet), value, keepSrcEltName);
    } catch (final JaxenException e) {
      XPathVariableQueryEvaluator.LOG.log(Level.SEVERE, "query assignment failed", e);
      throw BpelFaultException.generateBpelFaultException(BpelFault.subLanguageExecutionFault);
    }
  }

  @Override
  protected FunctionContext createFunctionContext() {
    return XPathVariableQueryEvaluator.FUNCTION_LIBRARY;
  }

  private Context getContext(final Node contextNode, final BpelExecution bpelExecution) {
    final ContextSupport support = this.getContextSupport();
    support.setVariableContext(new BpelVariableContext(bpelExecution));

    final Context context = new Context(support);
    context.setNodeSet(Collections.singletonList(contextNode));
    return context;
  }
}
