/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.Empty;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.EmptyActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.w3c.dom.Element;

/**
 * @author Charles Souillard
 */

public class EmptyBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(EmptyBinding.class.getName());


  public EmptyBinding() {
    super("empty", ActivityType.EMPTY);
  }
  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element
   *            dom element to parse
   * @param parse
   *            parse
   * @param parser
   *            parser
   * @return the build behaviour!
   */
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Empty empty = new Empty();
    setEnclosingScope(empty, parse);
    parseStandardAttributes(element, empty, parse);
    parseStandardElements(element, empty, parse);
    NodeImpl activityNode = createActivityNode(element, parse, parser, empty);
    return activityNode;
  }

  @Override
  public BpelActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new EmptyActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
