/**
 * Copyright (C) 2006-2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.Exit;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.ExitActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.w3c.dom.Element;

/**
 * @author S. Ali Tokmen
 */
public class ExitBinding extends ActivityBinding {

  /**
   * logger.
   */
  private static Logger log = Logger.getLogger(ExitBinding.class.getName());

  public ExitBinding() {
    super("exit", ActivityType.EXIT);
  }

  /**
   * Parses the given dom element to build a Node and the Exit behavior
   *
   * @param element dom element to parse
   * @param parse parse
   * @param parser parser
   * @return the Exit behavior
   */
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Exit exit = new Exit();
    setEnclosingScope(exit, parse);
    parseStandardAttributes(element, exit, parse);
    parseStandardElements(element, exit, parse);
    NodeImpl activityNode = createActivityNode(element, parse, parser, exit);
    return activityNode;
  }

  @Override
  public BpelActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new ExitActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
