/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.extensions.Test;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.ow2.orchestra.util.XmlConstants;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Element;

public class ExtensionActivityBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(ExtensionActivityBinding.class
      .getName());


  public ExtensionActivityBinding() {
    super("extensionActivity", ActivityType.EXTENSION_ACTIVITY);
  }
  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element
   *            dom element to parse
   * @param parse
   *            parse
   * @param parser
   *            parser
   * @return the build behaviour!
   */
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (ExtensionActivityBinding.log.isLoggable(Level.FINE)) {
      ExtensionActivityBinding.log.fine("parsing element = " + element);
    }
    final List<Element> childrenElements = XmlUtil.elements(element);
    if (childrenElements.size() > 1) {
      throw new OrchestraRuntimeException(
          "Only one Element is expected a s a child of extensionActivity");
    }
    final Element extensionActivityToParse = childrenElements.get(0);

    if (XmlConstants.ORCHESTRA_NS.equals(extensionActivityToParse
        .getNamespaceURI())
        && "test".equals(extensionActivityToParse.getLocalName())) {
      return this.parseTest(extensionActivityToParse, parse, parser);
    }
    parse.addProblem("Unknown ExtensionActivity : " + element.getNamespaceURI()
        + ":" + element.getLocalName());
    return null;
  }

  private Object parseTest(final Element element, final Parse parse, final Parser parser) {
    final Test testVarObject = new Test();
    final org.ow2.orchestra.pvm.model.Node testVarNode = this.createActivityNode(element, parse, parser,
        testVarObject);

    this.setEnclosingScope(testVarObject, parse);
    this.parseStandardAttributes(element, testVarObject, parse);
    this.parseStandardElements(element, testVarObject, parse);

    parse.pushObject(testVarNode);
    parse.pushObject(testVarObject);
    try {
      final String varNamesAttr = XmlUtil.attribute(element, "varNames");
      if (varNamesAttr != null) {
        final List<String> varNames = new ArrayList<String>();
        for (final String s : varNamesAttr.split(",")) {
          varNames.add(s.trim());
        }
        testVarObject.setVarNames(varNames);
      }

      final String plnkNamesAttr = XmlUtil.attribute(element, "plnkNames");
      if (plnkNamesAttr != null) {
        final List<String> plnkNames = new ArrayList<String>();
        for (final String s : plnkNamesAttr.split(",")) {
          plnkNames.add(s.trim());
        }
        testVarObject.setPlnkNames(plnkNames);
      }

      final String event = XmlUtil.attribute(element, "event");
      testVarObject.setEvent(event);

    } finally {
      parse.popObject();
      parse.popObject();
    }
    return testVarNode;
  }

}
