/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.RepeatUntil;
import org.ow2.orchestra.facade.def.full.RepeatUntilActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.RepeatUntilActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Element;

public class RepeatUntilBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(IfBinding.class.getName());

  public RepeatUntilBinding() {
    super("repeatUntil", ActivityType.REPEAT_UNTIL);
  }

  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    RepeatUntil repeatUntil = new RepeatUntil();
    org.ow2.orchestra.pvm.model.Node repeatUntilNode = createActivityNode(element, parse,
        parser, repeatUntil);

    setEnclosingScope(repeatUntil, parse);
    parseStandardAttributes(element, repeatUntil, parse);
    parseStandardElements(element, repeatUntil, parse);

    parse.pushObject(repeatUntilNode);
    parse.pushObject(repeatUntil);
    try {
      repeatUntil.setCondition(parseCondition(element, parse));
      parseChildActivity(element, parse, parser);
    } finally {
      parse.popObject();
      parse.popObject();
    }
    return repeatUntilNode;
  }

  private Expression parseCondition(final Element parentElement, final Parse parse) {
    Element conditionElement = XmlUtil.element(parentElement, "condition");
    if (conditionElement != null) {
      BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
      String expressionLanguage = XmlUtil.attribute(conditionElement,
          "expressionLanguage");
      if (expressionLanguage == null) {
        expressionLanguage = bpelProcess.getExpressionLanguage();
      }
      Expression expression = new Expression();
      expression.setLanguage(expressionLanguage);
      expression.setNamespaces(BpelUtil.getAllNameSpaces(conditionElement));
      String textContent = conditionElement.getTextContent();
      expression.setText(textContent);
      DefinitionKeeper definitionKeeper = parse.findObject(DefinitionKeeper.class);
      RepeatUntilActivityFullDefinition activityFullDefinition = (RepeatUntilActivityFullDefinition) definitionKeeper
        .getActivityFullDefinition();
      activityFullDefinition.setConditionBooleanExpression(textContent);
      activityFullDefinition.setConditionExpressionLanguage(expressionLanguage);
      return expression;
    }
    return null;
  }

  @Override
  public RepeatUntilActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new RepeatUntilActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
