/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.Rethrow;
import org.ow2.orchestra.exception.StaticAnalysisException;
import org.ow2.orchestra.facade.def.ActivityDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.RethrowActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.ow2.orchestra.pvm.model.Node;
import org.ow2.orchestra.util.StaticAnalysisFault;
import org.w3c.dom.Element;

/**
 * @author S. Ali Tokmen
 */
public class RethrowBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(RethrowBinding.class.getName());

  public RethrowBinding() {
    super("rethrow", ActivityType.RETHROW);
  }

  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element dom element to parse
   * @param parse parse
   * @param parser parser
   * @return the build behaviour!
   */
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }

    Rethrow rethrow = new Rethrow();
    setEnclosingScope(rethrow, parse);
    parseStandardAttributes(element, rethrow, parse);
    parseStandardElements(element, rethrow, parse);

    Node activityNode = createActivityNode(element, parse, parser, rethrow);

    // Static Analysis SA00006
    boolean foundCatchOrCatchAll = false;
    final DefinitionKeeper definitionKeeper = parse.findObject(DefinitionKeeper.class);
    for (Object parent : definitionKeeper.getParentsStack()) {
      ActivityDefinition parentActivity = (ActivityDefinition) parent;
      ActivityType parentActivityType = parentActivity.getType();
      if (parentActivityType == ActivityType.CATCH_HANDLER || parentActivityType == ActivityType.CATCHALL_HANDLER) {
        foundCatchOrCatchAll = true;
        break;
      }
    }
    if (!foundCatchOrCatchAll) {
      final String message = StaticAnalysisFault.getMessage("SA00006", "  The <rethrow> element \"" + rethrow.getName()
        + "\" is not in a <catch> or <catchAll> element.");
      parse.addProblem(message, new StaticAnalysisException("SA00006"));
      return null;
    }

    return activityNode;
  }

  @Override
  public BpelActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new RethrowActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
