package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.Sequence;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.SequenceActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.w3c.dom.Element;

/**
 * @author Tom Baeyens, Charles Souillard
 */
public class SequenceBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(SequenceBinding.class.getName());


  public SequenceBinding() {
    super("sequence", ActivityType.SEQUENCE);
  }

  /**
   * Parses the given dom element to build a Node and a behaviour.
   *
   * @param element
   *            dom element to parse
   * @param parse
   *            parse
   * @param parser
   *            parser
   * @return the build behaviour!
   */
  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    Sequence sequence = new Sequence();
    setEnclosingScope(sequence, parse);
    parseStandardAttributes(element, sequence, parse);
    parseStandardElements(element, sequence, parse);

    NodeImpl activityNode = createActivityNodeWithChildren(element, parse,
        parser, sequence);
    activityNode.setPreviousNeeded(true);

    return activityNode;
  }

  @Override
  public BpelActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    // TODO Auto-generated method stub
    return new SequenceActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }
}
