/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.Wait;
import org.ow2.orchestra.facade.def.ForDefinition;
import org.ow2.orchestra.facade.def.UntilDefinition;
import org.ow2.orchestra.facade.def.full.ActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.WaitActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.WaitActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.ForDefinitionImpl;
import org.ow2.orchestra.facade.def.impl.UntilDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Element;

/**
 * @author Pascal Verdage
 */
public class WaitBinding extends ActivityBinding {

  private static Logger log = Logger.getLogger(WaitBinding.class.getName());

  public WaitBinding() {
    super("wait", ActivityType.WAIT);
  }

  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (WaitBinding.log.isLoggable(Level.FINE)) {
      WaitBinding.log.fine("parsing element = " + element);
    }
    final Wait waitObject = new Wait();
    final NodeImpl waitNode = this.createActivityNode(element, parse, parser, waitObject);
//    waitNode.setLocalScope(true);

    this.setEnclosingScope(waitObject, parse);
    this.parseStandardAttributes(element, waitObject, parse);

    final DefinitionKeeper definitionKeeper = parse.findObject(DefinitionKeeper.class);
    final ActivityFullDefinition activityFullDefinition = definitionKeeper.getActivityFullDefinition();
    ForDefinition forDefinition = new ForDefinitionImpl();
    UntilDefinition untilDefinition = new UntilDefinitionImpl();

    this.parseStandardElements(element, waitObject, parse);

    final BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
    final Element deadlineElement = XmlUtil.element(element, "until");
    final Element durationElement = XmlUtil.element(element, "for");
    if (durationElement != null && deadlineElement != null) {
      parse.addProblem("Error in element: " + element + " only one element 'for' or 'until' should be used.");
    }
    if (durationElement != null) {
      final Expression expression = PickBinding.parseExpression(durationElement, bpelProcess);
      waitObject.setDurationExpression(expression);

      // Definition
      forDefinition = new ForDefinitionImpl(expression.getLanguage(), expression.getText());
    } else if (deadlineElement != null) {
      final Expression expression = PickBinding.parseExpression(deadlineElement, bpelProcess);
      waitObject.setDeadlineExpression(expression);

      // Definition
      untilDefinition = new UntilDefinitionImpl(expression.getLanguage(), expression.getText());
    } else {
      parse.addProblem("Error in element: " + element + " one element 'for' or 'until' should be used.");
    }

    ((WaitActivityFullDefinition) activityFullDefinition).addForDefinition(forDefinition);
    ((WaitActivityFullDefinition) activityFullDefinition).addUntilDefinition(untilDefinition);

    return waitNode;
  }
  @Override
  public BpelActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new WaitActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
