/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.parsing.binding;

import java.util.logging.Level;
import java.util.logging.Logger;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.While;
import org.ow2.orchestra.facade.def.full.BpelActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.WhileActivityFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.WhileActivityFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.lang.Expression;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.xml.Parse;
import org.ow2.orchestra.pvm.internal.xml.Parser;
import org.ow2.orchestra.util.BpelUtil;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Element;

public class WhileBinding extends ActivityBinding {

  /** log. */
  private static Logger log = Logger.getLogger(IfBinding.class.getName());


  public WhileBinding() {
    super("while", ActivityType.WHILE);
  }

  public Object parse(final Element element, final Parse parse, final Parser parser) {
    if (log.isLoggable(Level.FINE)) {
      log.fine("parsing element = " + element);
    }
    While whileObject = new While();
    NodeImpl whileNode = createActivityNode(element, parse, parser,
        whileObject);

    setEnclosingScope(whileObject, parse);
    parseStandardAttributes(element, whileObject, parse);
    parseStandardElements(element, whileObject, parse);

    parse.pushObject(whileNode);
    parse.pushObject(whileObject);
    try {
      whileObject.setCondition(parseCondition(element, parse));
      parseChildActivity(element, parse, parser);
    } finally {
      parse.popObject();
      parse.popObject();
    }

    return whileNode;
  }

  private Expression parseCondition(final Element parentElement, final Parse parse) {
    Element conditionElement = XmlUtil.element(parentElement, "condition");
    if (conditionElement != null) {
      BpelProcess bpelProcess = parse.findObject(BpelProcess.class);
      String expressionLanguage = XmlUtil.attribute(conditionElement,
          "expressionLanguage");
      if (expressionLanguage == null) {
        expressionLanguage = bpelProcess.getExpressionLanguage();
      }
      Expression expression = new Expression();
      expression.setLanguage(expressionLanguage);
      expression.setNamespaces(BpelUtil.getAllNameSpaces(conditionElement));
      String textContent = conditionElement.getTextContent();
      expression.setText(textContent);
      DefinitionKeeper definitionKeeper = parse.findObject(DefinitionKeeper.class);
      WhileActivityFullDefinition activityFullDefinition = (WhileActivityFullDefinition) definitionKeeper
        .getActivityFullDefinition();
      activityFullDefinition.setConditionBooleanExpression(textContent);
      activityFullDefinition.setConditionExpressionLanguage(expressionLanguage);
      return expression;
    }
    return null;
  }

  @Override
  public BpelActivityFullDefinition createActivityFullDefinition(final ProcessDefinitionUUID processDefinitionUUID,
    final ActivityDefinitionUUID activityDefinitionUUID, final String name, final Parse parse) {
    return new WhileActivityFullDefinitionImpl(processDefinitionUUID, activityDefinitionUUID, name);
  }

}
