package org.ow2.orchestra.persistence.db;

import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.def.full.impl.ProcessFullDefinitionImpl;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.full.impl.ProcessFullInstanceImpl;
import org.ow2.orchestra.persistence.QuerierDbSession;
import org.ow2.orchestra.services.Archivable;
import org.ow2.orchestra.services.Archiver;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.Misc;

/**
 *
 * @author Alfonso Fuca
 *
 */
public class DbHistory extends AbstractDbQuerier implements Archiver {

  public DbHistory(final QuerierDbSession dbSession) {
    super(dbSession);
  }

  public void archive(final ProcessFullInstance processFullInstance) {
    Misc.checkArgsNotNull(processFullInstance);
    if (this.getProcessInstance(processFullInstance.getUUID()) == null) {
      this.getQuerierDbSession().save(new ProcessFullInstanceImpl(processFullInstance));
    } else {
      throw new IllegalArgumentException(" Can't archive: " + processFullInstance + " --- A record with processInstanceUUID: "
        + processFullInstance.getUUID() + " has already been archived ");
    }
  }

  public void remove(final ProcessFullInstance processFullInstance) {
    Misc.checkArgsNotNull(processFullInstance);
    if (EnvTool.getHistoryQueriers().getProcessInstance(processFullInstance.getUUID()) != null) {
      this.getQuerierDbSession().delete(new ProcessFullInstanceImpl(processFullInstance));
    }
  }

  public void archive(final ProcessFullDefinition processFullDefinition) {
    Misc.checkArgsNotNull(processFullDefinition);
    if (this.getProcessDefinition(processFullDefinition.getUUID()) == null) {
      this.getQuerierDbSession().save(new ProcessFullDefinitionImpl(processFullDefinition));
    } else {
      throw new IllegalArgumentException(" Can't archive: " + processFullDefinition + " --- A record with procesDefinitionUUID: "
        + processFullDefinition.getUUID() + " has already been archived ");
    }
  }

  public void remove(final ProcessFullDefinition processFullDefinition) {
    Misc.checkArgsNotNull(processFullDefinition);
    if (EnvTool.getHistoryQueriers().getProcessDefinition(processFullDefinition.getUUID()) != null) {
      this.getQuerierDbSession().delete(new ProcessFullDefinitionImpl(processFullDefinition));
    }
  }

  public void archive(final Archivable archivable) {
    if (archivable instanceof ProcessFullDefinition) {
      this.archive((ProcessFullDefinition) archivable);
    } else if (archivable instanceof ProcessFullInstance) {
      this.archive((ProcessFullInstance) archivable);
    } else {
      Misc.unreachableStatement();
    }
  }

  public void remove(final Archivable archivable) {
    if (archivable instanceof ProcessFullDefinition) {
      this.remove((ProcessFullDefinition) archivable);
    } else if (archivable instanceof ProcessFullInstance) {
      this.remove((ProcessFullInstance) archivable);
    } else {
      Misc.unreachableStatement();
    }
  }
}
