/**
 * Copyright (C) 2008-2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.persistence.db;

import java.util.Date;

import org.ow2.orchestra.facade.data.runtime.ActivityInstanceData;
import org.ow2.orchestra.facade.data.runtime.CorrelationSetInitializedData;
import org.ow2.orchestra.facade.data.runtime.ForeachInitializedData;
import org.ow2.orchestra.facade.data.runtime.PartnerLinkUpdateData;
import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.data.runtime.ScopeStateUpdateData;
import org.ow2.orchestra.facade.data.runtime.TimerData;
import org.ow2.orchestra.facade.data.runtime.TransitionConditionUpdateData;
import org.ow2.orchestra.facade.data.runtime.VariableUpdateData;
import org.ow2.orchestra.facade.def.full.ProcessFullDefinition;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.runtime.ActivityType;
import org.ow2.orchestra.facade.runtime.WaitingActivity;
import org.ow2.orchestra.facade.runtime.full.ActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityWithChildrenFullInstance;
import org.ow2.orchestra.facade.runtime.full.ActivityWithSingleChildFullInstance;
import org.ow2.orchestra.facade.runtime.full.CompensationHandlerActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.FlowActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ForeachActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.PickActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.facade.runtime.full.ScopeActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.TerminationHandlerActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.WaitActivityFullInstance;
import org.ow2.orchestra.facade.runtime.full.impl.AssignActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CatchActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CatchAllActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CompensateActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CompensateScopeActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CompensationHandlerActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.CorrelationSetInitializationFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.EmptyActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ExitActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.FlowActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ForeachActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.IfActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.InvokeActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.PartnerLinkUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.PickActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ProcessFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ProcessInstanceStateUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ReceiveActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.RepeatUntilActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ReplyActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.RethrowActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ScopeActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ScopeStateUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.SequenceActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.TerminationHandlerActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.ThrowActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.TransitionConditionUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.UnknownActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.VariableUpdateFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.WaitActivityFullInstanceImpl;
import org.ow2.orchestra.facade.runtime.full.impl.WaitingActivityFullImpl;
import org.ow2.orchestra.facade.runtime.full.impl.WhileActivityFullInstanceImpl;
import org.ow2.orchestra.facade.uuid.ActivityDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ActivityInstanceUUID;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.QuerierDbSession;
import org.ow2.orchestra.services.Archivable;
import org.ow2.orchestra.services.Recorder;
import org.ow2.orchestra.util.Misc;

/**
 * @author Gael Lalire, Guillaume Renault
 */
public class DbJournal extends AbstractDbQuerier implements Recorder {

  public DbJournal(final QuerierDbSession dbSession) {
    super(dbSession);
  }

  public void recordInstanceStarted(final ProcessInstanceData processInstanceData) {
    final ProcessDefinitionUUID processUUID = processInstanceData.getProcessDefinitionData().getUUID();
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID);

    final ProcessFullInstance processFullInstance = new ProcessFullInstanceImpl(instanceUUID, processUUID);
    processFullInstance.addProcessInstanceState(new ProcessInstanceStateUpdateFullImpl(new Date(), processInstanceData
      .getState()));
    this.addProcessInstance(processFullInstance);
    this.getQuerierDbSession().save(processFullInstance);
  }

  public void recordInstanceEnded(final ProcessInstanceData processInstanceData) {
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(instanceUUID);

    final ProcessFullInstance processFullInstance = this.getProcessInstance(instanceUUID);

    processFullInstance.addProcessInstanceState(new ProcessInstanceStateUpdateFullImpl(new Date(), processInstanceData
      .getState()));
    processFullInstance.end(processInstanceData.getState());
  }

  /**
   * Record an instance state update.
   */
  public void recordInstanceUpdated(final ProcessInstanceData processInstanceData) {
    final ProcessInstanceUUID instanceUUID = processInstanceData.getUUID();
    Misc.checkArgsNotNull(instanceUUID);

    final ProcessFullInstance processFullInstance = this.getProcessInstance(instanceUUID);
    processFullInstance.addProcessInstanceState(new ProcessInstanceStateUpdateFullImpl(new Date(), processInstanceData
      .getState()));

  }

  public void recordActivityStarted(final ActivityInstanceData activityInstanceData) {
    final ProcessDefinitionUUID processUUID = activityInstanceData.getActivityDefinitionData().getProcessDefinitionData()
      .getUUID();
    final ProcessInstanceUUID instanceUUID = activityInstanceData.getProcessInstanceData().getUUID();
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityDefinitionUUID activityDefinitionUUID = activityInstanceData.getActivityDefinitionData().getUUID();
    final ActivityInstanceUUID encloserInstanceUUID = activityInstanceData.getEncloserUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID, activityUUID);

    ActivityFullInstance activityFullInstance;
    final ActivityType activityType = activityInstanceData.getActivityDefinitionData().getType();
    switch (activityType) {
      case ASSIGN:
        activityFullInstance = new AssignActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case CATCH_HANDLER:
        activityFullInstance = new CatchActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case CATCHALL_HANDLER:
        activityFullInstance = new CatchAllActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case TERMINATION_HANDLER:
        activityFullInstance = new TerminationHandlerActivityFullInstanceImpl(activityUUID, activityDefinitionUUID,
          processUUID, instanceUUID);
        break;
      case COMPENSATION_HANDLER:
        activityFullInstance = new CompensationHandlerActivityFullInstanceImpl(activityUUID, activityDefinitionUUID,
          processUUID, instanceUUID);
        break;
      case SCOPE:
        activityFullInstance = new ScopeActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case PROCESS:
        activityFullInstance = new ProcessFullInstanceImpl(instanceUUID, processUUID);
        break;
      case SEQUENCE:
        activityFullInstance = new SequenceActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case RECEIVE:
        activityFullInstance = new ReceiveActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case REPLY:
        activityFullInstance = new ReplyActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case INVOKE:
        activityFullInstance = new InvokeActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case WAIT:
        activityFullInstance = new WaitActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case PICK:
        activityFullInstance = new PickActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case EMPTY:
        activityFullInstance = new EmptyActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case EXIT:
        activityFullInstance = new ExitActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case IF:
        activityFullInstance = new IfActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);
        break;
      case REPEAT_UNTIL:
        activityFullInstance = new RepeatUntilActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case FOR_EACH:
        activityFullInstance = new ForeachActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case WHILE:
        activityFullInstance = new WhileActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case FLOW:
        activityFullInstance = new FlowActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case COMPENSATE:
        activityFullInstance = new CompensateActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case COMPENSATE_SCOPE:
        activityFullInstance = new CompensateScopeActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case RETHROW:
        activityFullInstance = new RethrowActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case THROW:
        activityFullInstance = new ThrowActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
      case VALIDATE:
      case EXTENSION_ACTIVITY:
      default:
        activityFullInstance = new UnknownActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID,
          instanceUUID);
        break;
    }

    this.addActivityInstance(activityFullInstance);

    if (encloserInstanceUUID == null) {
      Misc.unreachableStatement("EncloserUUID cannot be null");
    } else if (encloserInstanceUUID.equals(activityFullInstance.getUUID())) {
      Misc.unreachableStatement("EncloserUUID (" + encloserInstanceUUID
        + ") cannot have the same UUID that the activityFullInstance (" + activityFullInstance.getUUID());
    } else {
      // parent activity
      final ActivityFullInstance encloserActivityFullInstance = this.getActivityInstance(encloserInstanceUUID);

      if (activityType.equals(ActivityType.CATCH_HANDLER) || activityType.equals(ActivityType.CATCHALL_HANDLER)) {
        ((ScopeActivityFullInstance) encloserActivityFullInstance)
          .setFaultHandlerActivityInstance((ActivityWithSingleChildFullInstance) activityFullInstance);
      } else if (activityType.equals(ActivityType.TERMINATION_HANDLER)) {
        ((ScopeActivityFullInstance) encloserActivityFullInstance)
          .setTerminationHandlerActivityInstance((TerminationHandlerActivityFullInstance) activityFullInstance);
      } else if (activityType.equals(ActivityType.COMPENSATION_HANDLER)) {
        ((ScopeActivityFullInstance) encloserActivityFullInstance)
          .setCompensationHandlerActivityInstance((CompensationHandlerActivityFullInstance) activityFullInstance);
      } else if (encloserActivityFullInstance.isChildrenActivity()) {
        ((ActivityWithChildrenFullInstance) encloserActivityFullInstance).addEnclosedActivity(activityFullInstance);
      } else if (encloserActivityFullInstance.isSingleChildActivity()) {
        ((ActivityWithSingleChildFullInstance) encloserActivityFullInstance).setEnclosedActivity(activityFullInstance);
      } else {
        Misc.unreachableStatement("EncloserUUID (" + encloserInstanceUUID + ") has no possible enclosed activity. "
          + "It cannot be the parent of activityFullInstance (" + activityFullInstance.getUUID());
      }
    }
  }

  public void recordEventStarted(final ActivityInstanceData activityInstanceData) {
    final ProcessDefinitionUUID processUUID = activityInstanceData.getActivityDefinitionData().getProcessDefinitionData()
      .getUUID();
    final ProcessInstanceUUID instanceUUID = activityInstanceData.getProcessInstanceData().getUUID();
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityDefinitionUUID activityDefinitionUUID = activityInstanceData.getActivityDefinitionData().getUUID();
    final ActivityInstanceUUID encloserInstanceUUID = activityInstanceData.getEncloserUUID();
    Misc.checkArgsNotNull(processUUID, instanceUUID, activityUUID);

    final ScopeActivityFullInstance activityFullInstance =
      new ScopeActivityFullInstanceImpl(activityUUID, activityDefinitionUUID, processUUID, instanceUUID);

    this.addActivityInstance(activityFullInstance);

    if (encloserInstanceUUID == null) {
      Misc.unreachableStatement("EncloserUUID cannot be null");
    } else if (encloserInstanceUUID.equals(activityFullInstance.getUUID())) {
      Misc.unreachableStatement("EncloserUUID (" + encloserInstanceUUID
        + ") cannot have the same UUID that the activityFullInstance (" + activityFullInstance.getUUID());
    } else {
      // parent activity
      final ActivityFullInstance encloserActivityFullInstance = this.getActivityInstance(encloserInstanceUUID);
      ((ScopeActivityFullInstance) encloserActivityFullInstance).addEventInstance(activityFullInstance);
    }
  }

  public void recordExceptionOccured(final ActivityInstanceUUID activityInstanceUUID, final BpelFaultException exception) {
    final ActivityFullInstance activityFullInstance = this.getActivityInstance(activityInstanceUUID);
    activityFullInstance.setLastException(exception.fullCopy());
  }

  public void recordActivityEnded(final ActivityInstanceData activityInstanceData) {
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityFullInstance activityFullInstance = this.getActivityInstance(activityUUID);
    activityFullInstance.end(ActivityState.FINISHED);
  }

  public void recordActivityExited(final ActivityInstanceData activityInstanceData) {
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityFullInstance activityFullInstance = this.getActivityInstance(activityUUID);
    activityFullInstance.end(ActivityState.EXITED);
  }

  public void recordActivityTerminated(final ActivityInstanceData activityInstanceData) {
    final ActivityInstanceUUID activityUUID = activityInstanceData.getUUID();
    final ActivityFullInstance activityFullInstance = this.getActivityInstance(activityUUID);
    activityFullInstance.end(ActivityState.TERMINATED);
  }

  public void remove(final Archivable archivable) {
    if (archivable instanceof ProcessFullInstance) {
      this.remove((ProcessFullInstance) archivable);
    } else if (archivable instanceof ProcessFullDefinition) {
      this.remove((ProcessFullDefinition) archivable);
    } else {
      Misc.unreachableStatement();
    }
  }

  public void remove(final ProcessFullInstance processFullInstance) {
    Misc.checkArgsNotNull(processFullInstance);
    if (this.getProcessInstance(processFullInstance.getUUID()) != null) {
      this.getQuerierDbSession().delete(processFullInstance);
    }
  }

  public void remove(final ProcessFullDefinition processFullDefinition) {
    Misc.checkArgsNotNull(processFullDefinition);
    if (this.getProcessDefinition(processFullDefinition.getUUID()) != null) {
      this.getQuerierDbSession().delete(processFullDefinition);
    }
  }

  public void recordTimerStarted(final TimerData timerData) {
    final ActivityFullInstance activityFullInstance = this.getActivityInstance(timerData.getActivityInstanceUUID());
    final WaitingActivity waitingActivity = new WaitingActivityFullImpl(timerData.getActivityDefinitionUUID(), timerData
      .getExpectedStartedDate());
    switch (activityFullInstance.getType()) {
      case PICK:
        ((PickActivityFullInstance) activityFullInstance).addWaitingActivity(waitingActivity);
        break;
      case WAIT:
        final WaitActivityFullInstance waitActivityFullInstance = (WaitActivityFullInstance) activityFullInstance;
        Misc.badStateIfNotNull(waitActivityFullInstance.getWaitingActivity(), "wait can only own one timer");
        waitActivityFullInstance.setWaitingActivity(waitingActivity);
        break;
      case PROCESS:
      case SCOPE:
        // don't add record
        break;
      default:
        Misc.unreachableStatement("Only wait and pick activity can own timers, not " + activityFullInstance.getType());
    }
  }

  public void recordForeachInitialized(final ForeachInitializedData foreachInitializedData) {
    final ForeachActivityFullInstance foreachActivityFullInstance = (ForeachActivityFullInstance) this
      .getActivityInstance(foreachInitializedData.getActivityInstanceUUID());
    foreachActivityFullInstance.update(foreachInitializedData.getStartValue(), foreachInitializedData.getStopValue(),
      foreachInitializedData.getCompletionConditionValue());
  }

  public void recordVariableUpdated(final VariableUpdateData variableUpdateData) {
    // following scopeActivityFullInstance may be a scope or a process
    final ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) this
      .getActivityInstance(variableUpdateData.getScopeActivityUUID());

    scopeActivityFullInstance.getVariables().add(
      new VariableUpdateFullImpl(variableUpdateData.getActivityUUID(), new Date(), variableUpdateData.getName(),
        variableUpdateData.getValue()));
  }

  public void recordCorrelationSetInitialized(final CorrelationSetInitializedData correlationSetInitializedData) {
    final ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) this
      .getActivityInstance(correlationSetInitializedData.getScopeActivityUUID());

    scopeActivityFullInstance.getCorrelationSet().add(
      new CorrelationSetInitializationFullImpl(correlationSetInitializedData.getActivityUUID(), new Date(),
        correlationSetInitializedData.getName(), correlationSetInitializedData.getPropertyValues()));
  }

  public void recordPartnerLinkUpdated(final PartnerLinkUpdateData partnerLinkUpdateData) {
    final ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) this
      .getActivityInstance(partnerLinkUpdateData.getScopeActivityUUID());

    if (scopeActivityFullInstance.getPartnerLink() == null) {
      Misc.unreachableStatement();
    }

    scopeActivityFullInstance.getPartnerLink().add(
      new PartnerLinkUpdateFullImpl(partnerLinkUpdateData.getActivityUUID(), new Date(), partnerLinkUpdateData.getName(),
        partnerLinkUpdateData.getPartnerLinkUpdateType(), partnerLinkUpdateData.getPartnerLinkReference()));
  }

  public void recordTransitionConditionUpdated(final TransitionConditionUpdateData transitionConditionUpdateData) {
    final FlowActivityFullInstance flowActivityFullInstance = (FlowActivityFullInstance) this
      .getActivityInstance(transitionConditionUpdateData.getFlowActivityUUID());

    flowActivityFullInstance.getTransitionConditionUpdateList().add(
      new TransitionConditionUpdateFullImpl(transitionConditionUpdateData.getStatus(),
        transitionConditionUpdateData.getName(), new Date()));
  }

  public void recordScopeStateChanged(final ScopeStateUpdateData scopeStateUpdateData) {
    final ScopeActivityFullInstance scopeActivityFullInstance = (ScopeActivityFullInstance) this
      .getActivityInstance(scopeStateUpdateData.getScopeActivityUUID());

    scopeActivityFullInstance.getScopeState().add(
      new ScopeStateUpdateFullImpl(scopeStateUpdateData.getActivityUUID(), new Date(), scopeStateUpdateData.getState()));
  }

  public void recordProcessDeployed(final ProcessFullDefinition processFullDefinition) {
    this.getQuerierDbSession().save(processFullDefinition);
  }

  public void recordProcessUndeployed(final ProcessDefinitionUUID processDefinitionUUID) {
    final ProcessFullDefinition processFullDefinition = this.getProcessDefinition(processDefinitionUUID);
    processFullDefinition.setUndeployedDate(new Date());
  }
}
