package org.ow2.orchestra.persistence.db;

import java.util.List;

import javax.xml.namespace.QName;

import org.hibernate.FlushMode;
import org.hibernate.Hibernate;
import org.hibernate.LockMode;
import org.hibernate.Query;
import org.hibernate.Session;
import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.RuntimeDbSession;
import org.ow2.orchestra.persistence.db.hibernate.QNameUserType;
import org.ow2.orchestra.pvm.internal.hibernate.HibernatePvmDbSession;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.services.Receiver;

public class RuntimeDbSessionImpl extends
    HibernatePvmDbSession implements RuntimeDbSession {

  public RuntimeDbSessionImpl(final Session session) {
    this.setSession(session);
    session.setFlushMode(FlushMode.COMMIT);
  }

  public BpelProcess findBpelProcess(final String namespace, final String name, final int version) {
    final Query query = this.getSession().getNamedQuery("findBpelProcess");
    query.setString("namespace", namespace);
    query.setString("name", name);
    query.setInteger("version", version);
    final BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public BpelProcess findBpelProcess(final String namespace, final String name, final boolean lock) {
    final Query query = this.getSession().getNamedQuery("findLatestBpelProcess");
    query.setString("namespace", namespace);
    query.setString("name", name);
    if (lock) {
      query.setLockMode("process", LockMode.UPGRADE);
    }
    final BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public BpelProcess getBpelProcess(final ProcessDefinitionUUID pduuid) {
    final Query query = this.getSession().getNamedQuery("getBpelProcessByUUID");
    query.setParameter("uuid", pduuid);
    final BpelProcess processDefinition = (BpelProcess) query.uniqueResult();
    return processDefinition;
  }

  public List<BpelProcess> getBpelProcesses() {
    final Query query = this.getSession().getNamedQuery("getBpelProcesses");
    return query.list();
  }

  public Receiver getReceiver(final OperationKey operationKey) {
    final Query query = this.getSession().getNamedQuery("getReceiver");
    query.setParameter("processQName", operationKey.getProcessQName(),
        Hibernate.custom(QNameUserType.class));
    query.setParameter("portTypeQName", operationKey.getPortTypeQName(),
        Hibernate.custom(QNameUserType.class));
    query.setString("operationName", operationKey.getOperationName());
    return (Receiver) query.uniqueResult();
  }

  public List<Receiver> getReceivers(final QName processQName) {
    final Query query = this.getSession().getNamedQuery("getReceivers");
    query.setParameter("processQName", processQName, Hibernate
        .custom(QNameUserType.class));
    return query.list();
  }


  public List<BpelExecution> getBpelInstances() {
    final Query query = this.getSession().getNamedQuery("getBpelInstances");
    return query.list();
  }

  public BpelExecution getBpelInstance(final ProcessInstanceUUID processInstanceUUID) {
    final Query query = this.getSession().getNamedQuery("getBpelInstance");
    query.setString("processInstanceUUID", processInstanceUUID.toString());
    final BpelExecution bpelExecution = (BpelExecution) query.uniqueResult();
    return bpelExecution;
  }

  public List<BpelExecution> getBpelInstances(final QName processQName) {
    final Query query = this.getSession().getNamedQuery("getBpelInstancesOfProcess");
    query.setString("namespace", processQName.getNamespaceURI());
    query.setString("name", processQName.getLocalPart());
    return query.list();
  }

}
