/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.persistence.db.hibernate;

import java.io.ByteArrayOutputStream;
import java.io.Reader;
import java.io.StringReader;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.util.XmlUtil;
import org.w3c.dom.Element;

/**
 * @author Juan Cantu
 * @author Guillaume Porcher
 */
public class ElementType extends NodeType {

  @Override
  public Object nullSafeGet(final ResultSet rs, final String[] names, final Object owner) throws SQLException {
    // retrieve stream from database
    final String columnName = names[0];
    Reader reader = null;

    try {
      reader = rs.getCharacterStream(columnName);
      // if SQL value is NULL, element is null as well
      if (reader == null) {
        return null;
      }
      // parse XML text
      final Element element = XmlUtil.getDocumentFromReader(reader).getDocumentElement();


      return element;
    } finally {
      if (reader != null) {
        try {
          reader.close();
        } catch (final Exception e) {
          // nothing
        }
      }
    }
  }

  @Override
  public void nullSafeSet(final PreparedStatement st, final Object value, final int index) throws SQLException {
    // easy way out: null value
    if (value == null) {
      st.setNull(index, NodeType.SQL_TYPES[0]);
    } else {
      final Element element = (Element) value;
      try {
        final ByteArrayOutputStream byteStream = XmlUtil.getNodeAsBAOS(element);
        final Reader sr = new StringReader(byteStream.toString());
        st.setCharacterStream(index, sr, byteStream.size());
      } catch (final OrchestraRuntimeException e) {
        final Class clazz = value.getClass();
        if (e.getCause() != null && e.getCause() instanceof ClassCastException
            && clazz.getName().startsWith("org.apache.axis.message.")) {
          final Reader sr = new StringReader(value.toString());
          st.setCharacterStream(index, sr, value.toString().length());
        } else {
          throw e;
        }
      }
    }
  }

  @Override
  public Class returnedClass() {
    return Element.class;
  }
}


