/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.persistence.log;

import java.util.logging.Level;
import java.util.logging.Logger;

import javax.transaction.Synchronization;

import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.env.Transaction;
import org.ow2.orchestra.services.Archivable;
import org.ow2.orchestra.services.Archiver;

public class LoggerArchiver implements Archiver {

  public static final Logger LOG = Logger.getLogger(LoggerArchiver.class.getName());
  private final Level level;

  public LoggerArchiver() {
    this(Level.FINE);
  }

  public LoggerArchiver(final String levelName) {
    this(Level.parse(levelName));
  }

  public LoggerArchiver(final Level level) {
    this.level = level;
    final Transaction tx = Environment.getFromCurrent(Transaction.class);
    tx.registerSynchronization(new Synchronization() {

      public void beforeCompletion() {
      }

      public void afterCompletion(final int status) {
        if (status == javax.transaction.Status.STATUS_COMMITTED) {
          LoggerArchiver.LOG.log(level, "Transaction committed");
        } else {
          LoggerArchiver.LOG.log(level, "Transaction rolledback");
        }
      }
    });
  }

  private void log(final Archivable archivable) {
    if (LoggerArchiver.LOG.isLoggable(this.level)) {
      LoggerArchiver.LOG.log(this.level, "Archiving: " + archivable);
    }
  }

  public void archive(final Archivable archivable) {
    this.log(archivable);
  }

  public void remove(final Archivable archivable) {
    if (LoggerArchiver.LOG.isLoggable(this.level)) {
      LoggerArchiver.LOG.log(this.level, "Deleting from archive: " + archivable);
    }
  }
}
