/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.runtime;

import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.Exit;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.pvm.internal.model.ExceptionHandlerImpl;
import org.ow2.orchestra.pvm.internal.model.ExecutionImpl;
import org.ow2.orchestra.pvm.internal.model.ProcessElementImpl;

/**
 * @author S. Ali Tokmen
 */
public class FaultHandlerThatExits extends ExceptionHandlerImpl {

  private static final long serialVersionUID = 2305022461357079192L;

  private static Logger log = Logger.getLogger(FaultHandlerThatExits.class.getName());

  @Override
  public boolean matches(final Class< ? > exceptionClass) {
    return BpelFaultException.class.isAssignableFrom(exceptionClass);
  }

  @Override
  protected void executeHandler(final ExecutionImpl execution, final Exception exception,
    final ProcessElementImpl processElementImpl) {
    FaultHandlerThatExits.log.finest("Exception rethrown, exiting process with execution " + execution);

    Exit exit = new Exit();
    final BpelExecution bpelExecution = (BpelExecution) execution;
    exit.executeActivity(bpelExecution);
  }
}
