/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.runtime;

import java.util.logging.Logger;

import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.pvm.internal.model.ExceptionHandlerImpl;
import org.ow2.orchestra.pvm.internal.model.ExecutionImpl;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.model.ProcessElementImpl;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.Misc;

/**
 * @author S. Ali Tokmen
 */
public class FaultInCHHandler extends ExceptionHandlerImpl {

  private static final long serialVersionUID = 6330814889501735922L;

  private static Logger log = Logger.getLogger(FaultInCHHandler.class.getName());

  @Override
  public boolean matches(final Class< ? > exceptionClass) {
    return BpelFaultException.class.isAssignableFrom(exceptionClass);
  }

  @Override
  protected void executeHandler(final ExecutionImpl execution, final Exception exception,
    final ProcessElementImpl processElementImpl) {
    final NodeImpl chNode = (NodeImpl) processElementImpl;
    final BpelExecution bpelExecution = (BpelExecution) execution;
    final BpelFaultException bpelFault = (BpelFaultException) exception;

    EnvTool.getRecorder().recordExceptionOccured(bpelExecution.getActivityInstanceUUID(), bpelFault);

    BpelExecution chBpelExecution = bpelExecution;
    while (chBpelExecution != null && chBpelExecution.getNode() != chNode) {
      chBpelExecution = chBpelExecution.getParent();
    }
    if (chBpelExecution == null || chBpelExecution.getNode() != chNode) {
      Misc.unreachableStatement("Cannot find the bpelExecution for compensation handler " + chNode);
    }
    final ScopeRuntime scopeRuntime = chBpelExecution.getCurrentScopeRuntime();

    FaultInCHHandler.log.finest("Handling exception " + exception + " in BpelExecution " + bpelExecution
      + ": scopeRuntime state is " + scopeRuntime.getState() + ", CHBpelExecution is " + chBpelExecution);

    if (!ScopeState.COMPENSATING.equals(scopeRuntime.getState())) {
      Misc.unreachableStatement("The compensation handler " + chNode + " is in invalid state " + scopeRuntime.getState());
    }

    scopeRuntime.setLastException(bpelFault.fullCopy());
    chBpelExecution.terminate();
  }
}
