/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.runtime;


public class ForEachRuntime {

  public static final int NO_MORE_COUNTER_VALUE = -1;

  private int startCounterValue;
  private int finalCounterValue;
  // Completion condition B
  private int completionValue;
  private boolean onlySuccessful;

  // count terminated branches (successful only if onlySuccessful==true)
  private int branchCount;

  private int nextCounterValue;

  protected ForEachRuntime() {
  }

  public ForEachRuntime(final int startCounterValue,
      final int finalCounterValue, final int completionValue,
      final boolean onlySuccessful) {
    this.startCounterValue = startCounterValue;
    this.finalCounterValue = finalCounterValue;
    this.nextCounterValue = startCounterValue;
    branchCount = 0;
    this.completionValue = completionValue;
    this.onlySuccessful = onlySuccessful;
  }

  public boolean isCompleted() {
    if (branchCount >= completionValue) {
      return true;
    }
    return false;
  }

  public void addTerminatedBranch(final boolean successful) {
    if (!onlySuccessful) {
      branchCount++;
    } else {
      if (successful) {
        branchCount++;
      }
    }
  }

  /**
   * @return the next counter value or {@link ForEachRuntime.NO_MORE_COUNTER_VALUE} if no more executions are intended
   */
  public int getNextCurrentCounterValue() {
    int result = this.nextCounterValue;
    if (result > finalCounterValue) {
      result = NO_MORE_COUNTER_VALUE;
    } else {
      this.nextCounterValue++;
    }
    return result;
  }

  public int getStartCounterValue() {
    return startCounterValue;
  }

  public int getFinalCounterValue() {
    return finalCounterValue;
  }

  @Override
  public String toString() {
    return ForEachRuntime.class.getName()
      + "[startCounterValue: " + startCounterValue
      + ", finalCounterValue: " + finalCounterValue
      + ", nextCounterValue: " + nextCounterValue
      + "]";
  }
}
