/**
 * Copyright (C) 2009 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.runtime;

import java.util.logging.Logger;

import org.ow2.orchestra.definition.activity.Scope;
import org.ow2.orchestra.facade.exception.BpelFaultException;
import org.ow2.orchestra.facade.runtime.ScopeState;
import org.ow2.orchestra.pvm.internal.model.ExceptionHandlerImpl;
import org.ow2.orchestra.pvm.internal.model.ExecutionImpl;
import org.ow2.orchestra.pvm.internal.model.NodeImpl;
import org.ow2.orchestra.pvm.internal.model.ProcessElementImpl;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.util.Misc;

/**
 * @author S. Ali Tokmen
 */
public class OrchestraFaultHandler extends ExceptionHandlerImpl {

  private static final long serialVersionUID = 6330814889501735922L;

  private static Logger log = Logger.getLogger(OrchestraFaultHandler.class.getName());

  @Override
  public boolean matches(final Class< ? > exceptionClass) {
    return BpelFaultException.class.isAssignableFrom(exceptionClass);
  }

  @Override
  protected void executeHandler(final ExecutionImpl execution, final Exception exception,
    final ProcessElementImpl processElementImpl) {
    final NodeImpl scope = (NodeImpl) processElementImpl;
    final BpelExecution bpelExecution = (BpelExecution) execution;
    final BpelFaultException bpelFault = (BpelFaultException) exception;

    EnvTool.getRecorder().recordExceptionOccured(bpelExecution.getActivityInstanceUUID(), bpelFault);

    BpelExecution scopeBpelExecution = bpelExecution;
    while (scopeBpelExecution != null && scopeBpelExecution.getNode() != scope) {
      scopeBpelExecution = scopeBpelExecution.getParent();
    }
    if (scopeBpelExecution == null || scopeBpelExecution.getNode() != scope) {
      Misc.unreachableStatement("Cannot find the scope BpelExecution for scope " + scope);
    }
    final ScopeRuntime scopeRuntime = bpelExecution.getScopeRuntimeWithScope((Scope) scope.getBehaviour());

    OrchestraFaultHandler.log.finest("Handling exception " + exception + " in BpelExecution " + bpelExecution
      + ": scopeRuntime state is " + scopeRuntime.getState() + ", scopeBpelExecution is " + scopeBpelExecution);

    if (ScopeState.RUNNING.equals(scopeRuntime.getState())) {
      // set state = handling fault
      scopeRuntime.setLastException(bpelFault.fullCopy());
      scopeRuntime.setState(ScopeState.HANDLING_FAULT);
      scopeBpelExecution.terminate();
    } else if (ScopeState.TERMINATING.equals(scopeRuntime.getState())) {
      scopeRuntime.setState(ScopeState.COMPLETED_UNSUCCESSFUL);
      scopeBpelExecution.exit();
      if (scopeBpelExecution.getParent() != null) {
        OrchestraFaultHandler.log.finest("Terminating parent " + scopeBpelExecution.getParent());
        (scopeBpelExecution.getParent()).terminate();
      }
    } else {
      if (ScopeState.HANDLING_FAULT.equals(scopeRuntime.getState())) {
        scopeRuntime.setState(ScopeState.RETHROWING_FAULT);
      }
      ExceptionHandlerImpl.rethrow(exception, "Rethrown by BpelExecution " + bpelExecution);
    }
  }
}
