/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

/**
 *
 */
package org.ow2.orchestra.services;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.cmd.Command;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.itf.Repository;
import org.ow2.orchestra.util.EnvTool;
import org.ow2.orchestra.var.MessageVariable;

public class AsyncAssociateMessage implements Command<BpelExecution> {

  protected OperationKey operationKey;
  protected MessageVariable incomingMessage;
  protected MessageCarrier messageCarrier;
  /**
   * @param operationKey
   * @param incomingMessage
   * @param messageCarrier
   */
  public AsyncAssociateMessage(
      final OperationKey operationKey,
      final MessageVariable incomingMessage,
      final MessageCarrier messageCarrier) {
    this.operationKey = operationKey;
    this.incomingMessage = incomingMessage;
    this.messageCarrier = messageCarrier;
  }

  public BpelExecution execute(final Environment environment) throws Exception {
    final Repository repository = EnvTool.getRepository();

    BpelProcess bpelProcess = null;
    // First create pending message from incoming message.
    // This message will be stored in Receiver until it is successfully used in an instance.

    // lock process
    bpelProcess = repository.lockProcess(this.operationKey.getProcessQName());

    final Receiver receiver = bpelProcess.getReceiver(this.operationKey);
    if (receiver == null) {
      throw new OrchestraRuntimeException("impossible to find a receiver corresponding to: " + this.operationKey);
    }
    final PendingMessage pendingMessage = receiver.storeIncomingMessage(this.incomingMessage, this.messageCarrier);

    // try to associate pendingMessage
    final ExecElementToSignal newExecElt = receiver.associatePendingMessage(pendingMessage, bpelProcess);
    if (newExecElt != null) {
      // an execution has been found
      // signal it in the same transaction.
      SignalExecUtil.execute(
          environment,
          newExecElt.getBpelExecution(),
          newExecElt.getReceivingElement(),
          newExecElt.getPendingMessage());
    } else {
      receiver.storePendingMessage(pendingMessage);
    }

    return newExecElt != null ? newExecElt.getBpelExecution() : null;
  }

}