/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.services;

import javax.xml.namespace.QName;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobDbSession;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.itf.Repository;

/**
 * Try to find a message that can be given to the current execution.
 *
 * @author Guillaume Porcher
 *
 */
public class AsyncFindExecToSignal extends BpelMessageImpl<Boolean> {


  protected AsyncFindExecToSignal() { }
  public AsyncFindExecToSignal(final BpelExecution instance, final QName processQName) {
    this.processInstance = instance.getProcessInstance();
    this.execution = instance.getProcessInstance();
    this.processQName = processQName;
  }

  public Boolean execute(final Environment environment) throws Exception {
    BpelExecution instance = (BpelExecution) this.getProcessInstance();

    ExecElementToSignal execElt = null;

    // take lock on process
    final Repository repository = environment.get(Repository.class);
    final BpelProcess bpelProcess = repository.lockProcess(this.processQName);

    // check if execution is still up-to-date
    instance = repository.getUpdatedExecution(instance);
    if (instance != null && !instance.isEnded()) {
      // find a new child execution to signal
      execElt = Receiver.findExecutionToSignal(bpelProcess, instance);
      if (execElt != null) {
        // create signal job
        SignalExecUtil.execute(
            environment,
            execElt.getBpelExecution(),
            execElt.getReceivingElement(),
            execElt.getPendingMessage());
      }
    }

    // delete job
    final JobDbSession dbSession = environment.get(JobDbSession.class);
    if (dbSession == null) {
      throw new OrchestraRuntimeException("no " + JobDbSession.class.getName() + " in environment");
    }
    dbSession.delete(this);
    return true;
  }
}
