package org.ow2.orchestra.services;

import java.io.Serializable;

import javax.xml.namespace.QName;

public class OperationKey implements Serializable {

  private static final long serialVersionUID = 7629200185946199717L;

  /**
   * represent the key of each 'exchange' activity like receive, reply, invoke,
   * pick...
   *
   */
  protected QName processQName;

  protected QName portTypeQName;

  protected String operationName;

  /** For persistence * */
  protected OperationKey() {
  }

  public OperationKey(final QName processQName, final QName portTypeQName,
      final String operationName) {
    this.processQName = processQName;
    this.portTypeQName = portTypeQName;
    this.operationName = operationName;
  }

  /**
   * Copy constructor
   * @param operationKey
   */
  public OperationKey(final OperationKey operationKey) {
    this.processQName = operationKey.getProcessQName();
    this.portTypeQName = operationKey.getPortTypeQName();
    this.operationName = operationKey.getOperationName();
  }

  @Override
  public boolean equals(final Object o) {
    if (!(o instanceof OperationKey)) {
      return false;
    }
    final OperationKey other = (OperationKey) o;
    return this.processQName.equals(other.getProcessQName())
        && this.portTypeQName.equals(other.getPortTypeQName())
        && this.operationName.equals(other.getOperationName());
  }

  @Override
  public int hashCode() {
    int hashcode = 13;
    hashcode += hashcode * 37 + this.processQName.hashCode();
    hashcode += hashcode * 37 + this.portTypeQName.hashCode();
    hashcode += hashcode * 37 + this.operationName.hashCode();
    return hashcode;
  }

  public String getOperationName() {
    return this.operationName;
  }

  public QName getPortTypeQName() {
    return this.portTypeQName;
  }

  public QName getProcessQName() {
    return this.processQName;
  }

  @Override
  public String toString() {
    return "[" + this.processQName + "|" + this.portTypeQName + "|" + this.operationName + "]";
  }
}
