/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services;

import org.ow2.orchestra.env.EnvFactoryRepository;
import org.ow2.orchestra.pvm.internal.cmd.CommandService;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.var.MessageVariable;


/**
 * @author Guillaume Porcher
 *
 */
public class ReceivingService {
  /**
   * Handle incoming message for the given operation key. Use the given
   * {@link MessageCarrier} to store the reply.
   *
   * @param incomingMessage message received
   * @param operationKey operation key of the service that received the message
   * @param messageCarrier object that will be used to get the response
   * @return the execution to which the message was given. <code>null</code>
   *         if there is no execution waiting for this message (the message is
   *         stored).
   */
  public static BpelExecution handle(final MessageVariable incomingMessage,
      final OperationKey operationKey,
      final MessageCarrier messageCarrier) {
    final CommandService commandService = EnvFactoryRepository.get().get(CommandService.class);

    // give the message to an existing instance or create a new
    // instance
    return commandService.execute(
        new AsyncAssociateMessage(operationKey, incomingMessage, messageCarrier)
    );
  }



}
