/**
 * Copyright (C) 2008 Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.services;

import java.util.HashMap;
import java.util.Map;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.runtime.BpelExecution;

/**
 * Gives the pendingMessage to the execution
 *
 * @author Guillaume Porcher
 *
 */
public final class SignalExecUtil {

  public static final String RECEIVING_ELEMENT_PARAM = "receivingElement";
  public static final String MESSAGE_PARAM = "message";
  public static final String MESSAGE_CARRIER_PARAM = "messageCarrier";

  private SignalExecUtil() { }

  public static void execute(final Environment environment,
      final BpelExecution exec, final ReceivingElement recvElt,
      final PendingMessage pendingMessage) throws Exception {


    final Map<String, Object> parameters = new HashMap<String, Object>();
    parameters.put(SignalExecUtil.RECEIVING_ELEMENT_PARAM, recvElt);
    parameters.put(SignalExecUtil.MESSAGE_PARAM, pendingMessage.getMessage().duplicate());
    if (pendingMessage.getMessageCarrierId() != null) {
      final MessageCarrier messageCarrier =
        MessageCarrierRepository.removePendingMessageCarrier(pendingMessage.getMessageCarrierId());
      if (messageCarrier == null) {
        throw new OrchestraRuntimeException("Message carrier " + pendingMessage.getMessageCarrierId() + " not found !");
      }
      parameters.put(SignalExecUtil.MESSAGE_CARRIER_PARAM, messageCarrier);
    }
    // Signal execution
    exec.signal(parameters);
  }
}
