/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.handlers.impl;

import org.ow2.orchestra.facade.data.runtime.ProcessInstanceData;
import org.ow2.orchestra.facade.runtime.full.ProcessFullInstance;
import org.ow2.orchestra.services.Archiver;
import org.ow2.orchestra.services.Querier;
import org.ow2.orchestra.services.Recorder;
import org.ow2.orchestra.services.handlers.FinishedInstanceHandler;
import org.ow2.orchestra.util.ArchiveTool;
import org.ow2.orchestra.util.EnvTool;

/**
 * {@link FinishedInstanceHandler} that archive the records when the instance is
 * finished (removes records from the {@link Journal} and gives them to the
 * {@link Archiver}).
 *
 * @author Alfonso Fuca
 *
 */
public class ArchiveFinishedInstanceHandler implements FinishedInstanceHandler {

  public void handleFinishedInstance(final ProcessInstanceData processInstanceData) {

    final Archiver archiver = EnvTool.getArchiver();
    final Querier querier = EnvTool.getJournalQueriers();
    final Recorder recorder = EnvTool.getRecorder();

    final ProcessFullInstance processFullInstance = querier.getProcessInstance(processInstanceData.getUUID());

    ArchiveTool.atomicArchive(processFullInstance, recorder, archiver);

  }

}
