/**
 * Copyright (C) 2006  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.impl;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.definition.BpelProcess;
import org.ow2.orchestra.definition.activity.Receive;
import org.ow2.orchestra.definition.element.OnMessage;
import org.ow2.orchestra.facade.runtime.ActivityState;
import org.ow2.orchestra.facade.uuid.ProcessDefinitionUUID;
import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.persistence.RuntimeDbSession;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.PendingMessage;
import org.ow2.orchestra.services.ReceivingElement;
import org.ow2.orchestra.services.itf.Repository;

public class DbRepository implements Repository {

  /*
   * PROCESS REPOSITORY
   */
  public BpelProcess getProcess(final QName processQName) {
    return this.getBpelPersistenceService().findBpelProcess(processQName.getNamespaceURI(), processQName.getLocalPart(), false);
  }

  public List<BpelProcess> getProcesses() {
    return this.getBpelPersistenceService().getBpelProcesses();
  }

  public BpelProcess removeProcess(final QName processQName) {
    final BpelProcess process = this.getBpelPersistenceService().findBpelProcess(processQName.getNamespaceURI(),
      processQName.getLocalPart(), false);
    if (process != null) {
      this.getBpelPersistenceService().delete(process);
    }
    return process;
  }

  public BpelProcess removeProcess(final ProcessDefinitionUUID pduuid) {
    final BpelProcess process = this.getBpelPersistenceService().getBpelProcess(pduuid);
    if (process != null) {
      this.getBpelPersistenceService().delete(process);
    }
    return process;
  }

  public void storeProcess(final BpelProcess process) {
    this.getBpelPersistenceService().save(process);
  }

  public BpelProcess lockProcess(final QName processQName) {
    return this.getBpelPersistenceService().findBpelProcess(processQName.getNamespaceURI(), processQName.getLocalPart(), true);
  }

  /*
   * INSTANCE REPOSITORY
   */
  public BpelExecution getInstance(final ProcessInstanceUUID processInstanceUUID) {
    return this.getBpelPersistenceService().getBpelInstance(processInstanceUUID);
  }

  public List<BpelExecution> getInstances() {
    return this.getBpelPersistenceService().getBpelInstances();
  }

  public List<BpelExecution> getInstances(final QName processQName) {
    return this.getBpelPersistenceService().getBpelInstances(processQName);
  }

  public void removeInstance(final BpelExecution instance) {
    this.removeInstance(instance, false);
  }

  public void removeInstance(final BpelExecution instance, final boolean force) {
    if (force) {
      if (instance != null
          && (instance.getProcessInstanceState().equals(ActivityState.RUNNING)
              || instance.getProcessInstanceState().equals(ActivityState.SUSPENDED))
      ) {
        instance.exit();
      }
    }
    if (instance != null) {
      this.getBpelPersistenceService().delete(instance.getProcessInstance());
    }
  }

  public void storeInstance(final BpelExecution instance) {
    this.getBpelPersistenceService().save(instance.getProcessInstance());
  }

  public BpelExecution getUpdatedExecution(final BpelExecution execution) {
    return this.getBpelPersistenceService().get(BpelExecution.class, execution.getDbid());
  }

  public PendingMessage getUpdatedPendingMessage(final PendingMessage pendingMessage) {
    return this.getBpelPersistenceService().get(PendingMessage.class, pendingMessage.getDbid());
  }

  public ReceivingElement getUpdatedReceivingElement(final ReceivingElement receivingElement) {
    if (receivingElement instanceof Receive) {
      return this.getBpelPersistenceService().get(Receive.class, ((Receive) receivingElement).getDbid());
    } else if (receivingElement instanceof OnMessage) {
      return this.getBpelPersistenceService().get(OnMessage.class, ((OnMessage) receivingElement).getDbid());
    }
    return receivingElement;
  }

  /*
   * COMMON METHODS
   */
  private RuntimeDbSession getBpelPersistenceService() {
    return Environment.getCurrent().get(RuntimeDbSession.class);
  }

}
