package org.ow2.orchestra.services.itf;

import java.util.List;

import javax.xml.namespace.QName;

import org.ow2.orchestra.facade.uuid.ProcessInstanceUUID;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.PendingMessage;
import org.ow2.orchestra.services.ReceivingElement;

/**
 * component that stores the process instances.
 *
 * @author Guillaume Porcher
 * @author Charles Souillard
 */
public interface InstanceRepository {

  /**
   *
   * @param process
   *            bpelExecution to store
   */
  void storeInstance(BpelExecution instance);

  void removeInstance(BpelExecution instance);

  void removeInstance(BpelExecution instance, boolean force);
  /**
   * Returns one instance.
   */
  BpelExecution getInstance(ProcessInstanceUUID processInstanceUUID);
  /**
   * Returns the list of instances.
   */
  List<BpelExecution> getInstances();

  /**
   * Returns the list of instances for the given BPEL process.
   */
  List<BpelExecution> getInstances(QName processQName);

  /**
   * Returns an up-to-date execution
   *
   * @param old
   *            execution
   */
  BpelExecution getUpdatedExecution(BpelExecution execution);
  ReceivingElement getUpdatedReceivingElement(ReceivingElement receivingElement);
  PendingMessage getUpdatedPendingMessage(PendingMessage pendingMessage);
}
