package org.ow2.orchestra.services.itf;

import java.util.Set;

import javax.wsdl.Definition;

import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.var.MessageVariable;
import org.w3c.dom.Element;

/**
 * used by the engine during bpel process execution whenever a remote service
 * needs to be invoked.
 *
 * @author Tom Baeyens
 * @author Charles Souillard
 * @author Guillaume Porcher
 */
public interface Invoker {

  /** key to use to find an object in the wiring context. */
  String DEFAULT_KEY = "serviceInvoker";

  /**
   * perform an invoke using the given parameters.
   *
   * @param operationKey
   *            the operation of the invoke activity
   * @param partnerEndPointReference
   *            the endpoint reference of the service to invoke
   * @param requestMessage
   *            the message used in invoke request
   * @param wsdlDefinitions
   *            the wsdl info available in the bpel process.
   * @return the message received in response of this call (if there is one...)
   */
  MessageVariable invoke(
      OperationKey operationKey,
      Element partnerEndPointReference,
      MessageVariable requestMessage,
      Set<Definition> wsdlDefinitions);
}
