/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.job;

import java.util.HashSet;

import javax.wsdl.Definition;

import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.BpelMessageImpl;
import org.ow2.orchestra.services.InvokeExecutor;
import org.ow2.orchestra.services.OperationKey;
import org.ow2.orchestra.var.MessageVariable;
import org.w3c.dom.Element;


/**
 * @author Guillaume Porcher
 *
 */
public class ExecuteInvokeJob extends BpelMessageImpl<Void> {

  private boolean isExecuting = false;
  private MessageVariable requestMessage;
  private Element partnerEndPointReference;
  private OperationKey operationKey;

  /* needed for hibernate */
  protected ExecuteInvokeJob() {}

  public ExecuteInvokeJob(final MessageVariable requestMessage, final Element partnerEndPointReference, final OperationKey operationKey) {
    this.requestMessage = requestMessage != null ? requestMessage.duplicate() : null;
    this.partnerEndPointReference = partnerEndPointReference;
    this.operationKey = operationKey;
  }

  public Void execute(final Environment environment) throws Exception {
    // sets retries to 0 so that JobExecutor does not execute this job again
    this.setRetries(0);
    // set to true to specify job has not failed
    this.isExecuting = true;

    final BpelExecution bpelExecution = (BpelExecution) this.getExecution();
    // launch partner service invocation
    InvokeExecutor.executeInvoke(
        environment,
        bpelExecution,
        new OperationKey(this.operationKey),
        this.partnerEndPointReference,
        this.requestMessage.duplicate(),
        new HashSet<Definition>(bpelExecution.getWsdlInfos().getWsdlDefinitions()));

    // don't delete this job,
    // this will be deleted by InvokeExecutor when service invocation is ended.
    // This allows to recover on restart.
    return null;
  }


  public boolean isExecuting() {
    return this.isExecuting;
  }

}
