/**
 * Copyright (C) 2009  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.services.job;

import org.ow2.orchestra.definition.activity.AbstractBpelActivity;
import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.pvm.env.Environment;
import org.ow2.orchestra.pvm.internal.job.MessageImpl;
import org.ow2.orchestra.pvm.internal.jobexecutor.JobDbSession;
import org.ow2.orchestra.pvm.internal.model.ExecutionImpl.Propagation;
import org.ow2.orchestra.pvm.session.MessageSession;
import org.ow2.orchestra.runtime.BpelExecution;
import org.ow2.orchestra.services.AsyncFindExecToSignal;


/**
 * @author Guillaume Porcher
 *
 */
public class ExecuteSourcesMessage extends MessageImpl<Object> {

  protected ExecuteSourcesMessage() { }

  public ExecuteSourcesMessage(final BpelExecution bpelExecution) {
    this.setExecution(bpelExecution);
  }

  public Object execute(final Environment environment) throws Exception {
    final BpelExecution bpelExecution = (BpelExecution) this.getExecution();
    final AbstractBpelActivity activity = (AbstractBpelActivity) bpelExecution.getNode().getBehaviour();

    bpelExecution.unlock();
    bpelExecution.setPropagation(Propagation.UNSPECIFIED);
    activity.executeSources(bpelExecution);
    if (bpelExecution.getPropagation().equals(Propagation.UNSPECIFIED)) {
      bpelExecution.proceed();
    }

    // continue the execution
    if (bpelExecution.getProcessInstance() != null && !bpelExecution.getProcessInstance().isEnded()) {
      environment.get(MessageSession.class).send(
          new AsyncFindExecToSignal(bpelExecution.getProcessInstance(), bpelExecution.getProcessDefinition().getQName()));
    }
    // remove job
    final JobDbSession dbSession = environment.get(JobDbSession.class);
    if (dbSession == null) {
      throw new OrchestraRuntimeException("no " + JobDbSession.class.getName() + " in environment");
    }
    dbSession.delete(this);
    return true;
  }

}
