/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.util;

import org.ow2.orchestra.facade.exception.OrchestraRuntimeException;
import org.ow2.orchestra.services.Archivable;
import org.ow2.orchestra.services.Archiver;
import org.ow2.orchestra.services.Recorder;

/**
 *
 * @author Alfonso Fuca
 */
public abstract class ArchiveTool {

  private ArchiveTool() {
  }

  /**
   *
   * @author Alfonso Fuca
   */
  public static class AtomicArchiveException extends OrchestraRuntimeException {

    public AtomicArchiveException(final String msg, final Throwable cause) {
      super(msg, cause);
    }
  }

  /**
   * Perform a all-or-nothing archive.
   *
   * This method does the following: it first tries to archive the given
   * Archivable into the specified {@link Archiver}. If it succeeds and only in
   * this case, it tries to remove the given record from the specified
   *
   * @param record a record
   * @throws AtomicArchiveException if archiving failed.
   */
  public static void atomicArchive(final Archivable archivable, final Recorder recorder, final Archiver archiver) {
    try {
      archiver.archive(archivable);
    } catch (final Exception e) {
      throw new AtomicArchiveException("Can't archive archivable: " + archivable + " into archiver: " + archiver, e);
    }
    try {
      recorder.remove(archivable);
    } catch (final Exception e) {
      throw new AtomicArchiveException("Can't remove archivable: " + archivable + " from history: " + recorder, e);
    }
  }

}
