/**
 * Bull SAS / OW2 Orchestra
 * Copyright (C) 2007 Bull S.A.S, and individual contributors as indicated
 * by the @authors tag.
 *
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/
package org.ow2.orchestra.util;

import java.util.HashMap;
import java.util.Map;

/**
 * List of all the Static Analysis Faults
 *
 * @author Goulven Le Jeune
 *
 */
public final class StaticAnalysisFault {

  private StaticAnalysisFault() {
  }

  // Map of the code and description of the fault
  private static Map<String, String> saf = null;

  /**
   * Get the Static Analysis Fault description from its code.
   *
   * @param code
   *          : Code of the Static Analysis Fault
   * @return String with the description of the Fault
   */
  public static String getFault(final String code) {
    if (StaticAnalysisFault.saf == null) {
      StaticAnalysisFault.initiateSAF();
    }
    return StaticAnalysisFault.saf.get(code);
  }

  /**
   * Build the Fault Message based on its code and cause.
   *
   * @param code
   *          : Code of the Static Analysis Fault
   * @param cause
   *          : Explanation of the fault
   * @return a string with the definition of the Static Analysis fault and the
   *         explanation
   */
  public static String getMessage(final String code, final String cause) {
    String message = StaticAnalysisFault.getFault(code);
    message += System.getProperty("line.separator");
    message += cause;
    return message;
  }

  private static void initiateSAF() {
    StaticAnalysisFault.saf = new HashMap<String, String>();
    StaticAnalysisFault.saf.put("SA00001", "SA00001: A WS-BPEL processor MUST reject a WS-BPEL "
      + "that refers to solicit-response or notification operations portTypes.");
    StaticAnalysisFault.saf.put("SA00002", "SA00002: A WS-BPEL processor MUST reject any WSDL "
      + "portType definition that includes overloaded operation names.");
    StaticAnalysisFault.saf.put("SA00003", "");
    StaticAnalysisFault.saf.put("SA00004", "");
    StaticAnalysisFault.saf.put("SA00005", "SA00005: If the portType attribute is included for "
      + "readability, in a <receive>, <reply>, <invoke>, <onEvent> or "
      + "<onMessage> element, the value of the portType attribute MUST "
      + "match the portType value implied by the combination of the "
      + "specified partnerLink and the role implicitly specified by the activity.");
    StaticAnalysisFault.saf.put("SA00006", "SA00006: The <rethrow> activity MUST only be used "
      + "within a faultHandler (i.e. <catch> and <catchAll> elements).");
    StaticAnalysisFault.saf.put("SA00007", "SA00007: The <compensateScope> activity MUST only "
      + "be used from within a faultHandler, another compensationHandler, or terminationHandler.");
    StaticAnalysisFault.saf.put("SA00008", "SA00008: The <compensate> activity MUST only be used "
      + "from within a faultHandler, another compensationHandler, or a terminationHandler.");
    StaticAnalysisFault.saf.put("SA00009", "");
    StaticAnalysisFault.saf.put("SA00010", "");
    StaticAnalysisFault.saf.put("SA00011", "");
    StaticAnalysisFault.saf.put("SA00012", "");
    StaticAnalysisFault.saf.put("SA00013", "");
    StaticAnalysisFault.saf.put("SA00014", "");
    StaticAnalysisFault.saf.put("SA00015", "SA00015: To be instantiated, an executable business "
      + "process MUST contain at least one <receive> or <pick> activity annotated with a createInstance=yes attribute.");
    StaticAnalysisFault.saf.put("SA00016", "SA00016: A partnerLink MUST specify the myRole or the partnerRole, or both.");
    StaticAnalysisFault.saf.put("SA00017", "SA00017: The initializePartnerRole attribute MUST NOT "
      + "be used on a partnerLink that does not have a partner role.");
    StaticAnalysisFault.saf.put("SA00018", "SA00018: The name of a partnerLink MUST be unique "
      + "among the names of all partnerLinks defined within the same immediately enclosing scope.");
    StaticAnalysisFault.saf.put("SA00019", "");
    StaticAnalysisFault.saf.put("SA00020", "");
    StaticAnalysisFault.saf.put("SA00021", "");
    StaticAnalysisFault.saf.put("SA00022", "");
    StaticAnalysisFault.saf.put("SA00023", "SA00023: The name of a variable MUST be unique among "
      + "the names of all variables defined within the same immediately enclosing scope.");
    StaticAnalysisFault.saf.put("SA00024", "SA00024: Variable names are BPELVariableNames, that is,"
      + "NCNames (as defined in XML Schema specification) but in addition they MUST NOT contain the . character.");
    StaticAnalysisFault.saf.put("SA00025", "SA00025: The messageType, type or element attributes "
      + "are used to specify the type of a variable. Exactly one of these attributes MUST be used.");
    StaticAnalysisFault.saf.put("SA00026", "");
    StaticAnalysisFault.saf.put("SA00027", "");
    StaticAnalysisFault.saf.put("SA00028", "");
    StaticAnalysisFault.saf.put("SA00029", "");
    StaticAnalysisFault.saf.put("SA00030", "");
    StaticAnalysisFault.saf.put("SA00031", "");
    StaticAnalysisFault.saf.put("SA00032", "");
    StaticAnalysisFault.saf.put("SA00033", "");
    StaticAnalysisFault.saf.put("SA00034", "");
    StaticAnalysisFault.saf.put("SA00035", "");
    StaticAnalysisFault.saf.put("SA00036", "");
    StaticAnalysisFault.saf.put("SA00037", "");
    StaticAnalysisFault.saf.put("SA00038", "");
    StaticAnalysisFault.saf.put("SA00039", "");
    StaticAnalysisFault.saf.put("SA00040", "");
    StaticAnalysisFault.saf.put("SA00041", "");
    StaticAnalysisFault.saf.put("SA00042", "");
    StaticAnalysisFault.saf.put("SA00043", "");
    StaticAnalysisFault.saf.put("SA00044", "SA00044: The name of a <correlationSet> MUST be unique"
      + "among the names of all <correlationSet> defined within the same immediately enclosing scope.");
    StaticAnalysisFault.saf.put("SA00045", "");
    StaticAnalysisFault.saf.put("SA00046", "");
    StaticAnalysisFault.saf.put("SA00047", "");
    StaticAnalysisFault.saf.put("SA00048", "");
    StaticAnalysisFault.saf.put("SA00049", "");
    StaticAnalysisFault.saf.put("SA00050", "");
    StaticAnalysisFault.saf.put("SA00051", "SA00051: The inputVariable attribute MUST NOT be used on an "
      + "Invoke activity that contains <toPart> elements.");
    StaticAnalysisFault.saf.put("SA00052", "SA00052: The outputVariable attribute MUST NOT be used on an "
      + "<invoke> activity that contains a <fromPart> element.");
    StaticAnalysisFault.saf.put("SA00053", "");
    StaticAnalysisFault.saf.put("SA00054", "");
    StaticAnalysisFault.saf.put("SA00055", "SA00055: For <receive>, if <fromPart> elements are used on a "
      + "<receive> activity then the variable attribute MUST NOT be used on the same activity.");
    StaticAnalysisFault.saf.put("SA00056", "SA00056: A \"start activity\" is a <receive> or <pick> activity that is"
      + " annotated with a createInstance=\"yes\" attribute."
      + " Activities other than the following: start activities, <scope>, <flow> "
      + "and <sequence> MUST NOT be performed prior to or simultaneously with start activities.");
    StaticAnalysisFault.saf.put("SA00057", "SA00057: If a process has multiple start activities with correlation sets "
      + "then all such activities MUST share at least one common "
      + "correlationSet and all common correlationSets defined on all "
      + "the activities MUST have the value of the initiate attribute "
      + "be set to \"join\".");
    StaticAnalysisFault.saf.put("SA00058", "");
    StaticAnalysisFault.saf.put("SA00059", "SA00059: For <reply>, if <toPart> elements are used on a "
      + "<reply> activity then the variable attribute MUST NOT be used on the same activity.");
    StaticAnalysisFault.saf.put("SA00060", "");
    StaticAnalysisFault.saf.put("SA00061", "");
    StaticAnalysisFault.saf.put("SA00062", "");
    StaticAnalysisFault.saf.put("SA00063", "");
    StaticAnalysisFault.saf.put("SA00064", "SA00064: For <flow>, a declared link s name MUST be unique "
      + "among all <link> names defined within the same immediately enclosing <flow>.");
    StaticAnalysisFault.saf.put("SA00065", "SA00065: The value of the linkName attribute of <source> or "
      + "<target> MUST be the name of a <link> declared in an enclosing <flow> activity.");
    StaticAnalysisFault.saf.put("SA00066", "SA00066: Every link declared within a <flow> activity MUST "
      + "have exactly one activity within the <flow> as its source and exactly "
      + "one activity within the <flow> as its target.");
    StaticAnalysisFault.saf.put("SA00067", "");
    StaticAnalysisFault.saf.put("SA00068", "SA00068: An activity MAY declare itself to be the source of "
      + "one or more links by including one or more <source> elements. Each <source> element MUST use a distinct link name.");
    StaticAnalysisFault.saf.put("SA00069", "SA00069: An activity MAY declare itself to be the target of "
      + "one or more links by including one or more <target> elements. Each "
      + "<target> element associated with a given activity MUST use a link name "
      + "distinct from all other <target> elements at that activity.");
    StaticAnalysisFault.saf.put("SA00070", "");
    StaticAnalysisFault.saf.put("SA00071", "");
    StaticAnalysisFault.saf.put("SA00072", "");
    StaticAnalysisFault.saf.put("SA00073", "");
    StaticAnalysisFault.saf.put("SA00074", "SA00074: The expressions in <startCounterValue> and "
      + "<finalCounterValue> MUST return a TII (meaning they contain at least one character) "
      + "that can be validated as a xsd:unsignedInt.");
    StaticAnalysisFault.saf.put("SA00075", "SA00075: For the <forEach> activity, <branches> MUST BE an "
      + "integer value expression");
    StaticAnalysisFault.saf.put("SA00076", "SA00076: For <forEach> the enclosed scope MUST NOT declare "
      + "a variable with the same name as specified in the counterName attribute of <forEach>.");
    StaticAnalysisFault.saf.put("SA00077", "");
    StaticAnalysisFault.saf.put("SA00078", "");
    StaticAnalysisFault.saf.put("SA00079", "");
    StaticAnalysisFault.saf.put("SA00080", "SA00080: There must be at least one <catch> or <catchAll> in a <faultHandlers> list");
    StaticAnalysisFault.saf.put("SA00081",
      "SA00081: The faultVariable attribute must always have a corresponding faultMessageType"
        + "or faultElement attribute (only one of them, not both). The faultMessageType and faultElement "
        + "attributes must not be used unless accompanied by a faultVariable.");
    StaticAnalysisFault.saf.put("SA00082", "");
    StaticAnalysisFault.saf.put("SA00083", "SA00083: An event handler must contain at least one <onEvent> or <onAlarm> element.");
    StaticAnalysisFault.saf.put("SA00084", "SA00084: The partnerLink reference of <onEvent> MUST resolve to a partner link"
        + " declared in the process in the following order: the associated scope first and then the ancestor scopes.");
    StaticAnalysisFault.saf.put("SA00085", "SA00085: The syntax and semantics of the <fromPart> elements as"
        + " used on the <onEvent> element are the same as specified for"
        + " the receive activity. This includes the restriction that if"
        + " <fromPart> elements are used on an onEvent element then"
        + " the variable, element and messageType attributes MUST"
        + " NOT be used on the same element.");
    StaticAnalysisFault.saf.put("SA00086",
      "SA00086: For <onEvent>, variables referenced by the variable attribute of "
        + "<fromPart> elements or the variable attribute of "
        + "an <onEvent> element are implicitly declared in the associated scope of the event handler. Variables of the same "
        + "names MUST NOT be explicitly declared in the associated scope.");
    StaticAnalysisFault.saf.put("SA00087", "SA00087: For <onEvent>, the type of the variable (as specified by the "
        + " messageType attribute) MUST be the same as the type of the"
        + " input message defined by operation referenced by the"
        + " operation attribute. Optionally the messageType attribute may"
        + " be omitted and instead the element attribute substituted if the"
        + " message to be received has a single part and that part is"
        + " defined with an element type. That element type MUST be an"
        + " exact match of the element type referenced by the element"
        + " attribute.");
    StaticAnalysisFault.saf.put("SA00088", "SA00088: For <onEvent>, the resolution order of the correlation"
        + " set(s) referenced by <correlation> MUST be first the"
        + " associated scope and then the ancestor scopes.");
    StaticAnalysisFault.saf.put("SA00089", "SA00089: For <onEvent>, when the messageExchange attribute is"
        + " explicitly specified, the resolution order of the message"
        + " exchange referenced by messageExchange attribute MUST"
        + " be first the associated scope and then the ancestor scopes.");
    StaticAnalysisFault.saf.put("SA00090",
      "SA00090: If the variable attribute is used in the <onEvent> element, either the messageType "
        + "or the element attribute MUST be provided in the <onEvent> element.");
    StaticAnalysisFault.saf.put("SA00091", "");
    StaticAnalysisFault.saf.put("SA00092", "SA00092: Within a scope, the name of all named immediately "
      + "enclosed scopes MUST be unique.");
    StaticAnalysisFault.saf.put("SA00093", "SA00093: the <faultHandlers> element must not contain identical <catch> constructs. "
      + "<catch> constructs are considered identical if their faultName, faultElement and faultMessageType "
      + "attributes are identical. If an attribute is missing, it is identical to another missing attribute.");
    StaticAnalysisFault.saf.put("SA00094", "");
    StaticAnalysisFault.saf.put("SA00095", "");
    StaticAnalysisFault.saf.put("SA10001",
        "SA10001: Two start receiving elements should not have the same partnerLink, portType and operation.");
  }
}
